#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: astoControlProc.tcl,v 1.1 2004/09/06 14:47:13 cguirao Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# fsogni  13/01/98  created
#
#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


#############################################################################
#
#			Widge initialization Functions
#
#############################################################################

proc astoInitX {} {
    global gvar cv astoCV env

    
    set gvar(HOST) $env(HOST)
    if {[catch {exec which dfsmsg} msg]} { 
	puts "Cannot find the command \"dfsmsg\"" 
	exit 1
    }

    Log "Initialize astoControl" Info
    #Load Sybase-tcl libraries
    #load /usr/server/arc/sybtcl/lib/libSybtcl.so
    Debug  "Load sybase" 3
    if {[catch {package require Sybtcl}]} {
	Info "I cannot load sybtcl"
	Log "I cannot load sybtcl" Error
	exit 1
    }

    #Keep the original cv variable and the current directory.
    set astoCV $cv
    set gvar(localdir) [pwd]
    
    # Capture the window manager delete window action and override the
    # window manager's default action.
    set _top_level [winfo toplevel $astoCV]
    wm protocol $_top_level WM_DELETE_WINDOW "bell"

    #Trap signals
    signal trap {HUP INT QUIT TERM} {
	Info "Please use Quit to exit from the application."
    }
    signal trap {CONT} "wm deiconify ."
    
    #Define flags
    astoControlDefineFlags
    
    #Define PATHs
    astoControlDefineEnv
    
    Debug "Set defaults for astoControl" 3
    #Load configuration for astoControl
    astoControlDefaults

    Debug "Set defaults for writeControl" 3
    #Load configuration for writeControl
    writeControlDefaults

    #Set some bindings
    Debug "Set some bindings" 3
    bind $astoCV.astoRequestWindow.f1.uifFancyListBox <Button-1> {
	after 10 clearSelection $astoCV.astoRequestWindow request }
    bind $astoCV.astoVolumeWindow.f1.uifFancyListBox <Button-1> {
	after 10 clearSelection $astoCV.astoVolumeWindow volume }
    
    #Build the listbox headers
    set formatLine "%-6s %-5s %-9s %-8s %-7s %-12s"
    set formatHeader "Id SEG \"Disk(MB)\" \"TBI(MB)\" %Queue Action" 
    set gvar(astoRequestHdr) [eval format [concat \"$formatLine\" $formatHeader]]
    set formatLine "%-4s %-15s %-4s %-6s %-12s %-15s"
    set formatHeader "Id  media_name SEG  Copies State Action" 
    set gvar(astoVolumeHdr) [eval format [concat \"$formatLine\" $formatHeader]]
    
    
    #Prepare the menulist for the dhsSubscribe panel
    foreach host $gvar(host) {
	lappend gvar(hostlist) [lindex $host 0]
    }

    Debug "Start loops and status" 3
    #Start to loops, one for astoStatus and the second,
    #which checks if one of the queues has accumulated more
    #than 100% of a Volume.
    after 15000 statusLoop astoCheckLoop 30000
    after 15000 autoLoop astoCheckAuto 30000
    
    #Move to DHS_LOG
    cd $env(DHS_LOG)
    Log "AstoControl started" Info
} 


#############################################################################
#
#			Globals Initialization
#
#############################################################################

#Define some constants and flags
proc astoControlDefineFlags {} {
    global gvar 
    
    #Set some flags
    set gvar(ingestFlag) 0
    set gvar(queueFlag)  0
    set gvar(manualMode) 0
    set gvar(toggleIQ)   0
    set gvar(astoStatusFlag) 0
    set gvar(requestsStatusFlag) 0
    set gvar(volumesStatusFlag)  0
    set gvar(manualIngest) 0
    set gvar(autoIngest) 0
    set gvar(autoIngestFiles) 0
    set gvar(manualQueue) 0
    set gvar(autoQueue) 0
    set gvar(autoQueueFiles) 0
    set gvar(manualPrep) 0
    set gvar(autoPrep) 0
    set gvar(autoPrepVolumes) 0
    set gvar(autoWrite) 0
    set gvar(autoWriteVolumes) 0
    set gvar(autoCleanup) 0
    set gvar(autoCleanupVolumes) 0
    set gvar(getFreeDeviceFlag) ""
    set gvar(manualWriteBusy) 0
    set gvar(debugError) 0
 
    set sybHandle()       ""
    
    #set bgexec flags
    set gvar(ingestOutput) ""
    set gvar(ingestErrors) ""
    set gvar(ingestIndex)  0
    set gvar(queueOutput)  ""
    set gvar(queueErrors)  ""
    set gvar(queueIndex)   0
    set gvar(prepOutput)   ""
    set gvar(prepErrors)   ""
    set gvar(prepIndex)    0
    set gvar(writeOutput)  ""
    set gvar(writeErrors)  ""
    set gvar(writeIndex)   0
    set gvar(cleanupOutput) ""
    set gvar(cleanupErrors) ""
    set gvar(cleanupIndex) 0
    set gvar(exComOutput) ""
    set gvar(exComErrors) ""
    set gvar(exComIndex)  0
}


#Define some variables and flags
proc astoControlDefineEnv {} {
    global gvar env astoCV

    
    set errorMessage ""
    set errorFlag    0
    BusyWin $astoCV {
	if {![info exists env(DHS_LOG)] || ![file isdir $env(DHS_LOG)]} {
	    append errorMessage  "DHS_LOG is not properly configured\n" 
	    set errorFlag 1
	}
	if {![info exists env(HOST)]} {
	    append errorMessage "Environment variable HOST is not set\n"
	    set errorFlag 1
	} else {
	    set gvar(HOST) $env(HOST)
	}
	if {![info exists env(ASTO_CONFIG_DIR)] || ![file isdir $env(ASTO_CONFIG_DIR)]} {
	    append errorMessage "ASTO_CONFIG_DIR is not properly configured\n" 
	    set errorFlag 1
	}
	if {![info exists env(ASTO_STAGE_ROOT)] || ![file isdir $env(ASTO_STAGE_ROOT)]} {
	    append errorMessage "ASTO_STAGE_ROOT is not properly configured\n" 
	    set errorFlag 1
	}
	if {[catch {file stat $env(HOME)/.dbrc dbrcMode}]} {
	    append errorMessage "I cannot find $env(HOME)/.dbrc\nCheck the HOME variable or create a valid .dbrc file\n" 
	    set errorFlag 1
	}
	if {$dbrcMode(mode) != "33152"} {
	    append errorMessage "$env(HOME)/.dbrc doesn't have the right permissions (600)\n"
	    set errorFlag 1
	}
	
	if {$errorFlag} {
	    Info "The environment is not properly configured.\nCheck the standard output for more details"
	    Log "Error in the environment configuration." Error
	    Log "$errorMessage" Error
		puts "$errorMessage"
	    exit 1
	}
	    

	::.astoControl.bot.sh displayText "Opening Am configuration file..."
	update idletasks
	
        #Define the Asto configuration file
	if {[file readable $env(ASTO_CONFIG_DIR)/libAm.config]} {
	    set gvar(config_file) $env(ASTO_CONFIG_DIR)/libAm.config
	} else {
	    Log "Asto configuration file libAm.config not found.\nCheck the env variables ASTO_CONFIG_DIR or DEFAULT_CONFIG_DIR" Error
	    Info  "Asto configuration file libAm.config not found.\nCheck the env variables ASTO_CONFIG_DIR or DEFAULT_CONFIG_DIR"
	    exit 1
	}
	
	
	#Define the 3 ASTO  directories
	if {[ catch {set line [ eval exec cat $gvar(config_file) |  grep dataStage | grep -v \#]}]} { 
	    Log "Cannot open $gvar(config_file)" Error
	    Info "Cannot open $gvar(config_file)"
	    exit 1
	}
	set gvar(userDataStage)  [lindex $line 1]
	set gvar(archDataStage)  [lindex $line 2]
	set gvar(mediaDataStage) [lindex $line 3]
	Debug " User Stage = $gvar(userDataStage) " 1
	Debug " Arch Stage = $gvar(archDataStage)"  1
	Debug " Media Stage = $gvar(mediaDataStage)" 1

	if {![file isdir $gvar(userDataStage)]} {
	    append errorMessage "User Data Stage: $gvar(userDataStage) is not a valid directory\n" 
	    set errorFlag 1
	}
	if {![file isdir $gvar(archDataStage)]} {
	    append errorMessage "Arch Data Stage: $gvar(archDataStage) is not a valid directory\n" 
	    set errorFlag 1
	}
	if {![file isdir $gvar(mediaDataStage)]} {
	    append errorMessage "Media Data Stage: $gvar(mediaDataStage) is not a valid directory\n" 
	    set errorFlag 1
	}
	
	if {$errorFlag} {
	    Log "Error in the libAm library:" Error
	    Log "$errorMessage" Error
	    Info "Error in the libAm library.\Check the standard output for
more details"
		puts "$errorMessage"
	    exit 1
	}

	#Define the Asto Tables
	if {[ catch {set line [ eval exec cat $gvar(config_file) | grep amDb | grep -v \#]}]} {
	    Log "Cannot open $gvar(config_file)" Error
	    Info "Cannot open $gvar(config_file)"
	    exit 1
	}
	
	set gvar(db_server)      [lindex $line 1]
	set gvar(database)       [lindex $line 2]
	set gvar(media_table)    [lindex $line 3]
	set gvar(files_table)    [lindex $line 4]
	set gvar(units_table)    [lindex $line 5]
	set gvar(stage_table)    [lindex $line 6]
	set gvar(status_table)   [lindex $line 7]
	set gvar(actions_table)  [lindex $line 8]
	
	Debug " db server = $gvar(db_server) "   1  
	Debug " database = $gvar(database)"      1
	Debug " media table = $gvar(media_table) "     1   
	Debug " files table = $gvar(files_table)"      1 
	Debug " units table = $gvar(units_table)"      1  
	Debug " stage table = $gvar(stage_table) "     1
	Debug " status table = $gvar(status_table) "   1
	Debug " actions table = $gvar(actions_table) " 1
	
	#Define device tables
	if {[ catch {set line [ eval exec cat $gvar(config_file) | grep deviceTables | grep -v \#]}]} { 
	    Log "Cannot open $gvar(config_file)" Error
	    Info "Cannot open $gvar(config_file)"
	    exit 1 
	}

	set gvar(deviceTable)     [lindex $line 1]
	set gvar(bookTable)       [lindex $line 2]
	set gvar(logicalTable)    [lindex $line 3]
	set gvar(mediaTypesTable) [lindex $line 4]
	Debug " device table = $gvar(deviceTable)" 1 
	Debug " booking table = $gvar(bookTable)" 1
	Debug " logical table = $gvar(logicalTable)" 1
	Debug " media types table = $gvar(mediaTypesTable)" 1
	

	#Get the sybase username and password
	if {[ catch {set sybaseLogin [exec dbrcGet $gvar(db_server) $gvar(database)]}]} { 
	    Log "Failed to connect to set $gvar(db_server).\nCheck ~/.dbrc or $gvar(config_file)" Error
	    Info "Failed to connect to set $gvar(db_server).\nCheck ~/.dbrc or $gvar(config_file)"
	    exit 1
	}
	
	#	if {[ catch {set sybaseLogin [exec dbrc_get $gvar(db_server) $gvar(database)]}]} { append errorMessage "Failed to connect to set $gvar(db_server).\nCheck ~/.dbrc or $gvar(config_file)" ; exit }
	
	set gvar(sybaseUsername) [lindex $sybaseLogin 0]
	set gvar(sybasePassword) [lindex $sybaseLogin 1]
	
	Debug " sybase username = $gvar(sybaseUsername)" 1
	#Debug " sybase password = $gvar(sybasePassword)" 1
	::.astoControl.bot.sh displayText ""
    }

    #Verify that the user asto has associate "asto" as database
    
    if {[catch {set sybHandle [sybconnect $gvar(sybaseUsername) $gvar(sybasePassword) $gvar(db_server) ]} handle]} {
	Log "Cannot connect to server, reason: $sybmsg(dberrstr)" Error
	message "Cannot connect to server, reason: $sybmsg(dberrstr)"
	Debug  "Cannot connect to server, reason: $sybmsg(dberrstr)" 1
	#catch {sybclose $sybHandle($pid)}
	catch {sybclose $sybHandle}
	exit 1
    } 

    if {[catch {set database [sybuse $sybHandle]}]} {
	Log "Cannot use database $gvar(database), reason: $sybmsg(dberrstr)" Error
	message "Cannot use database $gvar(database), reason: $sybmsg(dberrstr)"
	Debug   "Cannot use database: $gvar(database), reason: $sybmsg(dberrstr)" 1
	catch {sybclose $sybHandle}
	#catch {sybclose $sybHandle($pid)}
	exit 1
    } 

    if {$database != "asto"} {
	Log "User asto, doesn't connect automatically\nto the database asto.\nAsk your DBA to change\nthe user definition." Error
	message "User asto, doesn't connect automatically\nto the database asto.\nAsk your DBA to change\nthe user definition."
	Debug "User asto, doesn't connect automatically\nto the database asto.\nAsk your DBA to change\nthe user definition." 1
	exit 1
    }
    
    catch {sybclose $sybHandle}

    #Get the mediaType information from the database.
    set query "
	    select mt.media_type,
		    mt.capacity
	    from $gvar(database)..$gvar(mediaTypesTable) mt"
    Debug $query 1
    set mediaTypes [Sybsql $query]

    #Parse the results from the query to extract the media types
    if { $mediaTypes == "error" || $mediaTypes == "" } {
	::.astoControl.bot.sh displayText "Data Base error !"
	return }
    Debug "Results of media types query: $mediaTypes" 2

    foreach line $mediaTypes {
	set type 			[lindex $line 0]
	set size			[lindex $line 1]
	set gvar(mediaType,${type})	$size

	Debug "Media Type $type has size: $size" 1
    }

    ###############  AD Table  ###############
    ::.astoControl.bot.sh displayText "Opening AD configuration file..."
    update idletasks
    
    # Define the Asto configuration file
    if {[file readable $env(ASTO_CONFIG_DIR)/libAd.config]} {
	set gvar(ad_config_file) $env(ASTO_CONFIG_DIR)/libAd.config
    } else {
     	Info "Asto configuration file libAd.config not found.\nCheck the env var iables ASTO_CONFIG_DIR or DEFAULT_CONFIG_DIR"
	Log "Asto configuration file libAd.config not found.\nCheck the env var iables ASTO_CONFIG_DIR or DEFAULT_CONFIG_DIR" Error
	exit 1
    }

    #Find the tables used by the various media.           
    if {[catch {set lines [ eval exec cat $gvar(ad_config_file) | grep medium | grep -v \#]}]} { 
	Log "Cannot open $gvar(ad_config_file)" Error
	Info "Cannot open $gvar(ad_config_file)"
	exit 1
    }
    
    foreach line [split $lines "\n"] {
	set name [lindex $line 2]
	set gvar(${name},file) [lindex $line 3]
	set gvar(${name},vol) [lindex $line 4]
	Debug "Volume table for medium $name = $gvar(${name},vol)" 2
	Debug "File table for medium $name   = $gvar(${name},file)" 2
    }
    
    ::.astoControl.bot.sh displayText "Finished initialization"
    update idletasks
}


#############################################################################
#
#		       Checks for automatic execution.
#
#############################################################################

proc astoCheckLoop {} {
    global gvar
    
    if {$gvar(autoStatus)} { astoStatus } 
}

proc astoCheckAuto {} {
    global gvar

    if {$gvar(autoIngest)}  { autoIngest }
    if {$gvar(autoQueue)}   { autoQueue  }
    if {$gvar(autoWrite)}   { autoWrite  }
    if {$gvar(autoPrep)}    { autoPrep    }
    if {$gvar(autoCleanup)} { autoCleanup }
}


#############################################################################
#
#			Update Functions
#
#############################################################################


#Update the media request (queues) info
proc checkRequests {} {
    global gvar astoCV sybmsg
    
    
    if {$gvar(requestsStatusFlag)} { return }
    
    set gvar(requestsStatusFlag) 1
    ::.astoControl.bot.sh displayText "Checking status of queues..."
    #Check the maximum size of a Vol and convert it in %
    set queueBuffer     ""
    set gvar(queueList) ""
    
    #Maintain the selectiosn if any
    set queueIndex [$astoCV.astoRequestWindow getIndex] 

    #??? This query will prevent requests that have data on disk but not
    #??? ingested yet from being displayed.
    #Build the select to get the queues
    set query "
            select m.media_request_id,
                    m.media_request_name,
                    m.media_request_type,
                    m.media_type,
                    isnull(sum(f.filesize/1048576.0), 0)
            from $gvar(database)..$gvar(media_table) m,
                    $gvar(database)..$gvar(files_table) f
            where f.status = 'R'
                    and m.media_request_id *= f.media_request_id
                    and m.media_request_type = \"ARCH\""
    if {$gvar(parallelAsto)} {
	append query "\n and m.host = \"$gvar(HOST)\""
    }
    append query "\n group by m.media_request_id
                   order by m.media_request_id"
    
    #and f.media_unit_name = NULL

    Debug $query 1
    set queues [ Sybsql $query ]

    #Get the result and put it in a list
    if {$queues == "error" } {
	::.astoControl.bot.sh displayText "Data Base error !" 
	set gvar(requestsStatusFlag) 0
	return 1}
    Debug  "Result of query queues: $queues" 2
     
    #Extract the single fields from the query
    set gvar(requestAttribList) ""
    set indx 0

    #set formatLine "%-5s %-15s %5d %6d %11.1f  %-10s"
    set formatLine "%-5s %-5s %-9s %-8s %-7.1f %-10s"
    foreach lines $queues {
	set id            [lindex $lines 0]
	set segregation   [lindex $lines 1]
	set queueType     [lindex $lines 2]
	set mediaType     [lindex $lines 3]
	set filesIngested [lindex $lines 4]
	set statusDesc    [getQueueAction $id ]

	#Determine the status of the request.
	# set query "
	#             select a.action,
	#                     s.short_description
	#             from $gvar(database)..$gvar(actions_table) a,
	#                     $gvar(database)..$gvar(status_table) s
	#             where a.media_request_id = $id
	#                     and a.media_unit_name = NULL
	#                     and s.status = a.action
	#                     and s.table_name = 'media_actions'"

	# 	Debug $query 1
	# 	set statusInfo [Sybsql $query]
	# 	Debug "result: $statusInfo" 2
	
	if { $statusDesc == "-" } {
	    set attrib red
	} elseif  {  $statusDesc == "None" } {
	    set attrib blue 
	} else {
	    set attrib seagreen4
	}
	
	#Determine the percentage of date to be queued
	set percentage [getVolPercentage $id $segregation $mediaType]


	# Determine how much data is on disk
	if { $queueType == "ARCH" } { 
	    set dir $gvar(archDataStage)
	} elseif { $queueType == "USER" } {
	    set dir $gvar(userDataStage) 
	} else {
	    ::.astoControl.bot.sh displayText "Data base error !" 
	    continue
	}		

	if { ![file isdir $dir/$id.$segregation] } {
	    Log "Segregation directory $dir/$id.$segregation is not valid.\nPlease check.\nSevere error exit now." Error
	    Info "Segregation directory $dir/$id.$segregation is not valid.\nPlease check.\nSevere error exit now."  
	    exit 1 
	}

	Debug  "check disk occupation under $dir/$id.$segregation" 2
	set megabytes [diskSpace $dir/$id.$segregation]

	#If files are deleted from the disk we show a negative number.

	# if { $megabytes >= $filesIngested } {
	# 	    set toBeIngested [ expr int($megabytes - $filesIngested) ]
	# 	} else {
	# 	    set toBeIngested 0
	# 	}

	set toBeIngested [ expr {int($megabytes - $filesIngested)} ]
	
	Debug "id = $id" 1
	Debug "segregation = $segregation" 1
	Debug "queuetype = $queueType" 1
	Debug "status = $statusDesc" 1
	Debug "filesIngested = $filesIngested" 1
	Debug "percentage = $percentage" 1
	Debug "megabytes = $megabytes" 1
	Debug "toBeIngested = $toBeIngested" 1
	Debug "attrib = $attrib" 1

	if {!($megabytes == 0 && $percentage == 0)} {
	    set entry "\"$id\" \"$segregation\" \"$megabytes\" \"$toBeIngested\" \"$percentage\" \"$statusDesc\""
	    lappend queueBuffer [eval format \"${formatLine}\" $entry]
	    lappend gvar(queueList) $entry
	} 
	lappend gvar(requestAttribList) $attrib
	incr indx +1
    }
    
    #Update the Queue List Box
    $astoCV.astoRequestWindow deleteItem 0 end
    foreach line $queueBuffer {
	$astoCV.astoRequestWindow addItem end $line
    }
    
    dominoAttrib $gvar(requestAttribList) $astoCV.astoRequestWindow
    if {$queueIndex != ""} {$astoCV.astoRequestWindow setSelection $queueIndex}
    
    set queues [$astoCV.astoRequestWindow getList]
    set indx 0
    set gvar(requestsStatusFlag) 0
}


#Update the media information that is displayed.
proc checkVolumes {} {
    global gvar astoCV
    
    
    if {$gvar(volumesStatusFlag)} { return }
    
    set gvar(volumesStatusFlag) 1

    ::.astoControl.bot.sh displayText "Checking Status of Medias..."
    set volumeBuffer ""
    set gvar(volumeAttrib) ""
    
    set query "
		select m.media_request_id,
			m.media_request_name,
			u.media_unit_name,
			m.media_type,
			m.num_copies - u.copies_written,
			u.status,
			s.short_description
	        from $gvar(database)..$gvar(units_table) u,
			$gvar(database)..$gvar(media_table) m,
			$gvar(database)..$gvar(status_table) s
		where u.status != 'D'
			and m.media_request_id = u.media_request_id
			and u.status = s.status
			and s.table_name = '$gvar(units_table)'"


    if {$gvar(parallelAsto)} {
	append query "\n and m.host = \"$gvar(HOST)\"
                       and u.host = \'$gvar(HOST)\'"
    }
    
    append query "\n order by m.media_request_id, u.media_unit_name"
    
    Debug $query 1
    set Media [Sybsql $query]
    
    #Fill an array with the output of the query
    if { $Media == "error" } { 
	::.astoControl.bot.sh displayText "Database Error !"
	set gvar(volumesStatusFlag) 0
	return 1
    }
    Debug "Result of query Media: $Media" 2


    #set formatLine "%-3s %-15s %-10s %-12s %-6s %-12s %-15s"
    set formatLine "%-3s %-15s %-4s %-6s %-12s %-15s"
    set gvar(volumeAttribList) ""
    set indx 0
    foreach i $Media {
	set id            [lindex $i 0]
	set segregation   [lindex $i 1]
	set volumeName    [lindex $i 2]
	set mediaType	  [lindex $i 3]
	set copies        [lindex $i 4]
	set status        [lindex $i 5]
	set statusValue   [lindex $i 6]

	
	#Find what action if any is being 
	set query "
	 	select a.action,
			s.short_description
		from $gvar(database)..$gvar(actions_table) a,
			$gvar(database)..$gvar(status_table) s
		where a.media_request_id = $id
			and a.media_unit_name= '$volumeName'
			and a.action = s.status
			and s.table_name = '$gvar(actions_table)'"
	Debug $query 1
	set results [Sybsql $query]
	if { "$results" == "error" } {
	    Log "Database error doing the query:" Error
	    Log "$query" Error
	    ::.astoControl.bot.sh displayText "Database Error !"
	    set gvar(volumesStatusFlag) 0
	    return 1
	} elseif { "$results" == "" } {
	    set action "None"
	    set actionValue "None"
	} else {
	    set action [ lindex [lindex $results 0] 0]
	    set actionValue [lindex [lindex $results 0] 1]
	}

	#Display the volume item
	set entry "\"$id\" \"$volumeName\" \"$segregation\" \"$copies\" \"$statusValue\" \"$actionValue\""
	lappend volumeBuffer [eval format \"$formatLine\" $entry]
	set stateList(I) { IR Q }
	set stateList(Q) { QR P }
	set stateList(P) { PR W }
	set stateList(W) { WR C }
	set stateList(C) { CR X }

	if { ( "$status" == "" && "$action" != "Q" ) ||
	     ( "$status" == "D" && "$action" != "" ) ||
	     ( [ lindex $stateList($status) 0 ] != "$action" &&
	       [ lindex $stateList($status) 1 ] != "$action" &&
	       "$action" != "None" ) } {
	    # Action should be being queued but it is not
	    # or action is done but being processed.
	    set attrib red
	} elseif { "$status" == "D" } {
	    #volume cleaned and ready to be inserted in the juke box
	    set attrib white
	} else {
	    #Set the attributes, based on the action.
	    #Q  = unit is being queued.
	    #QR = unit is being unqueud.
	    #P  = Unit is being prepared.
	    #PR = unit is being unprepared.
	    #W  = unit is being written.
	    #WR = Unit is being unwritten.
	    #C  = unit is being cleaned up.
	    
	    switch -- $action {
		Q   {set attrib seagreen4 }
		P   {set attrib seagreen4 }
		W   {set attrib seagreen4 }
		C   {set attrib seagreen4 }
		QR  {set attrib seagreen4 }
		PR  {set attrib seagreen4 }
		WR  {set attrib seagreen4 }
		None     {set attrib  blue}
		default  { set attrib red }
	    }
	}
	lappend gvar(volumeAttribList) "$attrib"
    }
    
    #Maintain the selection if any
    set volumeIndex    [$astoCV.astoVolumeWindow    getIndex]
    
    #Update the Media List Box
    $astoCV.astoVolumeWindow deleteItem 0 end
    foreach line $volumeBuffer {
	$astoCV.astoVolumeWindow addItem end $line
    }

    set num [ $astoCV.astoVolumeWindow getSize]
    set pos [expr {$num - 12}]; # 12 is the number of lines of the list.
    [$astoCV.astoVolumeWindow component uifFancyListBox] yview $pos
    
    dominoAttrib $gvar(volumeAttribList) $astoCV.astoVolumeWindow
    if {$volumeIndex != ""} {$astoCV.astoVolumeWindow setSelection $volumeIndex}
    ::.astoControl.bot.sh displayText ""
    
    set gvar(volumesStatusFlag) 0
}

#Update the listboxes
proc astoStatus {} {
    global gvar env astoCV sybmsg
    
    
    if {$gvar(astoStatusFlag)} { return }
    
    disableUpdateButtons
    BusyWin $astoCV  {
	set gvar(astoStatusFlag) 1    

	#Check that status of the media requests
	if {[checkRequests] != 1} {
	    # Check the Status of Media volumes.
	    checkVolumes
	}
	
	clearSelection $astoCV.astoRequestWindow request
	clearSelection $astoCV.astoVolumeWindow volume

    }
    set gvar(astoStatusFlag) 0
    if {$gvar(autoStatus) == 0}  { enableUpdateButtons }
}

#Procedure that determines the volume percentage that is ready to
#be queued.
proc getVolPercentage { id seg mediaType} {
    global gvar


    set query "select isnull(sum(f.filesize/1024.0), 0)
	       from $gvar(database)..$gvar(files_table) f
    	       where f.media_request_id = $id
			and f.media_unit_name = NULL
			and f.status = 'R'"
    Debug $query 1
    set filesIngested [Sybsql $query]
    if { "$filesIngested" == "" || "$filesIngested" == "error" } {
	Log "Database error doing the query: " Error
	Log "$query" Error
	::.astoControl.bot.sh displayText "Data Base error ! Getting percentage"
	return 0 
    }	
    
    set query "select m.capacity
	       from $gvar(database)..$gvar(mediaTypesTable) m
	       where m.media_type = '$mediaType'" 
    Debug $query 1
    set mediaSize [Sybsql $query]
    if { "$mediaSize" == "" || "$mediaSize" == "error" } {
	Log "Database error doing the query: " Error
	Log "$query" Error
	::.astoControl.bot.sh displayText "Data Base error ! Getting percentage"
	return 0 
    }
    
    if {[catch {set percentage [expr {ceil( ($filesIngested / $mediaSize) * 100 )}]}]} {
	Log "Error calculating the percentage: " Error
	Log "expr (ceil ($filesIngested / $mediaSize) * 100)" Error
	::.astoControl.bot.sh displayText "Data Base error ! Getting percentage"
	return 0 
    }	
    
    return $percentage
}

#Procedure to define colours of unit entries
proc dominoAttrib { attribList fancyListBox } {
    global gvar astoCV

    set indx 0
    foreach attrib $attribList {
	if {$attrib == "overstrike"} {
	    set option "-overstrike 1"
	} else { 
	    set option "-foreground $attrib"
	}
	eval [$fancyListBox component uifFancyListBox] item configure $indx $option
	incr indx +1
    }
}



#############################################################################
#
#			Ingest Functions
#
#############################################################################

proc autoIngest {} {
    global gvar astoCV

    if {$gvar(autoIngest) == 0 || $gvar(autoIngestFiles) != 0 ||
	$gvar(autoQueueFiles) != 0 || $gvar(manualIngest) != 0 
	|| $gvar(manualQueue) != 0 } { return }

    set gvar(autoIngestFiles) 0
    
    set query "select media_request_id, media_request_name
	from $gvar(database)..$gvar(media_table)
	where status != 'D' 
	order by media_request_id" 
    Debug $query 1
    set results [Sybsql $query]
    if { $results == "error" } { 
	set gvar(autoIngestFiles) 1
	return 
    }
    incr gvar(ingestIndex) 

    foreach lines $results {
	set id [lindex $lines 0]
	set seg [lindex $lines 1] 

	Log "Automatic ingest files for segragation $seg" Info
	::.astoControl.bot.sh displayText "Automatically ingesting files for segregation $seg"
	set cmd "mediaIngest -i $id -v"
	Debug "Executing command: $cmd" 1
	catch {eval blt::bgexec gvar(ingestStatus-$gvar(ingestIndex)) -output gvar(ingestOutput) \
		   -error gvar(ingestErrors) [lindex $cmd 0] [lrange $cmd 1 end]}\
	    msg

	if {[string match "*Error*" $gvar(ingestOutput)]} {
	    Log "Error invoking command:\n \"$cmd\"" Error
	    Info "Error invoking command:\n \"$cmd\"" 
	    Debug "mediaIngest Error: $gvar(ingestErrors) --- msg $msg" 1
	} else {::.astoControl.bot.sh displayText "Segregation $seg has been automatically ingested."}
    }
    
    after 1 checkRequests
    incr gvar(ingestIndex) -1 
    set gvar(autoIngestFiles) 1
}

proc manualIngest {} {
    global gvar astoCV manualIngErrors

    set selection   [string trim [$astoCV.astoRequestWindow getSelection] "{}"]
    
    if {$selection == ""} {
	Info "You have to select a queue first." 
	return }
    
    set selectIndex [$astoCV.astoRequestWindow getIndex]
    set selectColour [getAttrib astoRequestWindow $selectIndex]

    #???Disable
    set id 		[lindex $selection 0]
    set requestName 	[lindex $selection 1]
    set action 		[getQueueAction $id]
    #???    set gvar(manualMode) $id

    #If the not already queueing or ingesting automatically then ingest
    # the selected request.
    if {$gvar(autoIngest) != 0 || $gvar(autoQueue) != 0 } { 
	Info "Can not Ingest $requestName. Automatic Queue or Automatic Ingest is turned on."; return }

    if { $action !=  "None" } {
	Info "This segregation is already being processed.\nTry again later."  
	return }

    #gvar(manualIngest) = 1 --> Execute mediaIngest on selected request.
    #gvar(manualIngest) = 2 --> Execute mediaIngest -rollback

    if {$gvar(manualIngest) != 1 && $gvar(manualIngest) != 2} {
	Info "Ingest operation unknown." 
	return }
    if {$gvar(manualIngest) ==  1 } {
	set displayMsg "Manually ingesting files for segregation ${requestName}."
	set cmd "mediaIngest -i $id -v"
    	set colour seagreen4
    } elseif {$gvar(manualIngest) ==  2 } {
	set displayMsg "Manually removing free files for segregation ${requestName}."
	set cmd "mediaIngest -i $id -rollback -v"
	set colour seagreen4
    } else {
	Info "Ingest operation unknown." 
	return
    }

    incr gvar(ingestIndex)
    [$astoCV.astoRequestWindow component uifFancyListBox] item configure $selectIndex  -foreground $colour
    update idletasks
    ::.astoControl.bot.sh displayText $displayMsg
    after 1000 astoStatus
     
    Log "Executing command: $cmd" Info
    Debug "Executing command: $cmd" 1
    catch {eval blt::bgexec gvar(ingestStatus-$gvar(ingestIndex)) -output gvar(ingestOutput) \
	       -error gvar(ingestErrors) [lindex $cmd 0] [lrange $cmd 1 end]} msg
        
    if {[string match "*Error*" $gvar(ingestOutput)]} {
	Log "Error invoking command:\n \"$cmd\"" Error
	Info "Error invoking command:\n \"$cmd\"" 
	Debug "mediaIngest Error: $gvar(ingestErrors) --- msg: $msg" 1 
    } else { 
	::.astoControl.bot.sh displayText "Succesesful: $displayMsg"
    }
    [$astoCV.astoRequestWindow component uifFancyListBox] item clear $selectIndex
    #??? enableIngest
    set gvar(manualIngest) 0
    incr gvar(ingestIndex) -1
    checkRequests
}

#############################################################################
#
#			Queue Functions
#
#############################################################################

#Execute mediaQueue automatically.
proc autoQueue {} {
    global gvar env execErrors astoCV queueOutput queueStatus

    #If the autoQueue is set and we aren't already queuing or ingesting
    # requests then queue all requests that are ready.

    if {$gvar(autoQueue) == 0 ||
	$gvar(autoIngestFiles) != 0 || $gvar(manualIngest) != 0 
	|| $gvar(manualQueue) != 0 } { return }

    set query "select media_request_id, media_request_name, media_type
	from $gvar(database)..$gvar(media_table)
	where status != 'D' 
	order by media_request_id" 
    Debug $query 1
    set results [Sybsql $query]    
    if { $results == "error" } { 
	set gvar(autoQueueFiles) 1
	return 
    }
    
    foreach lines $results {
	set id [lindex $lines 0]
	set seg [lindex $lines 1]
	set mediaType [lindex $lines 2] 

	set percentage [getVolPercentage $id $seg $mediaType]
	if { $percentage < 100 } { continue }

	Log "Automatically queuing files for segregation $seg" Info
	::.astoControl.bot.sh displayText "Automatically queuing files for segregation $seg"
	set cmd "mediaQueue -i $id -v"
	Debug "Executing command: $cmd" 1
	catch {eval blt::bgexec gvar(queueStatus-$gvar(queueIndex)) -output gvar(queueOutput) \
		   -error gvar(queueErrors) [lindex $cmd 0] [lrange $cmd 1 end]}\
	    msg

	if {[string match "*Error*" $gvar(queueOutput)]} {
	    Log "Error invoking command:\n \"$cmd\"" Error
	     ::.astoControl.bot.sh displayText "Error invoking command:\n \"$cmd\"" 
	    Debug "mediaQueue Error: $gvar(queueErrors) --- msg $msg" 1
	} else {
	    
	    ::.astoControl.bot.sh displayText "Segregation $seg has been automatically queued."
	}
	if {$gvar(parallelAsto)} {
	    set query "update $gvar(units_table) 
	    set host = \'$gvar(HOST)\'
	    where media_request_id = $id and host = NULL"
	    Debug "query: $query" 1
	    set results [Sybsql $query]
		Debug "Result: $results" 2
	    if {$results == "error"} { 
		Log "Error setting hostname $gvar(HOST) in table $gvar(units_table)" Error
		Info "Error setting hostname $gvar(HOST) in table $gvar(units_table)"
		return 1
	    } 
	}

    }
    if {$gvar(autoStatus) == 0} { after 1 astoStatus }
}


#Manually execute mediaIngest, mediaQueue or mediaQueue - force
proc manualQueue {} {
    global gvar astoCV manualIngErrors queueStatus queueOutput

    
    set selection  [string trim [$astoCV.astoRequestWindow getSelection] "{}"]
    
    if {$selection == ""} {
	Info "You have to select a segregation first." 
	return }
    
    set selectIndex  [$astoCV.astoRequestWindow getIndex]
    set selectColour [getAttrib astoRequestWindow $selectIndex]

    #???Disable
    set id 		[lindex $selection 0]
    set requestName 	[lindex $selection 1]
    set action 		[getQueueAction $id ]
    set percentage 	[lindex $selection 4]
    #???    set gvar(manualMode) $id

    #If the not already queueing or ingesting automatically then ingest
    # the selected request.
    if {$gvar(autoIngest) != 0 || $gvar(autoQueue) != 0 } { 
	Info "Can not Queue $requestName. Automatic Queue or Automatic Ingest is turned on."
	return 
    }

    if { $action != "None" } {
	Info "This segregation is already being processed. Try again later."  
	return 
    }

    #gvar(manualQueue) = 1 --> Execute mediaIngest on selected request.
    #gvar(manualQueue) = 2 --> Execute mediaIngest -force
    #gvar(manualQueue) = 3 --> Execute mediaIngest -rollback

    if {$gvar(manualQueue) != 1 && $gvar(manualQueue) != 2 &&
	$gvar(manualQueue) != 3 } {
	Log "Ingest operation unknown: manualQueue flag is not 1,2 or 3" Error
	Info "Ingest operation unknown." 
	return 
    }

    if {$gvar(manualQueue) ==  1 } {
	if { $percentage < 100 } {
	    Info "Percentage is < 100%. Use Force Queue if you quant to Queue this id"  
	    return 
	}

	Log "Manually queuing files for segregation ${requestName}." Info
	set displayMsg "Manually queuing files for segregation ${requestName}."
	set cmd "mediaQueue -i $id -v"
    	set colour seagreen4
    } elseif {$gvar(manualQueue) ==  2 } {
	Log "Manual forced queuing of segregation ${requestName}." Info
	set displayMsg "Manual forced queuing of segregation ${requestName}."
	set cmd "mediaQueue -i $id -force"
    	set colour seagreen4
    } elseif {$gvar(manualQueue) ==  3 } {
	Log "Manually deleting queued volumes for segregation ${requestName}." Info
	set displayMsg "Manually deleting queued volumes for segregation ${requestName}."
	set cmd "mediaQueue -i $id -rollback"
	set colour seagreen4
    } else {
	Debug "Manual queue operation unknow. manualQueue flag: $gvar(manualQueue)" 3
	Log "Manual queue operation unknow. Turn on debug mode to investigate" Error
	Info "Manual queue operation unknown." 
	return
    }
    incr gvar(queueIndex)

    #Disable the buttons. More than one operation is not allowed.
    disableQueueButtons
    [$astoCV.astoRequestWindow component uifFancyListBox] item configure \
	$selectIndex  -foreground $colour
    update idletasks
    ::.astoControl.bot.sh displayText $displayMsg
    Debug "Executing command: $cmd" 1
    after 1000 astoStatus
    
    catch {eval blt::bgexec gvar(queueStatus-$gvar(queueIndex)) \
	       -output gvar(queueOutput) \
	       -error gvar(queueErrors) $cmd } msg 

    if {[string match "*Error*" $gvar(queueOutput)]} {
	Log "Error invoking command:\n \"$cmd\"\nMsg: $msg" Error
	Info "Error invoking command:\n \"$cmd\"\nMsg: $msg" 
	Debug "mediaQueue Error: $gvar(queueErrors) --- msg: $gvar(queueOutput)" 1 
    } else {
	::.astoControl.bot.sh displayText "Succesesful: $displayMsg"
    }

    if {$gvar(parallelAsto)} {
	set query "update $gvar(units_table) 
	    set host = \'$gvar(HOST)\'
	    where media_request_id = $id and host = NULL"
	Debug "query: $query" 1
	set results [Sybsql $query]
	Debug "Result: $results" 2
	if {$results == "error"} { 
	    Log "Error setting hostname $gvar(HOST) in table $gvar(units_table)" Error
	    Info "Error setting hostname $gvar(HOST) in table $gvar(units_table)"
	    return 1
	} 
    }
    
    
    if {[info exists gvar(queueOutput)]} {
	puts "Output: $gvar(queueOutput)"
	puts "Message: $msg"
    }
    
    [$astoCV.astoRequestWindow component uifFancyListBox] item clear \
	$selectIndex
    
    enableQueueButtons
    set gvar(manualQueue) 0
    incr gvar(queueIndex) -1
    if {$gvar(autoStatus) == 0} { after 1 astoStatus }
}

#Roll back a single volume
proc manualUnqueue {} {
    global gvar astoCV manualIngErrors 
    
    set selection  [string trim [$astoCV.astoVolumeWindow getSelection] "{}"]
    
    if {$selection == ""} {
	Info "You have to select a volume first." 
	return 
    }

    set selectIndex  [$astoCV.astoVolumeWindow getIndex]
    set selectColour [getAttrib astoRequestWindow $selectIndex]

    set id 	[lindex $selection 0]
    set volume 	[lindex $selection 1]
    set state [getVolumeStatus $id $volume]
    set action [getVolumeAction $id $volume]

    if {$state != "Queued" || $action != "None"} { 
	Info "This volume is not in Queued state\n or there is an action performing on it."
	return 
    }

    set query " select 	media_unit_name
		from $gvar(units_table)
		where status = 'Q'"
    Debug $query 1
    set volumes [Sybsql $query]

    set lastVolIndx [llength $volumes]
    set myVolIndx   [lsearch -exact $volumes $volume]
    set num_units   [expr {$lastVolIndx - $myVolIndx}]

    set cmd "mediaQueue -i $id -rollback -num_units $num_units -v"
    set displayMsg "Manually deleting queued volume $volume"
    set colour seagreen4
    incr gvar(queueIndex)

    disableWriteButtons
    disableQueueButtons
    [$astoCV.astoVolumeWindow component uifFancyListBox] item configure \
	$selectIndex  -foreground $colour
    update idletasks
    ::.astoControl.bot.sh displayText $displayMsg
    after 1000 astoStatus

    Log "Rollback $num_units unit(s) from segregation $id" Info
    Debug "Executing command: $cmd" 1
    catch {eval blt::bgexec gvar(queueStatus-$gvar(queueIndex)) -output gvar(queueOutput) \
	       -error gvar(queueErrors) [lindex $cmd 0] [lrange $cmd 1 end]} msg
    
    if {[string match "*Error*" $gvar(queueOutput)]} {
	Log "Error invoking command:\n \"$cmd\"" Error
	Info "Error invoking command:\n \"$cmd\"" 
	Debug "mediaQueue Error: $gvar(queueErrors) --- msg: $msg" 1 
    } else { 
	::.astoControl.bot.sh displayText "Successful: $displayMsg" 
    }
    
    [$astoCV.astoVolumeWindow component uifFancyListBox] item clear \
	$selectIndex
    
    incr gvar(queueIndex) -1
    enableWriteButtons
    enableQueueButtons

    if {$gvar(autoStatus) == 0} { after 1 astoStatus }
}

#Procedure to prepare AND write a CD.
proc mediaMake {args} {
    global gvar astoCV manualIngErrors prepStatus prepOutput
    
    
    if {$args == ""} {
	set selection  [string trim [$astoCV.astoVolumeWindow getSelection] "{}"]} else {set selection $args
	}
    
    if {$selection == ""} {
	Info "You have to select a volume first." 
	return 
    }

    set selectIndex  [$astoCV.astoVolumeWindow getIndex]
    set selectColour [getAttrib astoRequestWindow $selectIndex]

    set id 	[lindex $selection 0]
    set volume 	[lindex $selection 1]
    set state   [lindex [getVolumeStatus $id $volume] 0]
    set action  [lindex [getVolumeAction $id $volume] 0]
    set copies  [lindex $selection 3]
    set volumeSize [getVolumeSize $id $volume]
	if {"$action" != "None" } {
		Info "Volume $volume in process." 
		return 1
	}
	if { [ set stagingDir [ getFreeStageDir $id $volume ] ] == "error" } {
	    Log "Error: there is no free staging directory\nfor volume $volume" Error
	    Info "Error: there is no free staging directory\nfor volume $volume"
	    return 1
	}
    
    
    
    if { $state == "Queued" } {
	set diskSpace  [FreeSpace $gvar(mediaDataStage)/$stagingDir]
	if {$diskSpace < $volumeSize} { 
	    Log "Not enough disk space under $gvar(mediaDataStage)/$stagingDir.\nYou need at least $volumeSize Kbytes" Error
	    Info "Not enough disk space under $gvar(mediaDataStage)/$stagingDir.\nYou need at least $volumeSize Kbytes"
	    return 1 
	}
	set returnPrepValue [manualPrep $id $volume $stagingDir $selectIndex]
	if {$returnPrepValue == 0} {
	    #after 3000 "mediaMake $selection"
	    
	    while {$gvar(manualWriteBusy)} {
		after 1000
	    }
	    manualWrite $id $volume $selectIndex $copies
	}
    } elseif { $state == "Prepared"} {
	manualWrite $id $volume $selectIndex $copies 
    } else { 
	Info "This volume is not in the Queued/Prepared state."
	enableWriteButtons
	return 
    }
    
   # checkVolumes
}




#############################################################################
#
#			Prepare Functions
#
#############################################################################

proc autoPrep {} {
    global gvar astoCV execErrors prepOutput prepStatus

    #If the autoPrep is set and we aren't already preping then prepare
    #those that are ready.
	
    if {$gvar(autoPrep) == 0 || $gvar(autoPrepVolumes) == 1 ||
		$gvar(manualPrep) != 0 } { return }
    
    set query "select u.media_request_id, u.media_unit_name
	from $gvar(units_table) u
	where u.status = 'Q' 
	order by media_request_id" 
    Debug $query 1
    set results [Sybsql $query]
    
    if {$results == "" || $results == "error" } { return }
    incr gvar(prepIndex)

    set gvar(autoPrepVolumes) 1
    foreach lines $results {
		set id		[lindex $lines 0]
		set volume	[lindex $lines 1]
		set volumeSize [getVolumeSize $id $volume]
		if { [ set stagingDir \
				   [ getFreeStageDir $id $volume ] ]
			 == "error" } { continue }
		
		set diskSpace  [FreeSpace $gvar(mediaDataStage)/$stagingDir]
		#set diskSpace  [FreeSpace /tmp]

	if {$diskSpace < $volumeSize} { 
	    Log "Not enough disk space under $gvar(mediaDataStage)/$stagingDir\nto write volume $volume.\nYou need at least $volumeSize Kbytes" Error
	    ::.astoControl.bot.sh displayText "Not enough disk space under $gvar(mediaDataStage)/$stagingDir\nto write volume $volume.\nYou need at least $volumeSize Kbytes"
	    Debug "Not enough disk space under $gvar(mediaDataStage)/$stagingDir\nto write volume $volume.\nYou need at least $volumeSize Kbytes" 1
	    continue
	}
	
      	set action [getVolumeAction $id $volume]
	if { $action != "None" } { continue }
	
	Log "Automatically preparing volume $volume" Info
	::.astoControl.bot.sh displayText "Automatically preparing volume $volume"
	
	
	#??? Do some check to make sure it is okay to prep.
	#??? 	- Enough disk space base on capacity of media type.
	#???	- Volume is not busy
	
	set cmd "mediaPrep -i $id -n $volume -d $stagingDir -v"
	Debug "Executing command: $cmd" 1
	
	catch {eval blt::bgexec gvar(prepStatus-$volume) -output gvar(prepOutput-$volume) \
		   -error gvar(prepErrors) [lindex $cmd 0] [lrange $cmd 1 end] & } msg
	trace variable gvar(prepStatus-$volume) w tracePrep
	after 1000
    }
    
    incr gvar(prepIndex) -1
    set gvar(autoPrepVolumes) 0
    checkVolumes
}

proc tracePrep {varName index op} {
    global gvar
    
    if {[catch {regexp {^(prepStatus-)(.*)} $index tmp1 tmp2 suffix} msg]} {
	Log "Error parsing index in tracePrep procedure" Error
	Debug "Error parsing index in tracePrep procedure" 1
	return
    }
    #set suffix [lindex [split $index "-"] 1]
    set output "prepOutput-$suffix"
    
    if {[string match "*Error*" $gvar($output)]} {
	::.astoControl.bot.sh displayText "Error invoking mediaPrep for volume $suffix"
	Log "Error invoking mediaPrep for volume $suffix\noutput:$gvar($output)" Error
	Debug "Error invoking mediaPrep for volume $suffix\noutput:$gvar($output)" 1
	setHostToVolume $suffix
    } else {
	::.astoControl.bot.sh displayText "Volume $suffix prepared"
    }
    
    unset gvar($index)
    unset gvar($output)
    checkVolumes
}

proc manualPrep {id volume stagingDir selectIndex} {
    global gvar astoCV prepStatus prepOutput prepErrors

    
    set gvar(manualPrep) 1
    Log "Manually preparing volume ${volume}." Info
    set displayMsg "Manually preparing volume ${volume}."
    set cmd "mediaPrep -i $id -n $volume -d $stagingDir -v"
    set colour seagreen4
    incr gvar(prepIndex)
    
    [$astoCV.astoVolumeWindow component uifFancyListBox] item configure $selectIndex  -foreground $colour
    update idletasks
    ::.astoControl.bot.sh displayText $displayMsg
    #after 1000 checkVolumes
    Debug "Executing command: $cmd" 1
    catch {eval blt::bgexec gvar(prepStatus-$volume) -output gvar(prepOutput) \
	       -error gvar(prepErrors) $cmd &} msg 
    checkVolumes
    tkwait variable gvar(prepStatus-$volume)
    
    if {[string match "*Error*" $gvar(prepOutput)]} {
	Log "Error invoking command:\n \"$cmd\"\nError: $gvar(prepErrors) " Error
	Info "Error invoking command:\n \"$cmd\"\nError: $gvar(prepErrors) " 
	Debug "mediaPrep Error: $gvar(prepErrors) --- msg: $msg" 1 
	set retValue 1
    } else { 
	set gvar(prepStatus) ""
	set gvar(prepOutput) ""
	set gvargvar(prepErrors) ""
	set retValue 0
	::.astoControl.bot.sh displayText "Successful: $displayMsg" 
    }
    
    setHostToVolume $volume
    [$astoCV.astoVolumeWindow component uifFancyListBox] item clear $selectIndex
    checkVolumes
    set gvar(manualPrep) 0
    incr gvar(prepIndex) -1
    
    return $retValue
}

#Procedure for booking an unused media staging directory
proc getFreeStageDir { id volume } {
    global gvar

    #Check the existance of the cd directories.
    if {[catch {set dirsOnDisk [glob $gvar(mediaDataStage)/*]}]} {
	set msg "No Media Staging Directories defined. Check under $gvar(mediaDataStage)"
	Log $msg Error
	Info $msg
	return "error"
    }
    
    set dirList [getDirList $dirsOnDisk]
    if {$dirList == "error"} {
	set msg "Directory non valid. Check under $gvar(mediaDataStage)"
	Log $msg Error
	Info $msg  
	return "error"
    }

    #Get the list of free directories.
    set query "
        select s.directory, s.media_request_id, s.media_unit_name
        from $gvar(database)..$gvar(stage_table) s
        where s.media_request_id in ( NULL, $id )
        and s.media_unit_name in ( NULL, '$volume' )
        and s.directory in ($dirList)
	order by s.media_request_id desc" 
    
    Debug $query 1
    set freeDirs [Sybsql $query]

    if {$freeDirs == "error" } { 
	::.astoControl.bot.sh displayText "Database Error !"
	Log "Database error while getting a free directory" Error
	return "error"
    } elseif { "$freeDirs" == "" } {
	Debug "There are no staging directories available." 1
	return "error"
    }

    set index [lsearch -glob $freeDirs "*{$id}*${volume}"]
    if {$index == -1} {
	for {set index 0} {$index < [llength $freeDirs]} {incr index} {
	    set dir [lindex [lindex $freeDirs 0] 0]
	    set mediaStage "$gvar(mediaDataStage)/$dir" 
	    if {![file isdir $mediaStage] } {
		Info "Media Staging directory $mediaStage doesn't exist."
	    } else {
		break
	    }
	}
	if { $index == [llength $freeDirs] } {
	    Debug "There are no staging directories available." 1
	    return "error"
	}
    } 
    set row [lindex $freeDirs $index]
    set dir [lindex $row 0]
    
    Debug "Staging directory $dir booked for volume $volume" 1   
    return "$dir"
}

proc getDirList {dirsOnDisk} {

    set dirList ""
    foreach directory $dirsOnDisk {
	
	if {![file isdirectory $directory] ||
	    ![file readable $directory] ||
	    ![file writable $directory]} {
	    return "error"
	} 
	
	set cleanDir [file tail $directory]
	lappend dirList \"$cleanDir\"
    }
    
    set directoryList [join $dirList ","]
    return $directoryList
}


#############################################################################
#
#			Write Functions
#
#############################################################################

proc autoWrite {} {
    global gvar astoCV execErrors writeStatus writeOutput

    #if {$gvar(autoWriteVolumes) == 1} { return }
    
    #Set the -dummy option in case the dummy mode was chosen.
    set options ""
    if {$gvar(dummyMode)} {
	set options " -simulate"
    } 

    set query "select u.media_request_id, u.media_unit_name, m.num_copies
	from $gvar(database)..$gvar(units_table) u, $gvar(database)..$gvar(media_table) m
	where u.status = 'P'
        and u.media_request_id = m.media_request_id
	order by u.media_request_id" 
    Debug $query 1
    set results [Sybsql $query]
    
    if {$results == "" || $results == "error" } { return }
    
    #set gvar(autoWriteVolumes) 1
    foreach lines $results {
	set autoWriteId      [lindex $lines 0]
	set autoWriteName    [lindex $lines 1]
	set autoCopies       [lindex $lines 2]

	set action [getVolumeAction $autoWriteId $autoWriteName]
	if { $action != "None" } { continue }
	
	if { [set logicalDev [getFreeDevice $autoWriteId $autoWriteName $autoCopies]] == "error" } { 
	    ::.astoControl.bot.sh displayText "Unable to get a device for volume $autoWriteName"
	    continue 
	}
	Log "Writing volume $autoWriteName" Info
	::.astoControl.bot.sh displayText "Writing volume $autoWriteName"
        set cmd "mediaWrite $options -i $autoWriteId -n $autoWriteName -l $logicalDev -v"
	Debug "Executing command: $cmd" 1
	catch {eval blt::bgexec gvar(writeStatus-$autoWriteName) -output gvar(writeOutput-$autoWriteName) \
		   -error gvar(writeErrors-$autoWriteName)  $cmd & }\
	    msg
	trace variable gvar(writeStatus-$autoWriteName) w traceWrite
	after 1000
    }
    
    #incr gvar(writeIndex) -1
    #set gvar(autoWriteVolumes) 0
    checkVolumes
}

proc traceManualWrite {varName index op} {
    global gvar

    #set suffix [lindex [split $index "-"] 1]
    set suffix [string range $index 12 end]
    set output "writeOutput-$suffix"
    
    set flag1 [string match "*Error*" $gvar($output)]
    set flag2 [string match "*errors*" $gvar($output)]
    
    
    if {$flag1 || $flag2} {
	::.astoControl.bot.sh displayText "Error invoking mediaWrite for volume $suffix"
	Log "Error invoking mediaWrite for volume $suffix\noutput:$gvar($output)" Error
	Debug "Error invoking mediaWrite for volume $suffix\noutput:$gvar($output) " 1
	Info "Error writing volume $suffix"
    } else {
	::.astoControl.bot.sh displayText "Volume $suffix written."
    }
    
    
    # if {[string match "*Error*" $gvar($output)] ||
# 	[string match "*errors*" $gvar($output)] } {
# 	::.astoControl.bot.sh displayText "Error invoking mediaWrite for volume $suffix"
	
	
# 	Log "Error invoking mediaWrite for volume $suffix\noutput:$gvar($output)" Error
# 	Debug "Error invoking mediaWrite for volume $suffix\noutput:$gvar($output) " 1
# 	puts "Error invoking mediaWrite for volume $suffix\noutput:$gvar($output) " 1
# 	Info "Error writting volume $suffix"
#     } else  {
# 	::.astoControl.bot.sh displayText "Volume $suffix written."
#     }


    setHostToVolume $suffix
    unset gvar($index)
    unset gvar($output)  
    checkVolumes
}

proc traceWrite {varName index op} {
    global gvar
    
    if {[catch {regexp {^(writeStatus-)(.*)} $index tmp1 tmp2 suffix} msg]} {
	Log "Error parsing index in traceWrite procedure" Error
	Debug "Error parsing index in traceWrite procedure" 1
	return
    }
    
    #set suffix [lindex [split $index "-"] 1]
    set output "writeOutput-$suffix"
    set flag1 [string match "*Error*" $gvar($output)]
    set flag2 [string match "*errors*" $gvar($output)]
    
    if {$flag1 || $flag2} {
	::.astoControl.bot.sh displayText "Error invoking mediaWrite for volume $suffix"
	Log "Error invoking mediaWrite for volume $suffix\noutput:$gvar($output)" Error
	Debug "Error invoking mediaWrite for volume $suffix\noutput:$gvar($output)" 1
	setHostToVolume $suffix
    } else  {
	::.astoControl.bot.sh displayText "Volume $suffix written."
    }
    
    unset gvar($index)
    unset gvar($output) 
    checkVolumes
}

proc manualWrite {id volume selectIndex copies } {
    global gvar env astoCV manualIngErrors writeStatus writeOutput
    
    set gvar(manualWriteBusy) 1
    
    if { [set logicalDev [getFreeDevice $id $volume $copies] ] == "error" } {
	Info "Unable to get a device for writing.\nTry later."
	return 1
    } 
    
    #Set the -dummy option in case the dummy mode was chosen.
    set options ""
    if {$gvar(dummyMode)} {
	set options " -simulate"
    } 

    Log "Manually writing volume ${volume}." Info
    set displayMsg "Manually writing volume ${volume}."
    set cmd "mediaWrite $options -i $id -n $volume  -v -l $logicalDev"
    set colour seagreen4
    incr gvar(writeIndex)
    
    [$astoCV.astoVolumeWindow component uifFancyListBox] item configure $selectIndex  -foreground $colour
    update idletasks
    ::.astoControl.bot.sh displayText $displayMsg
    Debug "Executing command: $cmd" 1
    catch {eval blt::bgexec gvar(writeStatus-$volume) -output gvar(writeOutput-$volume) \
	       -error gvar(writeErrors-$volume) $cmd &} msg
    
    set gvar(manualWriteBusy) 0
    trace variable gvar(writeStatus-$volume) w traceManualWrite
    after 1000
    #tkwait variable gvar(writeStatus-$volume)
    [$astoCV.astoVolumeWindow component uifFancyListBox] item clear $selectIndex
    checkVolumes
    #incr gvar(writeIndex) -1
    set gvar(manualWrite) 0
}

proc getFreeDevice { id volume copies } {
    global gvar env

    #Get the list of free devices.
    set query "
              select ld.logicalName
              from  $gvar(database)..$gvar(logicalTable) ld, 
              $gvar(database)..$gvar(bookTable) bd
              where bd.inUse = 'N'
              and ld.deviceName = bd.deviceName
              and ld.deviceName in
              (select  d.deviceName 
               from $gvar(database)..$gvar(deviceTable) d,
                    $gvar(database)..$gvar(media_table) m
               where m.media_request_id = $id"
    if {$gvar(parallelAsto)} {
	append query "\n and d.host = '$gvar(HOST)'"
    }
    append query "\n and d.media_type = m.media_type
               and d.available = 'Y')
               and $copies = (select count(*) 
		       from $gvar(database)..$gvar(logicalTable) ldd
		       where ldd.logicalName = ld.logicalName)
               group by ld.logicalName
               having count(ld.logicalName) = $copies"
    
    Debug $query 1
    set logicalDev [Sybsql $query]

    if { "$logicalDev" == "" || "$logicalDev" == "error"} {
	return "error"
    }
    
    Debug "Free logical device found: $logicalDev" 1
    return "[lindex $logicalDev 0]"
}



proc getFreeDeviceOld { id volume copies } {
    global gvar

    #Get the list of free devices.
    set query " 
    select d.deviceName 
    from $gvar(database)..$gvar(deviceTable) d,
	    $gvar(database)..$gvar(media_table) m
    where m.media_request_id = $id
	    and m.media_type = d.media_type
	    and d.available = 'Y'"
    Debug $query 1
    set devices [Sybsql $query]
    set logicalDev ""
    foreach device $devices {
	set query "
    select l.logicalName, b.media_request_id, b.media_unit_name
    from $gvar(database)..$gvar(logicalTable) l,
	    $gvar(database)..$gvar(bookTable) b
    where b.inUse = 'N'
	    and b.deviceName = l.deviceName
    group by (l.logicalName)
    having b.deviceName = l.deviceName
	    and count(l.logicalName) = $copies
	    and l.deviceName = '$device'
	    and b.media_unit_name in ( NULL, '$volume' )
    order by b.media_unit_name desc"
	
	Debug $query 1
	set freeDevices [Sybsql $query]

	if { $freeDevices != "error" && $freeDevices != "" } {
	    set index [lsearch -glob $freeDevices "*{$id}*${volume}"]
	    if {$index == -1} {set index 0} 
	    set row [lindex $freeDevices $index]
	    set logicalDev [lindex $row 0]
	    break
	}
    }
    
    if { "$logicalDev" == "" } {
	return "error"
    }
    
    Debug "Free logical device found: $logicalDev" 1
    return "$logicalDev"
}

#############################################################################
#
#                  Cleanup  Functions
#
#############################################################################

proc autoCleanup {} {
    global gvar astoCV execErrors 

    if {$gvar(autoCleanupVolumes) == 1} { return }
    
    set query "select media_request_id, media_unit_name
	from $gvar(database)..$gvar(units_table)
	where status = 'W' 
	order by media_request_id" 
    Debug $query 1
    
    set results [Sybsql $query]
    
    if {$results == "" || $results == "error" } { return }
    incr gvar(cleanupIndex)

    set gvar(autoCleanupVolumes) 1
    foreach lines $results {
	set autoCleanupId      [lindex $lines 0]
	set autoCleanupName    [lindex $lines 1]

	Log "Cleaning volume $autoCleanupName" Info
	::.astoControl.bot.sh displayText "Cleaning volume $autoCleanupName"
	# If cleanVolume flag is set, remove just the physical volume, not
	# the files under the stagin area.
        if {$gvar(cleanVolume)} {
	    set cmd "mediaCleanup -i $autoCleanupId -n $autoCleanupName -keep -v"
	} else {
	    set cmd "mediaCleanup -i $autoCleanupId -n $autoCleanupName -v"
	}
	    Debug "Executing command: $cmd" 1
	catch {eval blt::bgexec gvar(cleanupStatus-$gvar(cleanupIndex)) -output gvar(cleanupOutput) \
		   -error gvar(cleanupErrors) [lindex $cmd 0] [lrange $cmd 1 end]}\
	    msg

        if {[string match "*Error*" $gvar(cleanupOutput)]} {
	    Log "Error invoking command:\n \"$cmd\"" Error
	    ::.astoControl.bot.sh displayText "Error invoking command:\n \"$cmd\""
	    Debug "cleanup Error: $gvar(cleanupErrors) --- msg $msg" 1
        } else {
	    ::.astoControl.bot.sh displayText "Volume $autoCleanupName cleaned."
	}
        after 1 checkVolumes
    }
    
    incr gvar(cleanupIndex) -1
    set gvar(autoCleanupVolumes) 0
}

proc manualCleanup {} {
    global gvar env astoCV manualIngErrors mediaCleanStatus mediaCleanOutput

    set selection [string trim [$astoCV.astoVolumeWindow getSelection] "{}"]
    
    if {$selection == ""} {
	Info "You have to select a volume first." 
	return 
    }
    

    set selectIndex [$astoCV.astoVolumeWindow getIndex]
    set selectColour [getAttrib astoRequestWindow $selectIndex]
    

    set id [lindex $selection 0]
    set volume [lindex $selection 1]
    set state  [getVolumeStatus $id $volume]
    set action [getVolumeAction $id $volume]
    
    if { $action != "None" } {
	Info "This volume is already being processed."  
	return }


    #gvar(manualCleanup) = 1 --> Execute mediaCleanup on selected item.
    if {$gvar(manualCleanup) != 1 } {
	Info "Cleanup operation unknown." 
	return 
    }

    #??? Should really get this from the database.  
    #??? Best if a global array stores these on start-up.
    if { $state != "Written" } {
	Info "This volume is not in the \"Written\" state."
	return}

    if {$gvar(manualCleanup) ==  1 } {
	set displayMsg "Releasing resources used by volume ${volume}."
	#In manual mode always clean up everything: physical volume and data.
	set cmd "mediaCleanup -i $id -n $volume -v"
	set colour seagreen4
    } else {
	Info "Cleanup operation unknown." 
	return
    }
    
    incr gvar(cleanupIndex)
    
    [$astoCV.astoVolumeWindow component uifFancyListBox] item configure $selectIndex  -foreground $colour
    update idletasks
    ::.astoControl.bot.sh displayText $displayMsg
    after 1000 astoStatus
    
    Debug "Executing command: $cmd" 1
    catch {eval blt::bgexec gvar(cleanupStatus-$gvar(cleanupIndex)) -output gvar(cleanupOutput) \
	       -error gvar(cleanupErrors) [lindex $cmd 0] [lrange $cmd 1 end]} msg
    
    if {[string match "*Error*" $gvar(cleanupOutput)]} {
	Log "Error invoking command:\n \"$cmd\"" Error
	Info "Error invoking command:\n \"$cmd\"" 
	Debug "mediaCleanup Error: $gvar(cleanupErrors) --- msg: $msg" 1 
    } else {
	#If the volume just cleaned is a user volume (4.USR)
	#set the status to 'D' in media_units
	if {$id == "4" || $id == "7" || $id == "8"} {
	    set query "update $gvar(units_table) 
	    set status = 'D'
	    where media_unit_name = '$volume' and media_request_type = \"ARCH\""
	    Debug "query: $query" 1
	    set results [Sybsql $query]
	    Debug "Result: $results" 2
	    if {$results == "error"} { 
		Log "Error setting setting \"D\"\nstatus for volume:\n$volume" Error
		Info "Error setting setting \"D\"\nstatus for volume:\n$volume"
		return
	    } 
	}
	::.astoControl.bot.sh displayText "Successful: $displayMsg"
    }
    
    [$astoCV.astoVolumeWindow component uifFancyListBox] item clear $selectIndex
    after 1 checkVolumes
    incr gvar(cleanupIndex) -1
    set gvar(manualCleanup) 0
}


#############################################################################
#
#			Disk Utilities
#
#############################################################################

#Calculate disk space occupied, in Mbytes.
proc diskSpace {dir} {

    if {![file isdir $dir]} { 
	Log "$dir does not exist. Exit." Error
	Info "$dir does not exist.\nCreate it and run astoControl again."
	exit 1
    }

    if { ![catch {set linkDir [file readlink $dir]} ] } {
	set dir $linkDir
    }
    catch {set fileList [astoExCom "find $dir -type f -follow"]}
    if {$fileList == ""} { return 0 }
    set size 0
    
    foreach file $fileList {
	if {[catch {set fileSize [file size $file]} msg]} {set fileSize 0}
	set size [expr {$size + $fileSize/1048576.0}]
    } 
    set size [expr {int($size)}]
    
    return $size
}

#Calculate the free space on disk
proc FreeSpace {dir} {
    

    if {[catch {set OS [exec uname]} msg]} {
	Info "Cannot determine operative system:\n$msg"
	return 1
    }
    if {$OS == "HP-UX"} {
	set fd [open "|/usr/bin/bdf $dir" ]
    } elseif {$OS == "SunOS" || $OS == "Linux"} {
	set fd [open "|/usr/bin/df -k $dir"]
    } else {
	Info "Unix System not recognized (OS=$OS). Cannot use df." 
	return 1
    }
 
    set space -1
    gets $fd line
    gets $fd line
    scan $line "%s %d %d %d %d %s %s" d cap f space free d disk
    
    if {$space == -1} {
	scan $line "%d %d %d %d %s %s" cap f space free d disk
    }
    close $fd
    Debug "Space on $dir = $space" 1
    return $space
}

#############################################################################
#
#			Start of infinite loops
#
#############################################################################


proc statusLoop {cmd time} {
    Debug "Run: $cmd with time: $time"  3       
    eval $cmd
    after $time statusLoop $cmd $time
}

proc autoLoop {cmd time} {
    Debug "Run: $cmd with time: $time" 3       
    eval $cmd
    after $time autoLoop $cmd $time
}

proc watchDogLoop { time} { 
    
    Debug "Run: watchDog with time: $time" 3  

}


#############################################################################
#
#		       Close Panel
#
#############################################################################

proc astoClosePanel {} {
    global gvar
    
    #Debug "Closing DB connection..."
    #set result [sybclose $sybHandle]
    #Debug "...DB connection closed. ---> $result"
    
    #Go back to the local directory.
    cd $gvar(localdir)
    exit
}





############################ Subscriber Part ################################

proc newdhs {} {
    
    wm withdraw .
    after 1000
    panImportPanel astoSubscribe
    #panCallPanel dummy
    
}


proc selectHost {selhost} {
    global gvar 
    
    set linesel [lindex $gvar(host) [lsearch -glob $gvar(host) $selhost*]]
    set gvar(DHS_HOST)  [lindex $linesel 0]
    set gvar(DHS_CONFIG) [lindex $linesel 1]
}

proc astoSubscribeCancel {} {

    panClosePanel no .astoSubscribe
    wm deiconify .
}

proc astoSubscribeAccept {} {
    global gvar env
    
    if {[checkDate $gvar(dominoFrom) $gvar(dominoTo)]} { return }
    set gvar(dominoFrom) [addZeroesToDate $gvar(dominoFrom)]
    if {$gvar(dominoTo) != ""} {
	set gvar(dominoTo) [addZeroesToDate $gvar(dominoTo)]
    }
    
    #Now I check if another dataSubscriberAsto with the same characteristics
    #is already running.
    set linesel [lindex $gvar(host) [lsearch -glob $gvar(host) $gvar(remoteHost)*]]
    set archID [lindex $linesel 0]
    set dhsConfig  [lindex $linesel 1]
    if {![regexp {@(.*):} $dhsConfig tmp remoteHost]} {
	Info "astoContol.config has an invalid entry in the host definition\n"
	return

    }
    set options ""
    
    set id $gvar(fileCategory)::$env(HOST)::$remoteHost
    if {[file readable $env(DHS_LOG)/dataSubscriberAsto-$id.pid]} {
	set pid [read_file $env(DHS_LOG)/dataSubscriberAsto-$id.pid]
	if {![catch {exec kill -26 $pid} result]} {
	    Info "There is already a dataSubscriberAsto running,\nconnected to $gvar(remoteHost)\nand retrieving $gvar(fileCategory) files."
	    return }
    }
    
    if {[info exists gvar(dominoTo)] && $gvar(dominoTo) != ""} {append options " -to $gvar(dominoTo)"}
    if {[info exists gvar(debugMode)]} {append options " -debug $gvar(debugMode)"}
    if {[info exists gvar(logName)]}   {append options " -log $gvar(logName).ds"}
    
    panClosePanel no .astoSubscribe
    wm deiconify .
    set env(DHS_HOST)   $remoteHost
    set env(DHS_CONFIG) $dhsConfig
    
    Debug "Calling: dataSubscriberAsto -filetype $gvar(fileCategory) -archid $archID -from $gvar(dominoFrom) $options" 1
    panCallPanel dataSubscriberAsto -filetype $gvar(fileCategory) -archid $archID -from $gvar(dominoFrom) $options
}

#
# ___oOo__

