#Make a select with all the checks.
proc Sybsql {select} {
    global sybmsg gvar sybHandle

    set pid [string trimleft [clock clicks] "-"]

    Debug "Connecting to sybase..." 1
    if {[catch {set sybHandle($pid) [sybconnect $gvar(sybaseUsername) $gvar(sybasePassword) $gvar(db_server) ]} handle]} {
	message "Cannot connect to server, reason: $sybmsg(dberrstr)"
	Debug  "Cannot connect to server, reason: $sybmsg(dberrstr)" 1
	#catch {sybclose $sybHandle($pid)}
	Sybclose "sybHandle($pid)"
	return "error"
    } 
    
    if {[catch {sybsql $sybHandle($pid) $select async} handle]} {
	Debug  "Cannot query/update database, reason: $sybmsg(msgtext); dberrstr = $sybmsg(dberrstr); $sybmsg(oserrstr)" 1
	Debug "The query was: $select" 2
	message "Cannot query/update database, reason: $sybmsg(msgtext); dberrstr = $sybmsg(dberrstr); $sybmsg(oserrstr)"
	#::.astoControl.bot.sh displayText "Error quering database."
	#set gvar(astoStatusFlag) 0
	set resultVar  "error" 
    } else {
	set resultVar [getResult $sybHandle($pid)]
    }
    
    Sybclose "sybHandle($pid)"
    #catch {sybclose $sybHandle($pid)}
    Debug "Result of query = $resultVar" 1
    return $resultVar
}

proc Sybcancel {} {
    global sybHandle gvar

    catch {sybcancel $sybHandle}
    catch {sybclose $sybHandle}
    set gvar(queryCancelled) 1
    set gvar(sybaseBusy) 0
}

proc Sybclose {sybVar} {
    global sybHandle

    if {![info exists [set sybVar]]} { return }

    set handle [set $sybVar]
    catch {sybclose $handle}
    unset $sybVar
    
}

#procedure to get results from Sybase
proc getResult {sybHandle} {
    global sybmsg

    set timer 0
    if {[catch {set available [sybpoll $sybHandle 1000]}]} { 
	Debug "Return after the first poll in procedure getResult" 1
	return }
    
    set inc 0
    while {[string length $available] == 0} {
	if {$inc == 60} { 
	    return 
	}
	
	if {[catch {set available [sybpoll $sybHandle 1000]}]} { 
	    Debug "Return after the second poll in procedure getResult" 1
	    return }
	incr inc
	update 
	update idletasks
    }
    
    set buffer ""
    if {[catch { set row [sybnext $sybHandle]}]} { 
	Debug "Nothing to retrieve." 3
	return }
    
    if {$sybmsg(nextrow) == "NO_MORE_RESULTS"} {
	Debug "No more results from query" 3
	return "" }
    
    while {$sybmsg(nextrow) != "NO_MORE_ROWS" && $sybmsg(nextrow) != "NO_MORE_RESULTS"} {
	lappend buffer $row
	if {[catch {set row [sybnext $sybHandle]}]} { 
	    message "Error retrieving data from the database."
	    return }
	update 
	update idletasks
    }
    return $buffer
}

#Convert seconds in sex degrees.
proc deg2sex {deg} {
    
    if {[string match *-* $deg]} {set sign "-"} else {set sign ""}
    set tmphour [expr {int(abs($deg))}]
    set tmp [format "%2d" $tmphour]
    regsub -all " " $tmp "0" hour
    Debug "sex-Sign = $sign" 3
    Debug "sex-Hour = $hour" 3
    
    set rawminute [expr {[expr {abs($deg) - $tmphour}]*60.0}]
    set tmpminute [expr {int($rawminute)}]
    set tmp [format "%2d" $tmpminute]
    regsub -all " " $tmp "0" minute
    Debug "sex-Minute = $minute" 3

    set rawsecond [expr {[expr {$rawminute - $tmpminute}]*60.0}]
    set tmpsecond [expr {int($rawsecond)}]
    set tmp [format "%2d" $tmpsecond]
    regsub -all " " $tmp "0" second
    Debug "sex-Second = $second" 3

    return "$sign$hour $minute $second"
}

#Convert seconds in hour
proc deg2hour {deg} {
    
    if {[string match *-* $deg]} {set sign "-"} else {set sign ""}
    set rawhour [expr {abs($deg/15.0)}]
    set tmphour [expr {int($rawhour)}]
    set tmp [format "%2d" $tmphour]
    regsub -all " " $tmp "0" hour
    Debug "Sign = $sign" 3
    Debug "Hour = $hour" 3

    set rawminute [expr {[expr {$rawhour - $tmphour}]*60.0}]
    set tmpminute [expr {int($rawminute)}]
    set tmp [format "%2d" $tmpminute]
    regsub -all " " $tmp "0" minute
    Debug "Minute = $minute" 3

    set rawsecond [expr {[expr {$rawminute - $tmpminute}]*60.0}]
    set tmpsecond [expr {int($rawsecond)}]
    set tmp [format "%2d" $tmpsecond]
    regsub -all " " $tmp "0" second
    Debug "Second = $second" 3
    
    return "$sign$hour $minute $second"
}

#Test procedure, always useful with tkinspect
proc testHdr {} {

    #Create Header
    set formatLine "%-6s %-10s %-8s %-8s %-10s %-10s"
    set formatHeader "obs_name seq/exp prog_id RA DEC exptime object" 
    set gvar(olasControlHdr) [eval format [concat \"$formatLine\" $formatHeader]]
    
} 

#Debug procedure, it shows internal messages
# proc Debug {text level} {
#     global gvar
    
#     if {![info exists gvar(debugMode)]} { return }
#     if { $gvar(debugMode) <  $level } { return }

#     if {[info exists gvar(logFile)]} {puts $gvar(logFile) "Debug> $text"
#     } else { puts "Debug> $text" }
# }

proc checkDate {from to} { 
    global gvar 
    
    set currentDate [clock seconds]
    
    if {$to == "" && $from == ""} {
	Info "Please enter a date"
	return 1
    }
    
    if {$to != "" && $from == ""} {
	Info "Please fill in the From field."
	return 1
    }
    
    if {$from != ""} {
	set fromDate [checkDateFormat $from]
	if {$fromDate == 0} {
	    Info "Error: The 'from' date is  not correct"
	    return 1}
	if {$fromDate > $currentDate} { 
	    Info "Error: The 'from' date is in the
future"
	    return 1
	}
    }
    if {$to != ""} {
	set toDate [checkDateFormat $to]
	if {$toDate == 0} {
	    Info "Error: The 'to' date is not correct"
	    return 1}
	if {$toDate > $currentDate} { 
	    Info "Error: The 'to' is in the future"
	    return 1
	}
	if {$fromDate > $toDate} { 
	    Info "Error: The 'to' date must be after From:"
	    return 1
	}    
    }
    return 0
}

proc addZeroesToDate {date} {

    set timeString ""
    if {[string match "*T*" $date]} {
	set splitDate [split $date "T"]
	set date [lindex $splitDate 0]
	set time [lindex $splitDate 1]
	set splittedTime [split $time ":"]
	set hours    [leadingZero [lindex $splittedTime 0]]
	set minutes  [leadingZero [lindex $splittedTime 1]]
	set seconds  [leadingZero [lindex $splittedTime 2]]
	set timeString "$hours:$minutes:$seconds"
    }
    
    set splittedDate [split $date "-"]
    set year  [lindex $splittedDate 0]
    set month [leadingZero [lindex $splittedDate 1]]
    set day   [leadingZero [lindex $splittedDate 2]]
    set goodDate "$year-$month-$day"
    if {$timeString != ""} {
	append goodDate "T$timeString"
    }
 
    if {[catch {set date [clock scan "$month/$day/$year $timeString"]}]} {
	return 0 
    } else { 
	return $goodDate
    }
}


proc checkDateFormat {date} {
    
    set timeString ""
    if {[string match "*T*" $date]} {
	set splitDate [split $date "T"]
	set date [lindex $splitDate 0]
	set time [lindex $splitDate 1]
	set timeString "$time"
    }
    
    set splittedDate [split $date "-"]
    set year  [lindex $splittedDate 0]
    set month [lindex $splittedDate 1]
    set day   [lindex $splittedDate 2]
    
    if {[string length $year] < 4} {
	return 0
    }
    if {[catch {set date [clock scan "$month/$day/$year $timeString"]}]} {
	return 0 
    } else { 
	return $date
    }
}

proc leadingZero {digit} {
    
    if {$digit == ""} { 
	return "00"
    }
    
    set length [string length $digit]
    if {$length < 2} {
	return "0$digit"
    } else {
	return $digit
    }
}


proc setHostToVolume {volume} {
    global gvar

    set query "update $gvar(units_table) 
	    set host = \"$gvar(HOST)\"
	    where media_unit_name = \"$volume\""
    Debug "query: $query" 1
    set results [Sybsql $query]
    Debug "Result: $results" 2
    if {$results == "error"} { 
	Info "Error setting setting host for volume:\n$volume"
	return
    } 
}


#Pop up info boxes.
proc Info {text} {
    tk_dialog .info "Asto Control Dialog" $text info 0 OK
    Debug $text 1
}

#Loop routine.
proc Loop { time cmd  } {
  global gvar

    if {$gvar(cancelLoop) == 1} {
	set gvar(cancelLoop) 0
	after cancel $gvar(loopId)
	#puts  "$type Loop - $gvar(loopId-$type) killed " 
    } else {
	eval $cmd
	set gvar(loopId) [after $time Loop $time \"$cmd\" ]
	#puts "LoopID = $gvar(loopId-$type) -- cmd = $cmd" 
    }
}


#-----------------------------------------------------------------------------
# run the given tcl command while displaying the busy cursor in the main window
# 
# Bugs: if $cmd contains a "return", it will not actually return the
# first time Busy is called (something to do with autoloading). After
# the first call, it will work ok. Either avoid return in $cmd, or
# call "Busy {}" first.
#-----------------------------------------------------------------------------
set busy_count 0
proc Busy {cmd} {
    set w .
    global errorInfo busy_count
    if {[incr busy_count] == 1} {
        #       set focus [focus]
        #       focus none
        blt::busy hold $w -cursor watch
        update
        update idletasks
    }
    
    # save any errors and report them later
    if {[set code [catch [list uplevel $cmd] msg]]} {
        set info $errorInfo
    } 

    if {[incr busy_count -1] == 0} {
        blt::busy release $w
        #       catch {focus $focus}
    }

    if {$code} {
        return -code $code -errorinfo $info $msg
    }
}

        
#-----------------------------------------------------------------------------
# Run the tcl command $cmd while displaying the busy cursor in  window $w.
# Also works to block a listbox but allow the scrollbars to 
# still be used. ($W should probably not be '.', use Busy for that).
#
# Note: had problems in Tk3.6 here with watch cursor showing up after
# deiconfiying the window. Try again in Tk4.0 ?
#-----------------------------------------------------------------------------
proc BusyWin {w cmd} {
    global errorInfo busywin_count

    if {! [info exists busywin_count($w)]} {
        set busywin_count($w) 0
    }

    if {[incr busywin_count($w)] == 1} {
        #       set focus [focus]
        #       focus none
        eval blt::busy hold $w -cursor watch
        update idletasks
        update
    }
    
    # save any errors and report them later
    if {[set code [catch [list uplevel $cmd] msg]]} {
        set info $errorInfo
    } 

    if {[incr busywin_count($w) -1] == 0} {
        update idletasks
        blt::busy release $w
        #       catch {focus $focus}
    }

    if {$code} {
        return -code $code -errorinfo $info $msg
    }
}

#Parse the arguments from the command line and set some options
proc setOptions {} {
    global gvar env

    for { set i 0} {$i <[llength $gvar(args)]} {incr i} {
        case [lindex $gvar(args) $i] -info {
            incr i
            explainAll
        } -debug {
            incr i
            set level [lindex $gvar(args) $i]
            if {$level == "" || $level > 3 || $level < 1} {
                explain
            }
            set gvar(debugMode) $level
        } -log {
            incr i
            set gvar(logName) [lindex $gvar(args) $i]
            if {[catch {set gvar(logFile) [open $gvar(logName) a]} var]} {
                puts "Cannot open $gvar(logName)"
                exit 1
	    }
	    if {![info exists gvar(debugMode)]} { set gvar(debugMode) 1 }
	} -cleanVolume {
	    set gvar(cleanVolume)  1
	} -dummy {
	    set gvar(dummyMode)    1
	} -writeOnly {
	    set gvar(romanicAction)  "writeOnly"
	} -printOnly {
	    set gvar(romanicAction)  "printOnly"
	} -parallelAsto {
	    set gvar(parallelAsto) 1
	} default {
            explain
        }
    }
}

proc ErrConf {msg} { 

    Log "Error reading the configuration file: $msg\nExit." Error
    Info "Error reading the configuration file: $msg\nExit."
}


proc message {txt} {
    global gvar
    
    if {$gvar(autoStatus) ||
	$gvar(autoQueue)  ||
	$gvar(autoWrite)  ||
	$gvar(autoPrep)   ||
	$gvar(autoCleanup)} {
	::.astoControl.bot.sh displayText  $txt
    } else {
	Info $txt
    }
}


#I won't explain explain =;-)
proc explain {} {
    
    puts "Usage: astoControl -info -log <filename> -debug \[1-3\] -cleanVolume -dummy -writeOnly -printOnly -parallelAsto"
    exit 1
}

proc explainAll {} {
    puts "-info           : This message"
    puts "-log <filename> : Puts all the messages in the specified file"
    puts "                : If you don't specify -debug the verbose level"
    puts "                : will be set to 1"
    puts "-debug \[1-3\]  : Define the verbose level"
    puts "-cleanVolume    : Remove the physical volume only."
    puts "                : Don't remove the files under the staging area"
    puts "-dummy          : Simulate the writing.Don't actually write"
    puts "-writeOnly      : Just write medium, don't print"
    puts "-printOnly      : Just print medium, don't write"
    puts "-parallelAsto   : Allow execution of astoControl on different machines"
    exit 1

}
#############################################################################
#
#			Debugging handler
#
#############################################################################

#Debug procedure, it shows internal messages
proc Debug {text level} {
    global gvar

    if {![info exists gvar(debugMode)]} { return }
    if { $gvar(debugMode) <  $level || $gvar(debugError) == 1 } { return }
    
    set date [clock format [clock seconds] -format "%Y-%m-%d %T"]
    if {[info exists gvar(logFile)]} {
	if {[catch {puts $gvar(logFile) "Debug>\[$date\] $text"} msg]} {
	    if {[popUp "There was an error writing to the log file.\nCheck the log file and then push retry\n or push cancel to disable the logging."]} {
		Debug $text $level
	    } else {
		set gvar(debugError) 1	
	    }
	}
    } else { 
	if {[catch {puts "Debug>\[$date\] $text"}]} {
	    set gvar(debugError) 1
	    Info "There was an error writing to the standard output, debug mode is disabled.\nIt is likely that the xterminal that launched astoControl died. Quit astoControl and restart it if you need debug information."
	} 
    }
    Log $text Debug    
}

proc Log {text priority} {
    global gvar

    if {$text == ""} { set text "-" }
    if {[catch { eval exec "dfsmsg -host $gvar(HOST) -component astoControl -priority $priority -msg \{$text\}"} msg]} {
	#puts "Text: $text"
	#puts "Priority: $priority"
	Info "Error logging message with dfsLog: $msg"
    }    
}

#Pop up a message window.
proc popUp {text} {

    #Pop up here a window with the choice cancel "request".
    set choice [tk_messageBox -icon warning -type retrycancel \
		    -title  "Asto Control Dialog" -message $text \
		    -default retry]
    if {$choice == "retry"} {
	return 1
    } else {
	return 0
    }
}





#############################################################################
#
#	   	   Database Utilities
#
#############################################################################

#Retrieve the Action of a queue from the database
proc getQueueAction { id } {
    global gvar
    
    set query "
            select s.short_description
            from $gvar(database)..$gvar(actions_table) a,
                    $gvar(database)..$gvar(status_table) s
            where a.media_request_id = $id
                    and a.media_unit_name = NULL
                    and s.status = a.action
                    and s.table_name = '$gvar(actions_table)'"
    
    Debug "query: $query" 1
    set actionInfo [Sybsql $query]
    Debug "Result of query getQueueAction: $actionInfo" 2
    
    if { "$actionInfo" == "error" } {
	::.astoControl.bot.sh displayText "Database Error !"
	return "-"
    } elseif { "$actionInfo" == "" } {
	set actionInfo "None"
    }
    
    return $actionInfo

}


#Retrieve the status of a volume from the database
proc getVolumeStatus { id volume } {
    global gvar 

    set query "select s.short_description
	        from $gvar(database)..$gvar(units_table) u,
		      $gvar(database)..$gvar(status_table) s
		where u.media_request_id = $id
                        and u.media_unit_name = '$volume'
			and u.status = s.status
			and s.table_name = '$gvar(units_table)'"
    Debug "query: $query" 1

    set volStatus [Sybsql $query]
    
    #Fill an array with the output of the query
    if { $volStatus == "error" } { 
	::.astoControl.bot.sh displayText "Database Error !"
	return "-"
    }
    Debug "Result of query getVolumeStatus: $volStatus" 2
    
    return $volStatus

}

proc getVolumeAction { id volume } {
    global gvar
    
    set query "select s.short_description
		from $gvar(database)..$gvar(actions_table) a,
			$gvar(database)..$gvar(status_table) s
		where a.media_request_id = $id
			and a.media_unit_name= '$volume'
			and a.action = s.status
			and s.table_name = '$gvar(actions_table)'"
    Debug $query 1
    set action [Sybsql $query]
    Debug "Result of query getVolumeAction: $action" 2
    if { "$action" == "error" } {
	::.astoControl.bot.sh displayText "Database Error !"
	return "-"
    } elseif { "$action" == "" } {
	set action "None"
    } 
    
    return $action
}


proc getVolumeSize { id volume } {
    global gvar
    
    set query "select total_size
		from $gvar(database)..$gvar(units_table) u
		where u.media_request_id = $id
		and u.media_unit_name= '$volume'"

    Debug $query 1
    set size [Sybsql $query]
    Debug "Result of query getVolumeSize: $size" 2
    if { "$size" == "error" } {
	::.astoControl.bot.sh displayText "Database Error !"
	return "-"
    } elseif { "$size" == "" } {
	set size 0
    }
    return $size
}

#############################################################################
#
#	   	   Miscellaneous Utilities
#
#############################################################################

#Define the colours of entries
proc getAttrib {window indx} {
    global astoCV 
    
    if {$indx == ""} { return }

    set tagName [lindex [[$astoCV.$window component uifFancyListBox] tag names [expr {$indx +1}].0] end]
    if {$tagName == ""} { return } 
    set tagAttrib [[$astoCV.$window component uifFancyListBox] tag cget $tagName -foreground]
    return $tagAttrib
}

#Change colour of selected entry
proc clearSelection { window list} {
    global gvar
    
    set indx [$window getIndex]
    if {$indx == ""} { return }

    switch -- $list {
	request {dominoAttrib  $gvar(requestAttribList) $window }
	volume  {dominoAttrib  $gvar(volumeAttribList) $window }
    } 
    
    eval [$window component uifFancyListBox] item configure $indx -foreground white
}




#Routine to execute commands in background
proc astoExCom {cmd} {
    global gvar

    incr gvar(exComIndex)
    Debug "Executing command: $cmd" 1
    catch {eval blt::bgexec gvar(exComStatus-$gvar(exComIndex)) -output gvar(exComOutput) \
	       -error gvar(exComErrors) [lindex $cmd 0] [lrange $cmd 1 end]} msg
    if {[string match "*Error*" $gvar(exComOutput)]} {
        Info "Error invoking command:\n\
                \"$cmd\"" 
	Debug "Exec Error: $gvar(exComErrors) --- msg: $msg " 1
        return -1
    }
    
    # vwait
    # tkwait variable execStatus
    incr gvar(exComIndex) -1
    return $gvar(exComOutput)
}

#Disable the Update buttons.
proc disableUpdateButtons {} {
    global astoCV
    
    $astoCV.astoControlUpdateWdg disable 
}

#Enable the Update buttons.
proc enableUpdateButtons {} {
    global astoCV
    
    $astoCV.astoControlUpdateWdg enable 
}

#Disable the Queue buttons.
proc disableQueueButtons {} {
    global astoCV
    
    $astoCV.astoControlQueueWdg disable
    $astoCV.astoForceWdg        disable
}

#Enable the Queue buttons.
proc enableQueueButtons {} {
    global astoCV
    
    $astoCV.astoControlQueueWdg enable
    $astoCV.astoForceWdg        enable
}

#Disable the Write Buttons.
proc disableWriteButtons {} {
    global gvar astoCV

    $astoCV.astoControlMediamakeWdg disable
    $astoCV.astoControlUnqueueWdg   disable
    $astoCV.mediaCleanupButtonWdg   disable
    if { $gvar(autoWrite) } {
	set gvar(autoPrep)    1
	set gvar(autoCleanup) 1}
}

#Enable the Write Buttons.
proc enableWriteButtons {} {
    global gvar astoCV
    
    $astoCV.astoControlMediamakeWdg enable
    $astoCV.astoControlUnqueueWdg   enable
    $astoCV.mediaCleanupButtonWdg   enable
    if { $gvar(autoWrite) == 0} { 
	set gvar(autoPrep)    0
	set gvar(autoCleanup) 0 }
}


#############################################################################
#
#			Popup Windows
#
#############################################################################

#Pop up a window with messages
proc astoPopUp { mess1 yesflag okflag noflag } {
    global errorInfo popflag
    set w .astoPopUp

    if {[winfo exists $w]} { destroy $w }

    set title  "Asto Error Dialog"

    # Determine an estimate of the window width

    set len       [string length $mess1]
    set msgWidth  "8i"
    if {[set idx [string first "\n" $mess1]] != -1 } {
        set len $idx
    }

    if {$len > 36 } { set msgWidth "12i" }

    toplevel $w -class Dialog
    wm title $w $title
    wm iconname $w Dialog

    frame $w.top -relief raised -bd 1
    frame $w.bot -relief raised -bd 1

    #label $w.lbl1 -text "Tcl/Tk Error Stack" \
        #              -font "-Adobe-helvetica-bold-r-normal--14*"

    #UifText $w.stack -width 36 -height 8 \
        #                 -font "-Adobe-courier-medium-r-normal--12*"

    pack $w.top            -fill both
    #pack $w.lbl1  -pady 5  -fill x
    #pack $w.stack -pady 3  -expand yes -fill x
    pack $w.bot            -expand yes -fill x
    
    #$w.stack insertText 0.0 $errorInfo
    
    #
    # Fill the top part with bitmap and message.
    #
    
    message $w.msg -width $msgWidth  -text $mess1 \
        -font -Adobe-Times-Medium-R-Normal-*-180-*
    pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 5m -pady 5m
    
    #
    # Create a row of buttons at the bottom of the dialog.
    #
    frame $w.bottoni -relief sunken -bd 1
    button $w.bottoni.yes -text YES -command "set popflag 1"

    button $w.bottoni.ok -text OK -command "set popflag 2"
    button $w.bottoni.no -text NO -command "set popflag 0"
    pack append $w.bottoni $w.bottoni.yes {left expand fillx padx 30} \
        $w.bottoni.ok {left expand fillx padx 30} \
        $w.bottoni.no {left expand fillx padx 30}
    pack append $w $w.bottoni {top fill}

    if {$yesflag == 1} {$w.bottoni.yes configure -state normal
    } else {$w.bottoni.yes configure -state disabled}

    if {$okflag == 1} {$w.bottoni.ok configure -state normal
    } else {$w.bottoni.ok configure -state disabled}

    if {$noflag == 1} {$w.bottoni.no configure -state normal
    } else {$w.bottoni.no configure -state disabled}

    update idletasks

    # Set a grab and claim the focus too.

    set oldFocus [focus]
    grab $w ; focus $w


    # Set Geometry to allow the user to re-size only along X

    set geo    [wm geometry $w]
    scan $geo "%dx%d+%d+%d" wid hei posx posy

    wm minsize $w $wid $hei
    wm maxsize $w 1000 $hei
    #
    # Wait for the user to respond, then restore the focus and
    # return the index of the selected button.dominoProc.tcl
    #
    
    tkwait variable popflag
    destroy $w
    return $popflag
    
}

#Pop up the ingest Volume panel
proc astoIngestVol {} {
    
    panImportPanel astoIngestVol
}

#############################################################################
#
#			Help
#
#############################################################################

proc showHelp {} {
    
    set text {
	D = Unit Completed
	C = Delay for Notify
	W = Done Writing
	Q = Unit Queue
	" "  = Unit Being Queued
	B = Writing in progress
	(Blocked)
    }

    #$dpCV.dpViewerListWdg deleteItem 0 end
    #$dpCV.dpViewerListWdg addItemList 0 $txt
    tk_dialog .info "astoControl Dialog" $text info 0 OK

}

