#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: olasControlProc.tcl,v 1.1 2004/09/06 14:47:14 cguirao Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# fsogni  24/06/98  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


proc olasControlInitX {} {
    global gvar cv olasCV sybmsg env


    #Load Sybtcl package
    if {[catch {package require Sybtcl}]} {Info "I cannot load Sybtcl.\nPlease check the library path or call the operator."; exit }
    if {![info exists env(DSQUERY)]} { Info "DSQUERY variable not set"; exit }
    


    Debug "Sybtcl package loaded" 3
    
    set olasCV $cv
    #Get the sybase username and password
    set gvar(db_server) $env(DSQUERY)
    set gvar(database)  observations
    if {[catch {set sybaseLogin [exec dbrcGet $gvar(db_server) $gvar(database)]} var]} {Info "I cannot get the DB password.\nCall the operator."; exit }
    set gvar(sybaseUsername) [lindex $sybaseLogin 0]
    set gvar(sybasePassword) [lindex $sybaseLogin 1]
    	
    Debug "Sybase username= $gvar(sybaseUsername)" 3
    
    #Set some variables and flags
    set gvar(expStartFrom) "[clock format [clock seconds] -format "%Y-%m-%d" -gmt 1]"
    set gvar(expStartTo)   ""
    set gvar(tmpToDate)    ""
    set gvar(entriesFlag)  0
    set gvar(cancelLoop) 0
    set gvar(olasControlGo) 0
    set gvar(queryCancelled) 0
    set gvar(olasDetailsFlag) 0
    set gvar(updateTimeRate) 10000
    set  gvar(updateTime) 10
    #Create Header
    set formatLine "%12s %-13s %-10s %-10s %10s %-10s"
    set formatHeader "\"  tpl:exp  \" prog_id RA DEC exptime object" 
    set gvar(olasControlHdr) [eval format [concat \"$formatLine\" $formatHeader]]

    #Bindings
    bind $olasCV.expStartFromWdg.uifEntry  <Return> {olasControlUpdateList}
    bind $olasCV.expStartToWdg.uifEntry  <Return> {olasControlUpdateList}
    #$olasCV.olasControlListWdg setBind <Double-1> {puts "Double click"}
    #bind $olasCV.olasControlListWdg.f1.uifFancyListBox <Double-1> {puts "double click"} 
    bind $olasCV.olasControlCancelWdg.uifButton <Enter> {arrowCursor}
    bind $olasCV.olasControlCancelWdg.uifButton <Leave> {watchCursor}
    bind $olasCV.olasControlQuitWdg.uifButton <Enter> {arrowCursor}
    bind $olasCV.olasControlQuitWdg.uifButton <Leave> {watchCursor}
    $olasCV.olasControlListWdg setBind <Button-1> {olasControlToggleList}
    $olasCV.updateTimeWdg enable

    set gvar(ypos) 0
    

    #Check Sybase connectivity
    if {[catch {set sybHandle [sybconnect $gvar(sybaseUsername) $gvar(sybasePassword) $gvar(db_server)]} handle]} {Info "Cannot connect to server, reason: $sybmsg(dberrstr)"
	puts "Cannot connect to server, reason: $sybmsg(dberrstr)"
	exit 
    }
    sybclose $sybHandle
    olasLoop olasControlUpdate
}

proc olasControlUpdate {} {
    global gvar

    if {$gvar(autoUpdate) == 0} { return }
    if {$gvar(expStartTo) != ""} { return }
    olasControlUpdateList
}


#Update the list by querying the database
proc olasControlUpdateList {} {
    global gvar olasCV obArray dpIdArray listOfObs obFlag

    
    if {$gvar(olasControlGo)} { return }
    if {[checkDate $gvar(expStartFrom) $gvar(expStartTo)] == 1} { return }

    set gvar(olasControlGo) 1
    set gvar(queryCancelled) 0
    #$olasCV.olasControlListWdg deleteItem 0 end
    #$olasCV.olasControlListWdg addItem 0 "Querying the database..."
    
    if {[info exists obArray]} {unset obArray}
    if {[info exists dpIdArray]} {unset dpIdArray}
    set listOfObs {}
    set oldObname ""
    set ret ""
    set formatLine "%-6s %-13s %-10s %10s %10.3f %-10s"

    set expStartTo $gvar(expStartTo)
    if {$gvar(expStartTo) == ""} {set expStartTo [clock format [clock seconds] -format "%Y-%m-%d" -gmt 1]}

    olasDisableEntries
    . config -cursor watch
    .olasControl.bot.sh displayText "Querying the database..."
    update; update idletasks
    #This is the select.
    
    set select "select obs_name, tpl_seqno, tpl_expno, prog_id, ra, dec, exptime, object, dp_id
from data_products
where exp_start > '$gvar(expStartFrom)' and exp_start < '$expStartTo 11:59PM'
order by obs_name, tpl_seqno, tpl_expno"
    Debug "Sybase select: $select" 1

    set obNames [Sybsql $select]
    Debug "Result from query = $obNames" 3
    
    if {$obNames == "error" } {
	.olasControl.bot.sh displayText "Database error !"
	olasEnableEntries
	return }
    
    #Extract the information from the query.
    foreach element $obNames {
	update; update idletasks
	set obname   [lindex $element 0]
	if {$obname == "" } { continue }
	set seqno    [lindex $element 1]:[lindex $element 2]
	set prog_id  [lindex $element 3]
	if {[catch {set ra [deg2hour [expr {[lindex $element 4]/3600000.0}]]}]} { set ra "-" }
	if {[catch {set dec [deg2sex [expr {[lindex $element 5]/3600000.0}]]}]} { set dec "-" }
	set exptime  [lindex $element 6]
	set object   [lindex $element 7]
	if {$object == "" } {set object "-"}
	set dp_id    [lindex $element 8]
	Debug "obname   = $obname
               seqno    = $seqno
               prog_id  = $prog_id
               ra       = $ra
               dec      = $dec
               exptime  = $exptime 
               object   = $object
               dp_id    = $dp_id" 1
	
	#Fill in an array with 0 or 1. It is used by the toggle
	#mechanism in order to show the complete select or just the OB
	if {$obname != $oldObname} {
	    lappend listOfObs $obname
	    if {![info exists obFlag($obname)]} {set obFlag($obname) 0}
	    set oldObname $obname
	    Debug "obname = $obname" 1
	}
	
	lappend obArray($obname) "{$seqno} {$prog_id} {$ra} {$dec} {$exptime} {$object} $dp_id"
	if {$gvar(queryCancelled) == 1} { break }
    }
    olasControlShowList
    if {$gvar(autoUpdate) == 0} { olasEnableEntries }
    . config -cursor top_left_arrow
    .olasControl.bot.sh displayText ""
    set gvar(olasControlGo) 0
}    

#Show the complete select or just the OB by reading an array
proc olasControlShowList {} {
   global  gvar olasCV obArray dpIdArray listOfObs obFlag

    $olasCV.olasControlListWdg deleteItem 0 end
    
    set formatLine "%-6s %-13s %-10s %10s %10.3f %-10s"
    set gvar(obListHidden) ""
    set obList ""
    
    foreach ob $listOfObs {
	lappend obList "$ob"
	lappend gvar(obListHidden) "$ob"
	if {$obFlag($ob) == 1} {
	    foreach elem $obArray($ob) {
		lassign $elem seqno prog_id ra dec exptime object dp_id
		lappend gvar(obListHidden) $dp_id
		set string "\t[eval format [concat \"$formatLine\" \"$seqno\" \"$prog_id\" \"$ra\" \"$dec\" \"$exptime\" \"\{$object\}\"]]"
		regsub -all "{" $string "" tmpstring
		regsub -all "}" $tmpstring "" finalstring
 		lappend obList $finalstring
	    }
	}
    }

    Debug "List : $obList" 2
    if {$obList ==  ""} {set obList {{No entries ...}}
	set gvar(entriesFlag) 0
    } else { set gvar(entriesFlag) 1 }
    $olasCV.olasControlListWdg addItemList 0 $obList
    set l $olasCV.olasControlListWdg
    #set num [$l getSize]
    #set pos [expr {$num - 24}]
    [$l component uifFancyListBox] yview $gvar(ypos)
    update idletasks
}

proc olasAUToggle {} {
    global gvar olasCV

    
    if {$gvar(autoUpdate) == 1} {
	set gvar(autoUpdate) 0
	set gvar(tmpToDate) $gvar(expStartTo)
	set gvar(expStartTo) ""
	
	if {[checkDate $gvar(expStartFrom) $gvar(expStartTo)] == 1 \
		|| [regexp  {^[0-9]+$} $gvar(updateTime)] == 0 \
		|| $gvar(updateTime) >999 \
		|| $gvar(updateTime) < 1 } {
	    Info "Auto Update not started.\nCheck From: and the refresh rate"
	    set gvar(expStartTo) $gvar(tmpToDate)
	    $olasCV.expStartToWdg enable
	    $olasCV.expStartFromWdg enable
	    olasAUToggle
	} else {
	    set gvar(autoUpdate) 1
	    olasDisableEntries
	    set gvar(updateTimeRate) [expr {$gvar(updateTime) * 1000}]
	    }
    } else {
	set gvar(expStartTo) $gvar(tmpToDate)
	olasEnableEntries
    }

}

proc olasEnableEntries {} {
    global olasCV
    
    $olasCV.expStartToWdg enable
    $olasCV.expStartFromWdg enable
    $olasCV.updateTimeWdg enable
    $olasCV.olasControlRefreshWdg enable
}

proc olasDisableEntries {} {
    global olasCV
    
    $olasCV.expStartToWdg disable
    $olasCV.expStartFromWdg disable
    $olasCV.updateTimeWdg disable
    $olasCV.olasControlRefreshWdg disable
}


#Toggle mechanism to expand/reduce the view
proc olasControlToggleList {} {
    global olasCV obFlag listOfObs gvar

    
    set selection [string trim [$olasCV.olasControlListWdg getSelection] "{}"]
    set index [$olasCV.olasControlListWdg  getIndex]
    if {$selection == "" || \
	    $index == "" || \
	    $gvar(entriesFlag) == 0 || \
	    $gvar(olasControlGo) == 1} { return }
    
    set gvar(olasControlGo) 1
    . config -cursor watch; update; update idletasks
    .olasControl.bot.sh displayText "Querying the database..."
    update; update idletasks
    set dp_id [lindex $gvar(obListHidden) $index]

    if {[lsearch -exact $listOfObs $selection] != -1} {
	if {$obFlag($selection) == 1} {set obFlag($selection) 0
	} else {set obFlag($selection) 1}
	set gvar(ypos) [[$olasCV.olasControlListWdg component uifFancyListBox] nearest 0]
        olasControlShowList
    } else {
	set query "select ob_id,
tel_id,     ins_id,     prog_id,   dp_class,   dp_cat,   dp_type,
dp_tech,    light_path, exp_start, mjd_obs,    exptime,
tpl_seqno,  tpl_expno,  det_id,    det_expid,  qual_flag,
ra,         dec,        object,    ins_mode,   obs_name,
naxis1,     naxis2,     tel_airm_start,        origfile,
det_chip1id,filter_path,gris_path,             grat_path,
slit_path from data_products where dp_id = '$dp_id'"
	Debug "Query: $query" 1
	
	set dp_info [lindex [Sybsql $query] 0]
	set ob_id [lindex $dp_info      0]
	set tel_id [lindex $dp_info     1]
	set ins_id [lindex $dp_info     2]
	set prog_id [lindex $dp_info    3]
	set dp_class [lindex $dp_info   4]
	set dp_cat [lindex $dp_info     5]
	set dp_type [lindex $dp_info    6]
	set dp_tech [lindex $dp_info    7]
	set light_path [lindex $dp_info 8]
	set exp_start  [lindex $dp_info 9]
	set mjd_obs  [lindex $dp_info   10]
	set exptime  [lindex $dp_info   11]
	set tpl_seqno [lindex $dp_info  12] 
	set tpl_expno [lindex $dp_info  13]  
	set det_id   [lindex $dp_info   14]
	set det_expid  [lindex $dp_info 15]
	set qual_flag  [lindex $dp_info 16]
	if {[catch { set ra [deg2hour [expr {[lindex $dp_info 17]/3600000.0}]]}]} { set ra "-" }
	    #set ra [lindex $dp_info         17]
	if {[catch {set dec  [deg2sex  [expr {[lindex $dp_info 18]/3600000.0}]]}]} {set dec "-"}
	#set dec [lindex $dp_info        18]
	set object  [lindex $dp_info    19]
	set ins_mode  [lindex $dp_info  20]
	set obs_name  [lindex $dp_info  21]
	set naxis1  [lindex $dp_info    22]
	set naxis2  [lindex $dp_info    23]
	set tel_airm_start  [lindex $dp_info 24]
	set origfile   [lindex $dp_info   25]
	set det_chip1id  [lindex $dp_info 26]
	set filter_path [lindex $dp_info  27] 
	set gris_path  [lindex $dp_info   28]
	set grat_path  [lindex $dp_info   29]
	set slit_path  [lindex $dp_info   30]
	
	set text "DP ID      = $dp_id
OB ID      = $ob_id 
OBJECT     = $object
RA         = $ra 
DEC        = $dec 
TEL ID     = $tel_id 
INS ID     = $ins_id 
PROG ID    = $prog_id 
DPR CATG   = $dp_cat 
DPR TYPE   = $dp_type 
DPR TECH   = $dp_tech 
FILTER     = $filter_path 
GRISM      = $gris_path 
GRATING    = $grat_path 
SLIT       = $slit_path
Light path = $light_path 
MJD-OBS    = $mjd_obs 
EXPTIME    = $exptime 
TPL SEQNO  = $tpl_seqno 
TPL EXPNO  = $tpl_expno 
DET ID     = $det_id 
INS Mode   = $ins_mode 
OB Name    = $obs_name 
NAXIS1     = $naxis1 
NAXIS2     = $naxis2 
Airm start = $tel_airm_start 
ORIGFILE   = $origfile 
CHIP1 ID   = $det_chip1id"
	
	Debug  "Table to show:  $text" 2
	
	if {$gvar(olasDetailsFlag) == 0} {panImportPanel olasControlDetails}
	.olasControlDetails.top.wdg.can.olasControlDetailsWdg deleteText 0.0 end
	.olasControlDetails.top.wdg.can.olasControlDetailsWdg insertText 0.0 $text
	    
    }
    set gvar(olasControlGo) 0
    . config -cursor top_left_arrow
    .olasControl.bot.sh displayText ""
    update; update idletasks
}

#Either expand or compress the view
proc viewAll {flag} {
    global listOfObs gvar obFlag

    if {![info exists listOfObs]} { return }

    .olasControl.bot.sh displayText "Show/Hide all..."
    Busy {
	foreach ob $listOfObs {
	    set obFlag($ob) $flag
	}
	olasControlShowList 
    }
}

#Simply close the panel
proc olasControlClose {} {
    global gvar
    
    if {[info exists gvar(logFile)]} { close $gvar(logFile) }
    panClosePanel no .

}

#Parse the arguments from the command line and set some options
proc setOptions {} {
    global gvar

    for { set i 0} {$i <[llength $gvar(args)]} {incr i} {
	switch -- [lindex $gvar(args) $i] {
	    -info {
		incr i
		explainAll}
	    -debug {
		incr i
		set level [lindex $gvar(args) $i]
		if {$level == "" || $level > 3 || $level < 1} {
		    explain
		}
		set gvar(debugMode) $level} 
	    -log {
		incr i
		set logFile [lindex $gvar(args) $i]
		if {[catch {set gvar(logFile) [open $logFile a]} var]} {
		    puts "Cannot open $logFile"
		    exit}
	    } 
	    default {
		explain
	    }
	}
    }
}

#I won't explain explain =;-)
proc explain {} {
    
    puts "Usage: olasControl -info -log <filename> -debug \[1-3\]"
    exit
}



proc explainAll {} {
    puts "-info           : This message"
    puts "-log <filename> : Puts all the messages in the specified file"
    puts "                : If you don't specify -debug the verbose level"
    puts "                : will be set to 1"
    puts "-debug \[1-3\]  : Define the verbose level"
    exit

}



proc olasLoop {  cmd  } {
    global gvar

    if {$gvar(cancelLoop) == 1} {
	set gvar(cancelLoop) 0
	after cancel $gvar(loopId)
	#puts  "$type Loop - $gvar(loopId-$type) killed " 
    } else {
	eval $cmd
	set gvar(loopId) [after $gvar(updateTimeRate) olasLoop \"$cmd\" ]
	#puts "LoopID = $gvar(loopId-$type) -- cmd = $cmd" 
    }
}

proc callDpViewer {} {
    
    panCallPanel dpViewer
}


proc watchCursor {} {
    global gvar

    if {$gvar(olasControlGo)} {
	. config -cursor watch
	update
	update idletasks
    }
    
}

proc arrowCursor {} {
    global gvar

    if {$gvar(olasControlGo)} {
	. config -cursor top_left_arrow
	update
	update idletasks
    }
    
}
#
# ___oOo___
