#Make a select with all the checks.
proc Sybsql {select} {
    global sybmsg gvar sybHandle

    Debug "Connecting to sybase..." 1
    if {[catch {set sybHandle [sybconnect $gvar(sybaseUsername) $gvar(sybasePassword) $gvar(db_server)]} handle]} {Info "Cannot connect to server, reason: $sybmsg(dberrstr)"
	Debug  "Cannot connect to server, reason: $sybmsg(dberrstr)" 1
	exit } 
    Debug "Sybase connected." 1

update idletasks    
    if {[catch {sybsql $sybHandle $select async} handle]} { Debug  "Cannot query/update database, reason: $sybmsg(msgtext); dberrstr = $sybmsg(dberrstr); $sybmsg(oserrstr)" 1
	Debug "The query was: $select" 2
 
	Info "Cannot query/update database, reason: $sybmsg(msgtext); dberrstr = $sybmsg(dberrstr); $sybmsg(oserrstr)"
	#::.astoControl.bot.sh displayText "Error quering database."
	#set gvar(astoStatusFlag) 0
	set resultVar  "error" 
    } else {set resultVar [getResult $sybHandle]}
    
    catch {sybclose $sybHandle}
    Debug "Result of query = $resultVar" 1
    return $resultVar
}

proc Sybcancel {} {
    global sybHandle gvar

    catch {sybcancel $sybHandle}
    catch {sybclose $sybHandle}
    set gvar(queryCancelled) 1
}


#procedure to get results from Sybase
proc getResult {sybHandle} {
    global sybmsg
    
    set timer 0
    if {[catch {set available [sybpoll $sybHandle 1000]}]} { return }
    while {[string length $available] == 0} {
	if {[catch {set available [sybpoll $sybHandle 0]}]} { return }
	update 
	update idletasks
    }
    
    set buffer ""
   if {[catch { set row [sybnext $sybHandle]}]} { return }
    if {$sybmsg(nextrow) == "NO_MORE_RESULTS"} {
	Debug "No more results form query" 3
	return "" }
    while {$sybmsg(nextrow) != "NO_MORE_ROWS"} {
	lappend buffer $row
	if {[catch {set row [sybnext $sybHandle]}]} { return }
	update 
	update idletasks
    }
    return $buffer

}

#Convert seconds in sex degrees.
proc deg2sex {deg} {
    
    if {[string match *-* $deg]} {set sign "-"} else {set sign ""}
    set tmphour [expr {int(abs($deg))}]
    set tmp [format "%2d" $tmphour]
    regsub -all " " $tmp "0" hour
    Debug "sex-Sign = $sign" 3
    Debug "sex-Hour = $hour" 3
    
    set rawminute [expr {[expr {abs($deg) - $tmphour}]*60.0}]
    set tmpminute [expr {int($rawminute)}]
    set tmp [format "%2d" $tmpminute]
    regsub -all " " $tmp "0" minute
    Debug "sex-Minute = $minute" 3

    set rawsecond [expr {[expr {$rawminute - $tmpminute}]*60.0}]
    set tmpsecond [expr {int($rawsecond)}]
    set tmp [format "%2d" $tmpsecond]
    regsub -all " " $tmp "0" second
    Debug "sex-Second = $second" 3
    
    return "$sign$hour $minute $second"
}

#Convert seconds in hour
proc deg2hour {deg} {
    
    if {[string match *-* $deg]} {set sign "-"} else {set sign ""}
    if {[catch {set rawhour [expr {abs($deg/15.0)}]}]} { 
	return "-" }
    set tmphour [expr {int($rawhour)}]
    set tmp [format "%2d" $tmphour]
    regsub -all " " $tmp "0" hour
    Debug "Sign = $sign" 3
    Debug "Hour = $hour" 3

    set rawminute [expr {[expr {$rawhour - $tmphour}]*60.0}]
    set tmpminute [expr {int($rawminute)}]
    set tmp [format "%2d" $tmpminute]
    regsub -all " " $tmp "0" minute
    Debug "Minute = $minute" 3

    set rawsecond [expr {[expr {$rawminute - $tmpminute}]*60.0}]
    set tmpsecond [expr {int($rawsecond)}]
    set tmp [format "%2d" $tmpsecond]
    regsub -all " " $tmp "0" second
    Debug "Second = $second" 3
    
    return "$sign$hour $minute $second"
}

#Test procedure, always useful with tkinspect
proc testHdr {} {

    #Create Header
    set formatLine "%-6s %-10s %-8s %-8s %-10s %-10s"
    set formatHeader "obs_name seq/exp prog_id RA DEC exptime object" 
    set gvar(olasControlHdr) [eval format [concat \"$formatLine\" $formatHeader]]
    
} 

#Debug procedure, it shows internal messages
proc Debug {text level} {
    global gvar
    
    if {![info exists gvar(debugMode)]} { return }
    if { $gvar(debugMode) <  $level } { return }

    if {[info exists gvar(logFile)]} {puts $gvar(logFile) "Debug> $text"
    } else { puts "Debug> $text" }
}

proc checkDate {from to} { 
    global gvar 
    
    set currentDate [clock seconds]
    if {$to != "" && $from == ""} {
	Info "Please fill in the From field."
	return 1}
    
    if {$from != ""} {
	set fromDate [checkDateFormat $from]
	if {$fromDate == 0} {
	    Info "From: date is  not correct"
	    return 1}
	if {$fromDate > $currentDate} { Info "From: is in the
future"
	    return 1}
    }
    
    if {$to != ""} {
	set toDate [checkDateFormat $to]
	if {$toDate == 0} {
	    Info "To: date is not correct"
	    return 1}
	if {$toDate > $currentDate} { Info "To: is in the future"
	    return 1}
	
	if {$fromDate > $toDate} { Info "To: must be after From:"
	    return 1}    
    }
}

proc checkDateFormat {date} {
    global gvar
    
    set splitDate [split $date "-"]
    set year  [lindex $splitDate 0]
    set month [lindex $splitDate 1]
    set day   [lindex $splitDate 2]

    if {[catch {set date [clock scan "$month/$day/$year"]}]} { return 0 
    } else { return $date}
}


#Pop up info boxes.
proc Info {text} {
    #puts "htInfo> $text"
    tk_dialog .info "Olas Control Dialog" $text info 0 OK
    Debug $text 1
}

#Loop routine.
proc Loop { time cmd  } {
  global gvar

    if {$gvar(cancelLoop) == 1} {
	set gvar(cancelLoop) 0
	after cancel $gvar(loopId)
	#puts  "$type Loop - $gvar(loopId-$type) killed " 
    } else {
	eval $cmd
	set gvar(loopId) [after $time Loop $time \"$cmd\" ]
	#puts "LoopID = $gvar(loopId-$type) -- cmd = $cmd" 
    }
}


#-----------------------------------------------------------------------------
# run the given tcl command while displaying the busy cursor in the main window
# 
# Bugs: if $cmd contains a "return", it will not actually return the
# first time Busy is called (something to do with autoloading). After
# the first call, it will work ok. Either avoid return in $cmd, or
# call "Busy {}" first.
#-----------------------------------------------------------------------------
set busy_count 0
proc Busy {cmd} {
    set w .
    global errorInfo busy_count
    if {[incr busy_count] == 1} {
        #       set focus [focus]
        #       focus none
        blt::busy hold $w -cursor watch
        update
        update idletasks
    }
    
    # save any errors and report them later
    if {[set code [catch [list uplevel $cmd] msg]]} {
        set info $errorInfo
    } 

    if {[incr busy_count -1] == 0} {
        blt::busy release $w
        #       catch {focus $focus}
    }

    if {$code} {
        return -code $code -errorinfo $info $msg
    }
}

        
#-----------------------------------------------------------------------------
# Run the tcl command $cmd while displaying the busy cursor in  window $w.
# Also works to block a listbox but allow the scrollbars to 
# still be used. ($W should probably not be '.', use Busy for that).
#
# Note: had problems in Tk3.6 here with watch cursor showing up after
# deiconfiying the window. Try again in Tk4.0 ?
#-----------------------------------------------------------------------------
proc BusyWin {w cmd} {
    global errorInfo busywin_count

    if {! [info exists busywin_count($w)]} {
        set busywin_count($w) 0
    }

    if {[incr busywin_count($w)] == 1} {
        #       set focus [focus]
        #       focus none
        eval blt::busy hold $w -cursor watch
        update idletasks
        update
    }
    
    # save any errors and report them later
    if {[set code [catch [list uplevel $cmd] msg]]} {
        set info $errorInfo
    } 

    if {[incr busywin_count($w) -1] == 0} {
        update idletasks
        blt::busy release $w
        #       catch {focus $focus}
    }

    if {$code} {
        return -code $code -errorinfo $info $msg
    }
}
