/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */
package org.eso.dfslog;


import javax.swing.*;
import java.awt.*;
import java.text.*;
import java.util.*;
import java.awt.event.*;

/**
 * MessageBar:
 * 
 * A JPanel which displays a message, along with the current date and
 * time. The MessageBar provides ok and goto buttons. The ok button clears
 * the message from the MessageBar, the goto button does nothing. It is
 * intended that the a listener will be added to the goto button, to
 * display a view of data appropriate to the current message.
 */


public class MessageBar extends JPanel implements ActionListener {
    private JLabel currentMessage = null;
    private JLabel currentTime = null;
    private JLabel lastUpdateTime = null;
    private Button okButton = null;
    private Button gotoButton = null;
    private javax.swing.Timer flashTimer = null;
    private Color foregroundColor = Color.red;

    private String currentMessageWorkStation = null;
    private String currentMessageComponent = null;
    private long currentMessageId = 0;

    private String currentMessageText = null;

    public MessageBar(ActionListener gotoAL) {
        setLayout(new BorderLayout());
        currentMessage = new JLabel();
        currentMessage.setForeground(java.awt.Color.red);

        JPanel buttonPanel = new JPanel();

        flashTimer = new javax.swing.Timer(1000, this);
        flashTimer.start();

        okButton = new Button("clear");
        okButton.addActionListener(this);
        okButton.setEnabled(false);

        gotoButton = new Button("goto");
        gotoButton.setEnabled(false);
        gotoButton.addActionListener(gotoAL);

        buttonPanel.add(okButton);
        buttonPanel.add(gotoButton);

        currentMessageText = null;
        currentMessageWorkStation = null;
        currentMessageComponent = null;

        GridLayout g = new GridLayout(2, 1);
        g.setHgap(5);
        g.setVgap(5);
        JPanel timePanel = new JPanel(g);
 
        currentTime = new JLabel("", JLabel.RIGHT);
        newTime();
        timePanel.add(currentTime);
        lastUpdateTime = new JLabel("", JLabel.RIGHT);
        newUpdate();
        timePanel.add(lastUpdateTime);
        JPanel northPanel = new JPanel(new BorderLayout());
        northPanel.add("Center",
            new JLabel("Dfs System-wide Error Summary"));
        northPanel.add("East", timePanel);
        add("North", northPanel);
        add("Center", currentMessage);
        add("East", buttonPanel);
    }




    private String getTime() {
        SimpleDateFormat formatter 
            = new SimpleDateFormat("dd MMM yyyy HH:mm:ss");
        return formatter.format(new Date());
    }



    private void newTime() {
        currentTime.setText(getTime());
    }



    public void newUpdate() {
        lastUpdateTime.setText("Last Update: "+getTime());
    }



    public void newMessage(String theMessage, String ws, String comp,
            long mid) {
        currentMessageText = theMessage;
        currentMessage.setText(theMessage);
        okButton.setEnabled(true);
        gotoButton.setEnabled(true);
        foregroundColor = Color.red;
        currentMessageWorkStation = ws;
        currentMessageComponent = comp;
        currentMessageId = mid;
    }




    /**
     * Returns null if there is no current message.
     */

    public String getLastMessage() {
        return currentMessageText;
    }



    public String getMessageWorkStation() {
        return currentMessageWorkStation;
    }




    public String getMessageComponent() {
        return currentMessageComponent;
    }



    public long getMessageId() {
        return currentMessageId;
    }



    /**
     * The setMessageId() sets the currentMessageId member
     * It is used to initialize or clear the member.
     */

    public void setMessageId(long mi) {
        currentMessageId = mi;
    }




    public void setLastMessage(String s) {
        currentMessageText = s;
    }




    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand() == "clear") {
            // clear the message, disable the buttons and stop the timer.
            currentMessage.setText("");
            // do not stop the flashTimer as we need to update the time.
            // flashTimer.stop();
            okButton.setEnabled(false);
            gotoButton.setEnabled(false);
        } else {
            // it is the timer going off.
            // stop the timer while we process the event.
            flashTimer.stop();
            if (foregroundColor == Color.red) {
                foregroundColor = Color.black;
            } else {
                foregroundColor = Color.red;
            }
            currentMessage.setForeground(foregroundColor);
            currentMessage.repaint();
            // update the time.
            newTime();
            // restart the timer.
            flashTimer.restart();
        }
    } // end of actionPerformed().

} // end of class MessageBar.

// end of file MessageBar.java.
