/**
 * @author N. Kornweibel
 * @version 1.0
 * A simple class to encalsulate the generation of file descriptions
 * in the DataModel.
 * For Fits files the list of keywords passed in the constructor are
 * used for the description.
 * For other files no action is taken at this stage.
 */
package org.eso.gasgano.datamodel;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.eso.gasgano.datamodel.gui.DBOBComponent;
import org.eso.gasgano.datamodel.gui.KeywordContainerOBComponent;
import org.eso.gasgano.datamodel.gui.OBComponent;
import org.eso.gasgano.keyword.Keyword;

public class BasicFileDescription implements FileDescription {

    private Hashtable dataColumns = null;
    private int classificationIndex = -1;
    private Vector dataColumnOrder = null;

    /**
     * @param k array of Keywords to be display in description
     */
  public BasicFileDescription(Keyword[] k) {
    dataColumns = new Hashtable();
    dataColumnOrder = new Vector();
    if (k != null) {
      for (int i = 0; i < k.length; i++) {
	if (k[i].getName().equals( Keyword.CLASSIFICATION_METAKEYWORD)) {
	  classificationIndex = i;
	}
	dataColumns.put(k[i].getName(), k[i]);
	dataColumnOrder.addElement(k[i]);
      }
      KeywordContainerOBComponent.addToDefaultKeywordQuery(k);
      DBOBComponent.addToDefaultKeywordQuery(k);
    }
  }

    /**
     * @param o OBComponent to have it description set.
     */
    public void setDescription(OBComponent o) {
        if (o instanceof KeywordContainerOBComponent) {
            KeywordContainerOBComponent obc = (KeywordContainerOBComponent)o;
            Keyword[] cols = KeywordContainerOBComponent.getDefaultKeywordQuery();
            // iterate through the getDefaultKeywordQuery filling in
            // the Keywords in the hashtable with the current values.
            for (int i = 0; i < cols.length; i++) {
                Keyword k = (Keyword)dataColumns.get(cols[i].getName());
                if (k != null) {
                    k.setValue(cols[i].getValue());
                }
            }
            // enumerate through the Vector to set the values in colData
            // in the correct order.
            Enumeration e = dataColumnOrder.elements();
            int i = 0;
            String[] colData = new String[cols.length];
            while (e.hasMoreElements()) {
                Keyword k = (Keyword)e.nextElement();
                if (k.getValue() == null) {
                    colData[i++] = new String("");
                } else {
                    colData[i++] = k.getValue();
                }
            }
            obc.setDisplayData(colData);
        } else {
            // do nothing for now.
        }
    }

    /**
     * @param o OBComponent to have it description set.
     */
    public void setDescription(DBOBComponent obc) {
            Keyword[] cols = DBOBComponent.getDefaultKeywordQuery();
            // iterate through the getDefaultKeywordQuery filling in
            // the Keywords in the hashtable with the current values.
            for (int i = 0; i < cols.length; i++) {
                Keyword k = (Keyword)dataColumns.get(cols[i].getName());
                if (k != null) {
                    k.setValue(cols[i].getValue());
                }
            }
            // enumerate through the Vector to set the values in colData
            // in the correct order.
            Enumeration e = dataColumnOrder.elements();
            int i = 0;
            String[] colData = new String[cols.length];
            while (e.hasMoreElements()) {
                Keyword k = (Keyword)e.nextElement();
                if (k.getValue() == null) {
                    colData[i++] = new String("");
                } else {
                    colData[i++] = k.getValue();
                }
            }
            obc.setDisplayData(colData);
    }

    public void classificationChanged(OBComponent o) {
        if (classificationIndex >= 0 && o instanceof KeywordContainerOBComponent) {
            KeywordContainerOBComponent obc = (KeywordContainerOBComponent)o;
            String[] obcColData = obc.getDisplayData();
            if (obcColData != null) {
                obcColData[classificationIndex] = obc.getClassification();
            }
        }
    }
}
// end of file FileDescription.java.
