/**
 * @author N. Kornweibel
 * @version 1.0
 * A class for defining a FitsFile Observation component.
 */

package org.eso.gasgano.datamodel.gui;

import java.awt.Color;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;

import org.eso.gasgano.datamodel.filesystem.DFSFile;
import org.eso.gasgano.datamodel.filesystem.DFSDataModel;
import org.eso.gasgano.gui.GUIUtils;
import org.eso.gasgano.properties.PropertyDB;

public class FitsFileOBComponent extends KeywordContainerOBComponent {

    private static JLabel rawLabel =
        new JLabel(
            new ImageIcon(Object.class.getResource(GUIUtils.rawGif)),
            (int) JLabel.LEFT_ALIGNMENT);
           
    private static JLabel redLabel =
        new JLabel(
            new ImageIcon(Object.class.getResource(GUIUtils.redGif)),
            (int) JLabel.LEFT_ALIGNMENT);
             
    /**
     * @param address The path of the fits file.
     */
    public FitsFileOBComponent(String address) {
        super(address);
        long startTime = 0;
        if (PropertyDB.getInstance().isDebug(PropertyDB.debugTiming)) {
            startTime = System.currentTimeMillis();
        }
        if (getStatus() == DFSFile.FILE_VALID) {
            // add this OBComponent to the DataModel.
            addToDataModel(
                defaultKeywordQuery[0].getValue(),
                defaultKeywordQuery[1].getValue(),
                myFile.getTelescope());

            if (PropertyDB.getInstance().isDebug(PropertyDB.debugTiming)) {
                initTime += (System.currentTimeMillis() - startTime);
                initNumber++;
            }
        }
    }

    /**
     * @param progId The program id under which this is t be added in the
     * DataModel.
     * @param obId The observation id under which this is to be added in
     * the DataModel.
     * Adding a FitsFileOBComponent into a DataModel - the program and
     * observation should be determined from the Keywords - otherwise
     * they are set to Unknown values.
     */
    public void addToDataModel(
        String progId,
        String obId,
        String telescopeId) {
        DFSDataModel dm = DFSDataModel.getDataModel();
        if (!dm.passFilter(this)) {
            myFile.setStatus(DFSFile.FILE_FILTERED);
            return;
        }
        if (progId == null) {
            progId = DFSDataModel.UNKNOWN_PROGRAM_ID;
        }
        if (obId == null) {
            obId = DFSDataModel.UNKNOWN_OBSERVATION_ID;
        }
        if (telescopeId == null) {
            telescopeId = DFSDataModel.UNKNOWN_TELESCOPE_ID;
        }

        String dir = myFile.getPath().getParent();
        dm.addObComponent(progId, obId, dir, telescopeId, this);
    }

    public JComponent getTreeCellComponent() {
        JLabel cell = null;
        // Use default colors for now.
        Color foreGround = null;
        if (isPipeProduct()) {
            cell = redLabel;
            foreGround = Color.red;
        } else {
            cell = rawLabel;
            foreGround = Color.blue;
        }
        cell.setText(toString());
        cell.setForeground(foreGround);
        cell.setFont(getLabelFont());
        return cell;
    }

    public String getArcfile() {
        if (isPipeProduct()) {
            return new String("");
        } else {
            return arcFile;
        }
    }

    public void classify() {
        myFile.classify(DFSDataModel.getDataModel().getRuleTable());
        if (getDefaultFileDescription() != null) {
            getDefaultFileDescription().classificationChanged(this);
        }
    }

} // end of class FitsFileOBComponent.java

// end of file FitsFileOBComponent.java
