package org.eso.gasgano.datamodel.gui;
/**
 * this component has no "soul" of its own.
 * It either refers to a database line (DBOBComponent)
 * or to a filesystem component (OBComponent).
 * The reference should be determined at instantiation time
 *
 */

import java.awt.Color;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;

import org.eso.gasgano.datamodel.filesystem.DFSDataModel;
import org.eso.gasgano.datamodel.filesystem.FitsFile;
import org.eso.gasgano.gui.GUIUtils;

public class ResultsComponent {

    private DBOBComponent dbComp = null;
    private KeywordContainerOBComponent fileComp = null;
    private String id = null;
    private Object father = null;

    private static JLabel rawLabel =
        new JLabel(
            new ImageIcon(Object.class.getResource(GUIUtils.rawGif)),
            (int) JLabel.LEFT_ALIGNMENT);

    public ResultsComponent(String id) {
        this.id = id;
        fileComp =
            (KeywordContainerOBComponent) DFSDataModel
                .getDataModel()
                .getObComponentByFilesystemBaseName(id + ".fits");
        if (fileComp == null) {
            //System.out.println("Component "+id+" is not loaded in memory\n");
        }
    }

    public void setFather(Object father) {
        this.father = father;
    }

    public Object getFather() {
        return father;
    }

    public String toString() {
        return id;
    }
    public String getId() {
        return toString();
    }

    public JComponent getTreeCellComponent() {
        JLabel cell = null;
        // Use default colors for now.
        Color foreGround = null;
        cell = rawLabel;
        foreGround = GUIUtils.dataBaseColor;
        if (getFather() != null) {
            if (getFather() instanceof ResultsContainer
                && ((ResultsContainer) getFather()).isLabel()) {
                foreGround = new Color(1.0f, 0.5f, 0.1f);
            }
        }
        cell.setText(toString());
        cell.setForeground(foreGround);
        //cell.setFont(getLabelFont());
        return cell;
    }

    public String[] getDisplayData() {
        if (fileComp != null) {
            return fileComp.getDisplayData();
        } else {
            String[] tmpString = { "Not Found", "Not Found", "Not Found" };
            return tmpString;
        }
    }

    /**
     * @return The JComponent for displaying this OB Component.
     */
    public JComponent getDisplayComponent() {
        JComponent rc = null;
        if (fileComp != null) {
            String title = null;
            title = toString() + " " + fileComp.getClassification();
            rc =
                new FitsFileDisplay(
                    title,
                    new FitsTableModel((FitsFile) fileComp.getSourceFile()));
        }
        return rc;
    }

}
