package org.eso.ohs.core.dbb.examples;

// $Id: ExampleBrowser3.java,v 1.4 2005/11/04 10:55:29 ddorigo Exp $

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JFrame;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbCheckBox;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.client.DbbWidget;
import org.eso.ohs.core.dbb.server.DbbSession;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.server.DbbSqlEngineImpl;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;

/**
 * This class demonstrates how to use the addWidget( DbbWidget, int, int )
 * method.
 * 
 * @author amchavan, 16-Apr-2002: Creation
 * @version $Revision: 1.4 $
 */

class ExampleBrowser3 extends JFrame {
  
private static final String rcsid = "$Id: ExampleBrowser3.java,v 1.4 2005/11/04 10:55:29 ddorigo Exp $";

   private static Logger stdlog_ = 
    Logger.getLogger( ExampleBrowser3.class );
 
   /** Associated sql engine (server) */  
    DbbSqlEngine sqle;

    /** Aggregated view (client) */
    DbbView view;

    /** SQL table we are going to browse */
    DbbSqlTable table;
  
    /** Label for the "name" widgets */
    String labName  = "Name";

    /** Label for the "ID" widgets */
    String labID    = "UserID";

    /** Label for the "email" widgets */
    String labEmail = "E-mail";

    /** Label for the "referee_acronym" widgets */
    String labRefAcr = "Ref";

    //----------------------------------------
    // Constructor
    //----------------------------------------
  
    /**
     * Create an object setting all its internal variable. The
     * constructor, invoking the private method buildSqlEngine,
     * creates the associated sql engine (DbbSqlEngineImpl implements
     * the DbbSqlEngine interface) using a direct connection to the
     * database.
     *
     * @param user_id		Database login
     * @param user_password	Database password
     */
    public ExampleBrowser3( String user_id, String user_password ) {
	// We browse the opc63..eso_users table
	//J. table = new DbbSqlTable( "opc63", "", "eso_users" );
	table = new DbbSqlTable( "opc70", "", "eso_users" );
    
	buildSqlEngine( user_id, user_password );		
	view = new DbbView(sqle);
    
	buildColumns();
	buildRows();
	buildSort();
    
	getContentPane().add(view);
    } 

    //----------------------------------------
    // Private Methods
    //----------------------------------------

    /**
     * Set up a direct database connection and build the SQL engine
     * with its default chunks and default joins. 
     *
     * @param user_id		Database login
     * @param user_password	Database password
     */
    private void buildSqlEngine ( String user_id, String user_password ) {
    
	try {
            
	    // Build a DBB session and connecto to the database
	    DbbSession session =
		new DbbSession( "jdbc:sybase:Tds:acdbdev.hq.eso.org:6789",
				"opc70", user_id, user_password );
            
	    // Build an SQL engine for our session
	    sqle = new DbbSqlEngineImpl( session, false );

	    // Default column: user ID
	    DbbSqlChunk def = 
		new DbbSqlChunk( table, "id", labID );

	    sqle.addDefaultColumnsChunk( def );
      
	} catch( Exception e ) { 
	    stdlog_.warn("",e); 
	}
    }


    /** Build the column selection panel. 
     *
     * The user ID column is always selected (see the call to
     * DbbSqlEngine.addDefaultColumnsChunk() in buildSqlEngine()); user
     * can also select from the surname_dl and email columns.  
     */

    private void buildColumns() {

	DbbSqlChunk c;
	DbbCheckBox w;
	DbbSqlOperator op;

	// Panel has three rows of widgets
	DbbPanel panel = new DbbPanel( "Selected Columns", 3 );

	// Add the widgets
	try {
	    op = new DbbSqlOperator( "=" );

	    c = new DbbSqlChunk( table, "surname_dl", labName, op );
	    w = new DbbCheckBox( labName, c );
	    panel.addWidget( w, 0, 0 );

	    c = new DbbSqlChunk( table, "email", labEmail, op );
	    w = new DbbCheckBox( labEmail, c );
	    panel.addWidget( w, 1, 1 );

	    c = new DbbSqlChunk( table, "referee_acronym", labRefAcr, op );
	    w = new DbbCheckBox( labRefAcr, c );
	    panel.addWidget( w, 2, 2 );
	}

	catch( Exception e ) { 
	    stdlog_.warn("",e); 
	}

	/* Add this panel to the view as the column selection panel */
	view.setColumnsPanel( panel, "West" );
    }


    /**
     * Build the rows selection panel.
     *
     * We allow the user to search by (exact) name.
     */
    private void buildRows() {

	DbbSqlChunk c;
	DbbTextField w;
	DbbSqlOperator op;

	// Panel has one row of widgets
	DbbPanel panel = new DbbPanel("Selection Criteria", 1);

	// Add the widget
	try {
	    op = new DbbSqlOperator( "LIKE" );

	    c = new DbbSqlChunk( table, "surname_dl", labName, op );
	    w = new DbbTextField( labName , c );
	    panel.addWidget( w );

	    c = new DbbSqlChunk( table, "referee_acronym", labRefAcr, op );
	    w = new DbbTextField( labRefAcr , c );
	    panel.addWidget( w );
	} 
	catch( Exception e ) { 
	    stdlog_.warn("",e); 
	}

	/* Add this panel to the view as the rows selection panel; that
	 * is, the panel to define the where clause 
	 */
	view.setWherePanel( panel, "West" );
    }



    /**
     * Build the row sorting selection panel.
     *
     * We allow the user to sort by ID and name
     */
    private void buildSort() {

	DbbSqlChunk c;
	DbbWidget w;

	// Panel has one rows of widgets
	DbbPanel panel = new DbbPanel("Sorting Criteria", 1);

	try {

	    c = new DbbSqlChunk( table, "id", labID );
	    w = new DbbArrowButton( labID, c );
	    panel.addWidget(w);

	    c = new DbbSqlChunk( table, "surname_dl", labName );
	    w = new DbbArrowButton( labName, c);
	    panel.addWidget(w);
	} 

	catch( Exception e ) { 
	    stdlog_.warn("",e); 
	}

	/* Add this panel to the view as the rows sorting panel; that
	 * is, the panel to define the where clause 
	 */
	view.setSortPanel( panel, "West" );
    }

    //----------------------------------------
    // Public Methods
    //----------------------------------------

    /** Main method, to run directly the example. */
    public static void main(String[] args) {

		int len = args.length;
		String msg = "Usage: ExampleBrowser3 db_login db_password";
		if (len < 2) {
			System.err.println(msg);
			System.exit(1);
		}

		ExampleBrowser3 dbb = new ExampleBrowser3(args[0], args[1]);
		dbb.setTitle("Example 3");

		WindowListener l = new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		};

		dbb.addWindowListener(l);
		dbb.pack();
		dbb.setVisible(true);
	}
}
