/* Created on Dec 9, 2003 */

package org.eso.ohs.core.dbb.htmlclient;

import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.sql.DbbInvalidSqlOperatorType;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;

/**
 * TODO: javadoc
 * 
 * @author amchavan
 * @version $Revision: 1.2 $
 */

// $Id: DbbHtmlWidget.java,v 1.2 2004/04/13 14:46:00 ddorigo Exp $

public abstract class DbbHtmlWidget {
	protected static Logger stdlog_ = Logger.getLogger(DbbHtmlWidget.class);

	/** Number of spaces representing an indentation level */
	public static final int INDENTSPACES = 2;

	/** A long sequence of blanks */
	private static final String SPACES =
			  "                              "
			+ "                              "
			+ "                              "
			+ "                              ";
    
    /** Counter: gets incremented every time a widget is created */
    private static int widgetCount_ = -1; 
     
	/** Default value to be displayed by the widget. */
	protected String default_;

	/** Label of the widget */
	protected String label_ = "";

    /** Unique ID of this widget */
    private String name_ = ""; 

    public static final String SORT_NO   = "none";
    public static final String SORT_ASC  = "ascending";
    public static final String SORT_DESC = "descending";
    
    protected static DbbSqlOperator OP_ASC    = null;
    protected static DbbSqlOperator OP_DESC   = null;
    protected static DbbSqlOperator OP_EQUALS = null;
    static {
        try {
            OP_ASC    = new DbbSqlOperator( SORT_ASC );    
            OP_DESC   = new DbbSqlOperator( SORT_DESC );    
            OP_EQUALS = new DbbSqlOperator( "=" );    
        }
        catch( DbbInvalidSqlOperatorType e ) {
            // should never happen
            e.printStackTrace();
        }
    }
	/**
	 * List of additional sql chunks defined in order to specify more deeply
	 * the sql clauses.
	 */
	protected Vector sqlAdditionalChunks_ = new Vector();

	/** SQL chunk whose values the user can define. */
	protected DbbSqlChunk sqlChunk_;

	/** TODO: javadoc */
	private String toolTipText;

	/** Value of the widget */
	protected String value_ = null;

	/**
	 * Create a DbbWidget object.
	 * 
	 * @param labeltext
	 *            Label text to be displayed.
	 * @param chunk
	 *            DbbSqlChunk object.
	 *  
	 */
	public DbbHtmlWidget(String labeltext, DbbSqlChunk chunk) {

		sqlChunk_ = chunk;
		label_    = labeltext;
        name_     = "DbbHtmlWidget" + ++widgetCount_;
	}

	/**
	 * @return The current text (or the value) displayed by the widget.
	 */
	public String get() {
		if (value_ == null)
			return default_;
		return value_;
	}

	/**
	 * @return The widget label.
	 */
	public String getLabel() {
		return label_;
	}
    
    /**
     * @return A unique string, to be used as the value to be given to
     *         the <code>name</code> attribute of an HTML form's
     *         <code>&lt;input&gt;</code> control.
     */
    protected String getName() {
        return name_;
    }

	/** @return The tooltip text defined for the widget */
	public String getToolTipText() {
		return toolTipText;
	}

	/**
	 * Generate a fragment of HTML code rendering this widget. The fragment is
	 * suitable to be included in the HTML rendering of a DbbHtmlPanel (an HTML
	 * table).
	 * <BR>
	 * The generated fragment is not indented.
	 * 
	 * @return An HTML fragment rendering this widget.
	 */
	public String render() {
		return render( 0 );
	}

	/**
	 * Generate a fragment of HTML code rendering this widget. The fragment is
	 * suitable to be included in the HTML rendering of a DbbHtmlPanel (an HTML
	 * table).
	 * 
	 * @param indentLevel
	 *            The level of indentation for the HTML fragment. Each level is
	 *            indented by INDENTSPACES spaces.
	 * 
	 * @return An HTML fragment rendering this widget.
	 * 
	 * @see #INDENTSPACES
	 */
	public abstract String render(int indentLevel);

	/**
	 * Reset the widget to its default state.
	 */
	public void reset() {
		set(default_);
	}

	/**
	 * Set the value displayed in the interactive widget.
	 * 
	 * @param value
	 *            String to be displayed as new value.
	 */
	public abstract void set( String value );

	/**
	 * Set the default value to be displayed in the interactive widget.
	 * 
	 * @param value   New value default (given as String).
	 */
	public void setDefault(String value) {
		default_ = value;
	}

	/**
	 * Set a specific tooltip for the widget.
	 * 
	 * @param s
	 *            Tooltip text
	 */
	public void setToolTipText(String s) {
		toolTipText = s;
	}
	
	/**
	 * @return A string of blanks corresponding to the input
	 *         indentation level.
	 */
	protected static String indentSpaces( int indentLevel ) {
		return SPACES.substring( 0, indentLevel * INDENTSPACES );		
	}

    /**
     * @return The SQL chunk associated to this widget.
     */
    public DbbSqlChunk getSqlChunk() {
        return sqlChunk_;
    }

}
