package org.eso.ohs.core.dbb.sql;

// $Id: DbbSqlChunkQueryMember.java,v 1.1 2004/02/17 11:16:24 tcanavan Exp $



/** 
 * This class defines a configurable SQL "chunk", that is, an object
 * which can be used to generate a fragment of an SQL SELECT
 * statement.
 * 
 * @author A M Chavan, ESO 26-Jul-99
 * <br>ChangeLog: Carlo Boarotto, ESO 01-Sep-1999
 */
public class DbbSqlChunkQueryMember extends DbbSqlChunkGroupMember {
    boolean valueImmutable_ = false;
    //--------------------------------------------------
    // Constructors
    //--------------------------------------------------

    /**
     * Create an SQL chunk setting the table, the column, the column
     * label, data type and the SQL operator.<br> This constructor
     * should be used mostly for row selection chunks.
     * 
     * @param table	Database table.
     * @param column	Database column name.
     * @param label	Column label.
     * @param op		SQL default operator.  
     * @param data_type	Database column type (see DbbSqlChunk.datatype_).
     */
    public DbbSqlChunkQueryMember(DbbSqlTable table,
		       String column,
		       String label,
		       DbbSqlOperator op,
		       Class data_type) {
	super(table, column, label, op, data_type);
    }


    /** 
     * Create an SQL chunk setting the table, the column, data type and
     * the SQL operator.<br> This constructor should be used mostly for
     * row selection chunks.
     * 
     * @param table	Database table.
     * @param column	Database column name.
     * @param op		SQL default operator.  
     * @param data_type	Database column type (see DbbSqlChunk.datatype_).
     */
    public DbbSqlChunkQueryMember(DbbSqlTable table,
		       String column,
		       DbbSqlOperator op,
		       Class data_type) {    
	super(table, column, op, data_type);
    }
  

    /**
     * Create an SQL chunk setting the table, the column and the
     * database column type.<br> This constructor should be used mostly
     * for column selection chunks.
     * 
     * @param table	Database table.
     * @param column	Database column name.  
     * @param data_type	Database column type (see DbbSqlChunk.datatype_).
     */
    public DbbSqlChunkQueryMember(DbbSqlTable table,
		       String column,
		       Class data_type) {
	super(table, column, data_type);
    }


    /**
     * Create an SQL chunk setting the table, the column and the
     * database column type.<br> This constructor should be used mostly
     * for column selection chunks.
     * 
     * @param table	Database table.
     * @param column	Database column name.  
     * @param label	Column label.  
     * @param data_type	Database column type (see DbbSqlChunk.datatype_). The 
     * widget in which the chunk is placed, passes to the sql chunk its value,
     * the value.toString is then used by the datatype.
     */
    public DbbSqlChunkQueryMember(DbbSqlTable table,
		       String column,
		       String label,
		       Class data_type) {
	super(table, column, label, data_type);
    }


    /** 
     * Create an SQL chunk setting the table, the column and the SQL
     * operator.<br> This constructor should be used mostly for row
     * selection chunks.
     * 
     * @param table	Database table.
     * @param column	Database column name.
     * @param op		SQL default operator.  
     */
    public DbbSqlChunkQueryMember(DbbSqlTable table,
		       String column,
		       DbbSqlOperator op) {
    
	super(table, column, op);
    }
  

    /** 
     * Create an SQL chunk setting the table, the column and the SQL
     * operator.<br> This constructor should be used mostly for row
     * selection chunks.
     * 
     * @param table	Database table.
     * @param column	Database column name.
     * @param label	Column label.  
     * @param op		SQL default operator. 
     *
     * <br>ChangeLog: A M Chavan, 18-Feb-2000
     */
    public DbbSqlChunkQueryMember(DbbSqlTable table,
		       String column,
		       String label,
		       DbbSqlOperator op) {
    
	super(table, column, label, op);
    }
  

    /**
     * Create an SQL chunk setting the table, the column and the
     * label.<br> This constructor should be used mostly for column
     * selection chunks.
     * 
     * @param table	Database table.
     * @param column	Database column name.
     * @param label	Column label.  
     */
    public DbbSqlChunkQueryMember(DbbSqlTable table,
		       String column,
		       String label) {

	super(table, column, label);
    }


    /**
     * Create an SQL chunk setting the table and the column.<br> This
     * constructor should be used mostly for column selection chunks.
     * 
     * @param table	Database table.
     * @param column	Database column name.
     */
    public DbbSqlChunkQueryMember(DbbSqlTable table,
		       String column) {

	super(table, column);
    }


    //--------------------------------------------------
    // Public class methods
    //--------------------------------------------------



  /**
     * Set chunck value immutable.
     *
     * @return String containing the sql clause.
     */

    public void setValueImmutable(boolean value) {
	valueImmutable_ = value;
    }

  /**
     * get chunck value immutable. This when I don't want the widget 
     * value alter my default set up.
     *
     * @return String containing the sql clause.
     */

    public boolean getValueImmutable() {
	return valueImmutable_ ;
    }



    /**
     * Set the value only if not immutable.
     *
     * @param value New value.  
     */
    public void setValue(String value) {  
	if (!getValueImmutable()) {
	    String strExpr = null;
	
	    if (isActive() && value != null ) {
		strExpr = " ("+value+")";
		value_ = strExpr;
	    }
	} 
    }


    //--------------------------------------------------
    // Protected class methods
    //--------------------------------------------------
}
