package org.eso.ohs.core.docview.examples;

import java.awt.Container;
import java.awt.GridLayout;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JTextField;

import org.eso.ohs.core.docview.datatrans.DocViewObject;
import org.eso.ohs.core.docview.gui.DocView;

/**
 * A simple demo showing two GUI components: a JTextField and a JSlider.
 * Each component is wrapped in a DocView, which updates them from the
 * "IntProperty" property of an instance of TestObject.  Updates to the
 * value in each GUI component are likewise stored in the TestObject
 * instance when the component loses focus and thus (indirectly) passed
 * to the other GUI component.
 * 
 * <p>
 * To execute this example, type
 * <blockquote>
 * java org.eso.ohs.docview.datatrans.examples.Easy
 * </blockquote>
 * <p>
 * $Id: Easy.java,v 1.1 2004/02/17 12:38:00 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */
public class Easy extends JFrame {

    /** 
     * Construct a window containing two GUI components: JTextField and JSlider.
     * Each component is linked through DocView to the "IntProperty" property
     * of an TestObject instance.
     */
    public Easy() {
	Container content = getContentPane();

	content.setLayout(new GridLayout(0, 1));
	setTitle("Easy DocView Demo");

		// Create DocView instances wrapped around
		//  the JTextField and JSlider components
	DocView textf = new DocView(new JTextField());
	DocView slider = new DocView(new JSlider());

		// Add the DocView instances to the panel.
		// DocView is a subclass of JPanel and contains
		//  the GUI components.
	content.add(new JLabel("Will update when focus lost"));
	content.add(textf);
	content.add(new JLabel(" "));		// spacer
	content.add(slider);

	TestObject currObj = new TestObject();

		// Link the DocView object (and GUI component)
		//  with the TextObject's property.
	textf.setObjectProperty(currObj, "IntProperty");
	slider.setObjectProperty(currObj, "IntProperty");
    }


    /**
     * Entry point to the program.
     * Creates objects and starts windows.
     * 
     * @param args	Command-line arguments -- ignored
     */
    public static void main(String[] args) {
	Easy win = new Easy();

	win.pack();
	win.setLocation(770, 90);
	win.setVisible(true);
    }


    /**
     * Test object implementing DocViewObject interface.
     * Note that addPropertyChangeListener() and
     * removePropertyChangeListener() are required by the
     * DocViewObject.
     */
    public static class TestObject implements DocViewObject {

	private int intProperty_;

	private PropertyChangeSupport support_
			= new PropertyChangeSupport(this);


	public void setIntProperty(int value) {
	    intProperty_ = value;
	    support_.firePropertyChange(null, null, null);
	}

	public int getIntProperty() {
	    return intProperty_;
	}


	/**
	 * Adds a new listener to be notified of changes.
	 * 
	 * @param lnr the listener
	 */
	public void addPropertyChangeListener(PropertyChangeListener lnr) {
	    support_.addPropertyChangeListener(lnr);
	}

	/**
	 * Removes a new listener, which will no longer be notified of changes.
	 * 
	 * @param lnr the listener
	 */
	public void removePropertyChangeListener(PropertyChangeListener lnr) {
	    support_.removePropertyChangeListener(lnr);
	}

    }

}

