package org.eso.ohs.core.gui.models;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;

/**
 * A Document model to only permit the input of a right ascension value.
 * 
 * <p>
 * $Id: RightAscensionModel.java,v 1.1 2004/02/17 10:35:21 tcanavan Exp $
 * @author cmavros 29-10-01 Add insertString()
 * <br>ChangeLog: cmavros 05/03/02 Correct the max value for ra 
 */

public class RightAscensionModel extends InputMaskModel {

    private final float RA_MAX = 24;

    /**
     * The regular expression describing legal buffer values.
     * Identical to the equivalent in DeclinationModel,
     * except that these values <em>must</em> be positive,
     * i.e.\ no optional sign.
     */
    private final static String legalValuesPattern_ =
    "\\s*(\\d{0,2}(:\\d{0,2}(:\\d{0,2}(\\.\\d{0,3})?)?)?)?";

    /**
     * Constructor for the model.
     */
    public RightAscensionModel() {
	super(legalValuesPattern_);
    }

    /**
     * Refuse input that is above or below the limits of the
     * textfield.
     */
    public void insertString(int offset, String str, AttributeSet a)
      throws BadLocationException {

        //stdlog_.debug("String: <"+str+">");
        super.insertString(offset, str, a);
        //stdlog_.debug("buffer: <"+buffer_+">");
        try {
            String buffer_val = buffer_.toString();

            //Do not accept a single :
            if (buffer_val.equals(":")){
                remove(offset, str.length());
            }
            else{
            int len = buffer_val.length();
                if (len > 1){
                    Character x = new Character(buffer_val.charAt(len-2));
                    Character xend = new Character(':');
                    if (x.equals(xend)){
                        //Do not accept a . after a :
                        if(str.equals("."))
                            remove(offset, str.length());

                        //Do not accept a : after a :
                        if(str.equals(":"))
                            remove(offset, str.length());

                        //Check that mins and secs are less than 60
                        if(Integer.parseInt(str) > 5 ){
                            remove(offset, str.length());
                        }
                    }
                    else{
                        Float f = Float.valueOf(buffer_val);
                        if(f.floatValue() >= RA_MAX) {
                            remove(offset, str.length());
                        }
                    }
                }
            }

        } catch (NumberFormatException e) {
            // nothing, the string is not complete...
        }
    }

}
