package org.eso.ohs.core.gui.widgets;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.border.Border;


/**
 * A dialog box containing a progress bar and a Stop button.
 * Methods are provided for incrementing the progress so far and for
 * querying whether the Stop button has been pressed.
 * 
 * <p>
 * It might be useful to allow listeners for the Stop button
 * but that is not currently implemented.
 * 
 * <p>
 * $Id: StoppableProgressDialog.java,v 1.1 2004/02/17 10:35:21 tcanavan Exp $
 */

public class StoppableProgressDialog extends JDialog {

    private JProgressBar progressBar_;
    private JButton stopButton_;
    private StopListener stopListener_;

    public StoppableProgressDialog(Frame frm, String msg, int min, int max) {
	super(frm);

	JPanel content = (JPanel)getContentPane();

	content.setLayout(new BorderLayout());

	content.add(new JLabel(msg), BorderLayout.NORTH);

	progressBar_ = new JProgressBar(min, max);
	progressBar_.setForeground(Color.blue.darker());
	progressBar_.setStringPainted(true);
	content.add(progressBar_, BorderLayout.CENTER);

	stopButton_ = new JButton("Stop");

	Box buttonPanel = Box.createHorizontalBox();

	buttonPanel.add(Box.createHorizontalGlue());
	buttonPanel.add(stopButton_);
	buttonPanel.add(Box.createHorizontalGlue());

	content.add(buttonPanel, BorderLayout.SOUTH);

	stopListener_ = new StopListener();
	stopButton_.addActionListener(stopListener_);

	content.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

	Border bdr = BorderFactory.createEtchedBorder();
	progressBar_.setBorder(
	    BorderFactory.createCompoundBorder(
		BorderFactory.createEmptyBorder(10, 10, 20, 10),
		bdr
	    )
	);

	pack();

	Point pt = frm.getLocation();
	Dimension sz = frm.getSize();
	pt.translate(sz.width/2, sz.height/2);
	setLocation(pt);
    }

    public int getValue() {
	return progressBar_.getValue();
    }

    public void setValue(int n) {
	progressBar_.setValue(n);
    }

    public void incrementValue(int n) {
	setValue(n+getValue());
    }

    public void setMaximum(int newMax) {
	progressBar_.setMaximum(newMax);
    }

    public boolean shouldStop() {
	return stopListener_.shouldStop();
    }


    private class StopListener implements ActionListener {
	private boolean stopPressed_ = false;

	public void actionPerformed(ActionEvent ev) {
	    stopPressed_ = true;
	}

	public boolean shouldStop() {
	    return stopPressed_;
	}
    }

}

