package org.eso.ohs.core.utilities;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * Utility methods to aid debugging.
 *
 * <p>
 * $Id: DebugUtils.java,v 1.2 2004/04/08 12:09:51 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */

public class DebugUtils {

    private static Logger stdlog_ = 
       Logger.getLogger( DebugUtils.class );

    public static void stackTrace() {
       stackTrace("@@@@@@@");
    }

    public static void stackTrace(String title) {
       new Exception(title).printStackTrace();
    }

    public static String printFields(Object obj) {
       return printFields(obj, propertyNames(obj));
    }

    public static String printFields(Object obj, String[] propNames) {
        StringBuffer rc = new StringBuffer();
        Class cls = obj.getClass();
    
        rc.append("\n");
    
        try {
            for (int j=0; j < propNames.length; j++) {
                Method get = cls.getMethod(
                            "get" + propNames[j],
                            new Class[] { }
                           );
                Object result = get.invoke(obj, new Object[] {});
                if (result instanceof Object[]) {
                    rc.append(
                        propNames[j]
                        + "--> array of size "
                        + ((Object[])result).length
                    +"\n");
                }
                else {
                    rc.append(propNames[j] + "--> " + result+"\n");
                }
            }
        }
        catch (IllegalAccessException e) {
            rc.append(e.getMessage()+"\n");
        }
        catch (InvocationTargetException e) {
            rc.append(e.getTargetException().getMessage()+"\n");
        }
        catch (NoSuchMethodException e) {
            rc.append(e.getMessage()+"\n");
        }
    
        rc.append("\n");
        return rc.toString();
    }


    public static String[] propertyNames(Object obj) {
        return propertyNames(obj.getClass());
    }

    public static String[] propertyNames(Class cls) {
        Method[] methods = cls.getMethods();
        Vector names = new Vector();
    
        for (int j=0; j < methods.length; j++) {
            String methName = methods[j].getName();
            if (methName.startsWith("get")) {
            names.addElement(methName.substring(3));
            }
        }
        String[] result = new String[names.size()];
        names.copyInto(result);
    
        return result;
    }

    public static void printPropertyNames(Object obj) {
        StringBuffer rc = new StringBuffer(printFields(obj));

        String[] propNames = propertyNames(obj);
        rc.append("\nProperty names:\n");
        for (int j=0; j < propNames.length; j++) {
            rc.append("--> " + propNames[j]+"\n");
        }
        rc.append("\n");
        stdlog_.debug(rc.toString());
    }

}

