package org.eso.ohs.dfs;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;

import org.eso.ohs.core.dbb.client.DbbDataType;

public class CycleSteps implements DbbDataType {

	/**
     * 
     */
    private static final long serialVersionUID = 1L;

    public CycleSteps() {
		createHashTables();
	}

	/**
	 *  
	 */
	private void createHashTables() {
		//Create the hash table and fill in the values:
		stepsDescriptionTable = new Hashtable();
		stepsDescriptionTable.put(PROPOSAL_SUBMISSION_INT, PROPOSAL_SUBMISSION);
		stepsDescriptionTable.put(DISTRIBUTE_TO_REF_INT, DISTRIBUTE_TO_REF);
		stepsDescriptionTable.put(REPORT_CARDS_INT, REPORT_CARDS);
		stepsDescriptionTable.put(PRELIMINARY_CLASSIFICATION_INT,
				PRELIMINARY_CLASSIFICATION);
		stepsDescriptionTable.put(REVIEW_CLASSIFICATION_INT,
				REVIEW_CLASSIFICATION);
		stepsDescriptionTable.put(PANEL_OPC_MEET_INT, PANEL_OPC_MEET);
		stepsDescriptionTable.put(COMMENT_CARDS_INT, COMMENT_CARDS);
		stepsDescriptionTable.put(LONG_TERM_SCHED_INT, LONG_TERM_SCHED);
		stepsDescriptionTable.put(SCHED_PREP_INT, SCHED_PREP);
		stepsDescriptionTable.put(INACTIVE_INT, INACTIVE);
		stepsDescriptionTable.put(TEST_INT, TEST);

		stepsIdTable = new Hashtable();
		stepsIdTable.put(PROPOSAL_SUBMISSION, PROPOSAL_SUBMISSION_INT);
		stepsIdTable.put(DISTRIBUTE_TO_REF, DISTRIBUTE_TO_REF_INT);
		stepsIdTable.put(REPORT_CARDS, REPORT_CARDS_INT);
		stepsIdTable.put(PRELIMINARY_CLASSIFICATION,
				PRELIMINARY_CLASSIFICATION_INT);
		stepsIdTable.put(REVIEW_CLASSIFICATION, REVIEW_CLASSIFICATION_INT);
		stepsIdTable.put(PANEL_OPC_MEET, PANEL_OPC_MEET_INT);
		stepsIdTable.put(COMMENT_CARDS, COMMENT_CARDS_INT);
		stepsIdTable.put(LONG_TERM_SCHED, LONG_TERM_SCHED_INT);
		stepsIdTable.put(SCHED_PREP, SCHED_PREP_INT);
		stepsIdTable.put(INACTIVE, INACTIVE_INT);
		stepsIdTable.put(TEST, TEST_INT);

	}

	//Until we don't cleanup and define with visas the list of steps and their
	// meaning,
	//it is better to hardcode them in this class.
    //TODO Refactor this class as soon as we decide the meaning and the content of the steps table.
	public static final Integer PROPOSAL_SUBMISSION_INT = new Integer(1);

	public static final Integer DISTRIBUTE_TO_REF_INT = new Integer(2);

	public static final Integer REPORT_CARDS_INT = new Integer(3);

	public static final Integer PRELIMINARY_CLASSIFICATION_INT = new Integer(4);

	public static final Integer REVIEW_CLASSIFICATION_INT = new Integer(5);

	public static final Integer PANEL_OPC_MEET_INT = new Integer(6);

	public static final Integer COMMENT_CARDS_INT = new Integer(7);

	public static final Integer LONG_TERM_SCHED_INT = new Integer(8);

	public static final Integer SCHED_PREP_INT = new Integer(9);

	public static final Integer INACTIVE_INT = new Integer(10);

	public static final Integer TEST_INT = new Integer(103);

	public static final String PROPOSAL_SUBMISSION = "Proposal Submission";

	public static final String DISTRIBUTE_TO_REF = "Referee Distribution";

	public static final String REPORT_CARDS = "Report Cards";

	public static final String PRELIMINARY_CLASSIFICATION = "Preliminary Classification";

	public static final String REVIEW_CLASSIFICATION = "Review Classification";

	public static final String PANEL_OPC_MEET = "Panel Meetings";

	public static final String COMMENT_CARDS = "Comment Cards";

	public static final String LONG_TERM_SCHED = "Long Term Scheduling";

	public static final String SCHED_PREP = "Scheduling";

	public static final String INACTIVE = "Cycle Closed";

	public static final String TEST = "Test Cycle";

	private final static String DEFAULT_ = "-----------------------";

	private Hashtable stepsIdTable = null;

	private Hashtable stepsDescriptionTable = null;

	public int getDataTypeLength() {
		return DEFAULT_.length();
	}

	public Class getDataType() {
		return String.class;
	}

	public Class getDatabaseType() {
		return Integer.class;
	}

	public String fromDatabaseData(Object value) {
		String stepDescription = (String) stepsDescriptionTable.get(value);
		if (stepDescription != null) {
			return stepDescription;
		} else {
			return "No step id available";
		}

	}

	public Object toDatabaseData(String value) {

		Integer stepId = (Integer) stepsIdTable.get(value);
		if (stepId != null) {
			return stepId;
		} else {
			return new Integer(0);
		}
	}

	public String[] getStepDescriptions() {

		//Let's hide the datastructure with this method, giving back all the
		// steps descriptions.
		Collection descCollection = stepsDescriptionTable.values();
		int i = descCollection.size();
		String[] descriptions = new String[i];
		Iterator iterator = descCollection.iterator();
		while (iterator.hasNext()) {
			i--;
			String nextValue = (String) iterator.next();
			//System.out.println("Adding value to description array: "
			//		+ nextValue);
			descriptions[i] = nextValue;

		}
		return descriptions;
	}

	public Integer[] getStepIds() {

		//Let's hide the datastructure with this method, giving back all the
		// steps descriptions.
		Collection idCollection = stepsIdTable.values();
		int i = idCollection.size();
		Integer[] ids = new Integer[i];
		Iterator iterator = idCollection.iterator();
		while (iterator.hasNext()) {
			i--;
			Integer nextValue = (Integer) iterator.next();
			ids[i] = nextValue;

		}
		return ids;
	}
}