package org.eso.ohs.instruments;


import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Paramfile;


/**
 * This is a parameter class which stores a complete
 * file in zipped format in Local Storage
 * <p>
 *
 * $Id: ParamFileParameter.java,v 1.20 2004/04/08 12:09:52 tcanavan Exp $
 *
 * @author Tim Canavan
 */

public class ParamFileParameter extends FileParameter {

    static final long serialVersionUID = -1919846079097974958L;

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( ParamFileParameter.class );



    /**
     * Filename of the file to be saved
     */

    private String fileName_;

    /**
     * An array of compress bytes which represent the file
     */


    private byte [] compressedValue_ = {};


    /**
     * This is the length of the actual file uncompressed
     */

    private int uncompressedLength_;
 
    /**
     * Constructor
     * @param parameterName name of parameter
     */

    public ParamFileParameter(ParamMetaData pMeta) {
        super(pMeta);
        setValue(super.getValue());       
    }


    public boolean displayValueDifferent() {
      return true;
    }

    public char getDBTypeCode() {
        return 'p';
    }
   

    /**
     * The setValue method takes the full contents of the 
     * file and then compresses it and stores it in the compressValue_
     * array.
     * Note : It does not set the base class String value as in the 
     *        other parameters. This had to be done because Serialization
     *        does not support strings greater than 64K
     * @param value Contents of the file
     */

    public void setValue(String value) {
        if(value == null || value.equals("NODEFAULT"))
            return;
        stdlog_.debug("@@@@ Actual value length " + value.length());      
        ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
        try {
           GZIPOutputStream stream = new GZIPOutputStream(byteStream);
           byte [] copyBuff = value.getBytes();
           stream.write(copyBuff,0,copyBuff.length);
           stream.close();
           uncompressedLength_ = copyBuff.length;
           compressedValue_ = byteStream.toByteArray();
           fileName_ = getFilenameValue(value);
        }
        catch(IOException e) {
           e.printStackTrace();
        }
    }


    /**
     * Return the Value in uncompreesed form
     * @return uncompressed value (contents of the original file)
     */

    public String getValue() {
        if(fileName_ == null || fileName_.equals("NODEFAULT"))
            return "NODEFAULT";
        try {
            ByteArrayInputStream byteStream = new ByteArrayInputStream(compressedValue_);
            GZIPInputStream stream = new GZIPInputStream(byteStream,uncompressedLength_);
            byte [] destArray = new byte[uncompressedLength_];
            int numbytesRead =  stream.read(destArray,0,destArray.length);
            //stream.read(destArray,0,uncompressedLength_);
            stream.close();
            return new String(destArray);
        }
        catch(IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Return the nae of the file from the uncompressed value
     * This is stored in PAF.NAME in the original file
     * @return filename
     */


    public String getFilenameFromValue() {         
         String value = getValue();
         stdlog_.debug("Value = " + value);
         String filename = "NODEFAULT";
         if(value == null || value.length() <= 0)
             return filename;
	 int index = value.indexOf("PAF.NAME");
	 if (index >= 0) {
	     String nameStr = value.substring(index);
	     int start = nameStr.indexOf('"');
	     int end;
	     if (start >= 0) {
	         end = nameStr.indexOf('"', start+1);
		 filename = nameStr.substring(start+1, end);
	     }
	  }
	  return filename;
    }

    
    /**
     * Return the nae of the file from the uncompressed value
     * This is stored in PAF.NAME in the original file
     * @return filename
     */

    public static String getFilenameValue(String value) {
         String filename = null;
         if(value == null || value.length() <= 0)
             return filename;
	 int index = value.indexOf("PAF.NAME");
	 if (index >= 0) {
	     String nameStr = value.substring(index);
	     int start = nameStr.indexOf('"');
	     int end;
	     if (start >= 0) {
	         end = nameStr.indexOf('"', start+1);
		 filename = nameStr.substring(start+1, end);
	     }
	  }
	  return filename;
  
    }

    /**
     * Verify that the filename is valid
     * @param s filename 
     */
    
    public boolean verifyValue(String s) {
         if(s == null || s.equals("NODEFAULT") || s.length() <=0)
             return false;
         return true;
    }

    public boolean verify() {
	return verifyValue(getFilename());
    }
   

    public String getDisplayValue() { 
        //org.eso.ohs.utilities.DebugUtils.stackTrace();
        if(fileName_ == null) {
            return "NODEFAULT";
        }
        //stdlog_.debug(">>>> Filename -   " + fileName_);
	return fileName_;
    }


    /**
     * Set the filename for the pram file that is being stored
     * @param s filename
     */


    public void setFilename(String s) {
        fileName_ = s;
    }

    /** 
     * Return the filename for the ParamFileParameter
     * @return filename
     */

    public String getFilename() {
         return fileName_;
    }

    /**
     * Return the Compressed value of the file
     * @return an array of bytes that represnt the compressed file
     */

    public byte[] getCompressedValue() {
        return compressedValue_;
    }

    /**
     * Set the Compressed value.
     * @param compressValue an array of bytes.
     */
     
   
    public void setCompressedValue(byte [] compressedValue) {
        compressedValue_ = compressedValue;
    }


    /**
     * Set the unCompressedLength for the file
     * @param uncompressedLength    
     */

    public void setUncompressedLength(int uncompressedLength) {
        uncompressedLength_ = uncompressedLength;
    }

    /**
     * Return the uncompressed Length for the file
     * @return the uncompressed Lengthy for the file
     */

    public int getUncompressedLength() {
        return uncompressedLength_;
    }


    /**
     * @return A Paramfile object built from the value
     *         of this parameter; of <code>null</code> if
     *         no value is found.
     */
    public Paramfile getParamfile() {
	String paramfileText = getValue();
	if( paramfileText.equals( "NODEFAULT" ))
	    return( null );
	
	Paramfile p = null;
	try {
	    p = new Paramfile( new StringReader( paramfileText ));
	}
	catch( Exception e ) {
	    e.printStackTrace();
	}
	return( p );    
    }
}
