package org.eso.ohs.persistence;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import org.apache.log4j.Logger;

/**
 * This class manages application version information. 
 *
 * @author A M Chavan, 12-Dec-2001: creation.
 * @version $Revision: 1.3 $
 */

// $Id: OHSVersion.java,v 1.3 2005/06/17 14:22:40 gdonino Exp $

public class OHSVersion {

    private static String version_ = null;
    private static final String errVersion_ = "???";
    private static final String vdirname_ = "org/eso/ohs/config";
    private static final ClassLoader loader_ =
        OHSVersion.class.getClassLoader();
	private static Logger stdlog_ = Logger.getLogger(OHSVersion.class);

    /** 
     * @return A version string extracted from a file in the
     *         org/eso/ohs/config directory. The file itself 
     *         will be looked for among the system resources; 
     *         i.e., within one of JAR files or directories 
     *         listed in the CLASSPATH.
     *         <BR>
     *         In case of any error, an error version string
     *         is returned.
     *         <P>
     *         The version string is read only once, and its value
     *         cached. Any subsequent call of this function will
     *         return the same value, regardless of the current
     *         contents of the file.
     *
     * @param filename Basename of the file to look for. For
     *                 instance, if filename is <em>p2pp-version</em>, 
     *                 the version string will be extracted
     *                 from <em>org/eso/ohs/config/p2pp-version</em>.
     *
     * @see #getErrorVersion()
     */
    public static String getVersion(String filename) {
        String result;
		
		if (version_ == null){
	        byte[] buf = new byte[512]; // input buffer
	        String vfilename = vdirname_ + "/" + filename;
	        URL vfile = ClassLoader.getSystemResource(vfilename);
	
	        if (vfile == null){
	            result = errVersion_;
				stdlog_.error("Cannot retrieve version: cannot retrieve resource " + vfilename);
	        }
	        else{
				try {
					InputStream s = (InputStream) vfile.getContent();
					s.read(buf, 0, buf.length);
					version_ = (new String(buf)).trim();
					result = version_;
				}
				catch (IOException e) {
					result = errVersion_;
					stdlog_.error("Cannot retrieve version", e);
				}
	        }
        }else 
			result = version_;
			
		return result;
    }

    /** 
     * @return A conventional error "version" string.
     */
    public static String getErrorVersion() {
        return errVersion_;
    }
}
