/* $Id: DbaseHandlerVltiTarget.java,v 1.2 2005/04/06 16:38:12 ddorigo Exp $ */

package org.eso.ohs.persistence.dbase.phase1;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.BusinessVisitor;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.dfs.VltiTarget;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * Sql handler to retrieve, to get and to updateEsoUser objects on the database
 * <br>
 * $Id: DbaseHandlerVltiTarget.java,v 1.2 2005/04/06 16:38:12 ddorigo Exp $
 * 
 *
 * @author Carlo Boarotto, ESO 21-Mar-2000
 */
public class DbaseHandlerVltiTarget extends DbaseIOHandler {

    private static long MAX_PHASE1_TARGET = 10000;

    /**
     * Field stdlog_
     */
    private static Logger stdlog_ = Logger
            .getLogger(DbaseHandlerVltiTarget.class);

    /**
     * Managed class
     * 
     * @return EsoUser class
     */
    public Class getManagedClass() {
        return VltiTarget.class;
    }

    /** A reference to the singleton DbaseHandlerEsoUser instance. */
    private static DbaseHandlerVltiTarget singleton_ = null;

    /** column names */
    private static String PROG_ID = "programme_id";

    public static final String ID = "target_id";

    /**
     * Field SURNAME
     */
    public static final String MAG_VISUAL = "mag_visual";

    /**
     * Field SURNAME_DL
     */
    public static final String RESOLUTION = "resolution";

    /**
     * Field PASSWD
     */
    public static final String MAG_WAVE = "mag_wave";

    /** eso_users obsolete (and not null) column names */
    public static final String OBS_WAVE = "obs_wave";

    /**
     * Field EMAIL
     */
    public static final String SIZE_WAVE = "size_wave";

    /**
     * Field PANEL
     */
    public static final String BASELINE = "baseline";

    /**
     * Field time requested integer: (hours * 10)
     */
    public static final String TIME_ON_TARGET = "time_targets";

    /**
     * Field CHAIR
     */
    public static final String VISIBILITY = "visibility";

    /**
     * Field CHAIR
     */
    public static final String MAG_C = "mag_c";

    /** vlti_target_runs column names */
    public static final String RUN_ID = "run_id";

    public static final String RUN_TARGET_ID = "target_id";

    private static String RUN_PROG_ID = "programme_id";

    /**
     * Sql select statement to retireve EsoUser entry. other fileds could be
     * added in subclass
     */
    protected static final String target_fields = Phase1SelectStmt.vltiTargetsTable
            .get()
            + "."
            + ID
            + ","
            + Phase1SelectStmt.vltiTargetsTable.get()
            + "."
            + PROG_ID
            + ","
            + Phase1SelectStmt.vltiTargetsTable.get()
            + "."
            + MAG_VISUAL
            + ","
            + Phase1SelectStmt.vltiTargetsTable.get()
            + "."
            + RESOLUTION
            + ","
            + Phase1SelectStmt.vltiTargetsTable.get()
            + "."
            + MAG_WAVE
            + "\n,"
            + Phase1SelectStmt.vltiTargetsTable.get()
            + "."
            + OBS_WAVE
            + ","
            + Phase1SelectStmt.vltiTargetsTable.get()
            + "."
            + SIZE_WAVE
            + ",\n"
            + Phase1SelectStmt.vltiTargetsTable.get()
            + "."
            + BASELINE
            + ","
            + Phase1SelectStmt.vltiTargetsTable.get()
            + "."
            + VISIBILITY
            + "\n,"
            + Phase1SelectStmt.vltiTargetsTable.get()
            + "."
            + MAG_C
            + ","
            + Phase1SelectStmt.vltiTargetsTable.get()
            + "."
            + TIME_ON_TARGET;

    protected static final String run_target_fields = Phase1SelectStmt.runTargetsTable
            .get()
            + "."
            + RUN_TARGET_ID
            + ","
            + Phase1SelectStmt.runTargetsTable.get()
            + "."
            + RUN_PROG_ID
            + ","
            + Phase1SelectStmt.runTargetsTable.get() + "." + RUN_ID;

    protected final String target_select_sql = "SELECT DISTINCT "
            + target_fields;

    protected final String run_target_select_sql = "SELECT DISTINCT "
            + run_target_fields;

    protected final String target_sql = target_select_sql + "\n FROM "
            + Phase1SelectStmt.vltiTargetsTable.get() + " ";

    /** ctor */
    public DbaseHandlerVltiTarget() {
        super();
    }

    /**
     * Read user data
     * 
     * @param stmt
     *            statement
     * @param objId
     *            id to be read
     * @return UnpackedStorableObject
     * @exception ObjectIOException
     *                IO Eror occured while reading from the database
     * @exception ObjectNotFoundException
     *                the requested object was not found in the database
     * @exception SQLException
     *                an error occurred during the dbase operation
     */
    public UnpackedStorableObject read(Statement stmt, long objId)
            throws ObjectIOException, ObjectNotFoundException, SQLException {

        VltiTarget target = (VltiTarget) newInstance();
        UnpackedStorableObject uso = new UnpackedStorableObject(target);

        String sql = target_sql + " WHERE " + ID + " = "
                + DbaseHandlerPhase1Target.targetIdFromId(objId) + " AND "
                + PROG_ID + " = "
                + DbaseHandlerPhase1Target.programmeIdFromId(objId);
        stdlog_.debug(sql);
        ResultSet rs = stmt.executeQuery(sql);

        if (!rs.next()) { // Only one with this id !!
            throw new ObjectNotFoundException("Object not found in database");
        }

        target.setId(Config.getCfg().tableToUniqueId(objId, VltiTarget.class));
        setTargetFields(target, rs);
        rs.close();
        // load the affiliation for the user
        getRunId(stmt, target);
        return new UnpackedStorableObject(target);
    }

    /**
     * Read data objects are unique)
     * 
     * @param stmt
     *            statement
     * @param whereClause
     *            where condition for retreiving the object
     * @param otherTables
     *            String table condition
     * @return UnpackedStorableObject[]
     * @exception ObjectIOException
     *                IO Eror occured while reading from the database
     * @exception ObjectNotFoundException
     *                the requested object was not found in the database
     * @exception SQLException
     *                an error occurred during the dbase operation
     */
    public UnpackedStorableObject[] read(Statement stmt, String whereClause,
            String otherTables) throws ObjectIOException,
            ObjectNotFoundException, SQLException {

        if (otherTables == null) {
            otherTables = "";
        }

        UnpackedStorableObject[] target_array = null;
        VltiTarget target = null;
        String sql = target_sql + otherTables + "\n WHERE " + whereClause;
        stdlog_.debug(sql);
        ResultSet rs = stmt.executeQuery(sql);

        try {
            Set targets = new LinkedHashSet();
            while (rs.next()) {
                target = (VltiTarget) newInstance();

                long targetId = rs.getInt(ID);
                long progId = rs.getInt(PROG_ID);
                // Set only when attaching the object as a new
                // element
                target.setId(Config.getCfg().tableToUniqueId(
                        DbaseHandlerPhase1Target.idForPhase1Target(
                                targetId, progId), VltiTarget.class));
                setTargetFields(target, rs);
                // New element
                targets.add(target);
            }
            rs.close();

            VltiTarget[] targetList = (VltiTarget[]) targets
                    .toArray(new VltiTarget[targets.size()]);
            target_array = new UnpackedStorableObject[targetList.length];
            // add the affiliation to the user and insert them in array
            for (int i = 0; i < target_array.length; i++) {
                target = targetList[i];
                getRunId(stmt, target);
                target_array[i] = new UnpackedStorableObject(target);
            }
        } catch (SQLException e) {
            stdlog_.error(e);
            throw new ObjectIOException(e);
        }
        return target_array;
    }

    /**
     * Set the user fields from the result set. Only for avoiding duplicated
     * code
     * 
     * @param user
     *            EsoUser
     * @param rs
     *            ResultSet
     * @throws SQLException
     */
    protected void setTargetFields(VltiTarget target, ResultSet rs)
            throws SQLException {
        target.setTargetId(rs.getInt(ID));
        target.setProgrammeId(rs.getInt(PROG_ID));
        target.setMagVisual(rs.getBigDecimal(MAG_VISUAL));
        target.setResolution(rs.getInt(RESOLUTION));
        target.setMagWave(rs.getBigDecimal(MAG_WAVE));
        target.setObsWave(rs.getBigDecimal(OBS_WAVE));
        target.setSizeWave(rs.getInt(SIZE_WAVE));
        target.setBaseline(rs.getString(BASELINE));
        target.setVisibility(rs.getString(MAG_WAVE));
        target.setMag(rs.getString(MAG_WAVE));
        target.setTimeTarget(rs.getInt(TIME_ON_TARGET));
    }

    protected void getRunId(Statement stmt, VltiTarget target)
            throws ObjectIOException, ObjectNotFoundException, SQLException {

        Vector runList = new Vector();
        String star_sql = run_target_fields + " WHERE " + RUN_TARGET_ID + " = "
                + target.getTargetId() + " AND " + RUN_PROG_ID + " = "
                + target.getProgrammeId();
        stdlog_.debug(star_sql);
        ResultSet rs = stmt.executeQuery(star_sql);

        // get the affiliation list reusing the statement
        if (!rs.next()) { // Only one with this id !!
            throw new ObjectNotFoundException("Object not found in database");
        }

        long runId = rs.getLong(RUN_ID);
        target.setRunId(runId);
        rs.close();
    }

    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be changed
     * from within P2PP.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * 
     * @exception ObjectIOException
     *                operation is not permitted exception
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
            throws ObjectIOException {

        VltiTarget target = (VltiTarget) obj.getObject();

        String sqlText = buildInsertStmt(target);

        try {
            stmt.executeUpdate(sqlText);
        } catch (SQLException e) {
            stdlog_.error("INSERT error", e);
            throw new ObjectIOException("Error executing write: " + sqlText);
        }
    }

    public static String buildInsertStmt(VltiTarget target) {
        StringBuffer sb = new StringBuffer()
        .append(target.getTargetId())
        .append(",")
        .append(target.getProgrammeId())
        .append(",")
        .append(target.getMagVisual())
        .append(",")
        .append(target.getResolution())
        .append("\n,")
        .append(target.getMagWave())
        .append(",")
        .append(target.getObsWave())
        .append(",")
        .append(target.getSizeWave())
        .append(",\'")
        .append(TextUtils.quoteSqlString(target.getBaseline()))
        .append("\',\'")
        .append(TextUtils.quoteSqlString(target.getVisibility()))
        .append("\'\n,\'")
        .append(TextUtils.quoteSqlString(target.getMag()))
        .append("\',")
        .append(target.getTimeTarget())
        .append("");

        StringBuffer runtargetStmt = insertRunLinkStmt(target);
        String sqlText = "INSERT INTO "
                + Phase1SelectStmt.vltiTargetsTable.get() + "\n ("
                + target_fields + " ) \n" + " VALUES (" + sb.toString()
                + " )\n " + runtargetStmt.toString();
        stdlog_.debug(sqlText);
        return sqlText;
    }

    /**
     * 
     * @param user
     *            EsoUser
     * @param userId
     *            long
     * @return StringBuffer
     */
    private static StringBuffer insertRunLinkStmt(VltiTarget target) {
        StringBuffer stmt = new StringBuffer();
        stmt
                .append("\n ")
                .append(" INSERT INTO ")
                .append(Phase1SelectStmt.runTargetsTable.get())
                .append(" (")
                .append(run_target_fields)
                .append(" ) ")
                .append(" VALUES (")
                .append(target.getTargetId())
                .append(",")
                .append(target.getProgrammeId())
                .append(",")
                .append(target.getRunId())
                .append(")");
        return stmt;
    }

    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be changed
     * from within P2PP.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * 
     * @exception ObjectIOException
     *                operation is not permitted exception
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject obj)
            throws ObjectIOException {

        VltiTarget user = (VltiTarget) obj.getObject();
        update(stmt, user);
    }

    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be changed
     * from within P2PP.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param obj
     *            an array: the first element is the BO to be stored.
     * 
     * @exception ObjectIOException
     *                operation is not permitted exception
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, StorableObject obj)
            throws ObjectIOException {

        VltiTarget target = (VltiTarget) obj;
        String sqlText = buildUpdateStmt(target);

        try {
            stmt.executeUpdate(sqlText);
        } catch (SQLException e) {
            stdlog_.error("INSERT error", e);
            throw new ObjectIOException("Error executing write: " + sqlText);
        }
    }

    public static String buildUpdateStmt(VltiTarget target) {
        StringBuffer sb = new StringBuffer();
        sb
                .append(MAG_VISUAL)
                .append(" = ")
                .append(target.getMagVisual())
                .append(",")
                .append(RESOLUTION)
                .append(" = ")
                .append(target.getResolution())
                .append(",")
                .append(TIME_ON_TARGET)
                .append(" = ")
                .append(target.getTimeTarget())
                .append(",")
                .append(MAG_WAVE)
                .append(" = ")
                .append(target.getMagWave())
                .append(",")
                .append(OBS_WAVE)
                .append(" = ")
                .append(target.getObsWave())
                .append(",")
                .append(BASELINE)
                .append(" = ")
                .append(target.getBaseline())
                .append(",")
                .append(SIZE_WAVE)
                .append(" = ")
                .append(target.getSizeWave())
                .append(",")
                .append(VISIBILITY)
                .append(" = ")
                .append(target.getVisibility())
                .append(",")
                .append(MAG_C)
                .append(" = ")
                .append(target.getMag())
                .append(" ");

        String runUpdateStmt = updateRunStmt(target);

        String sqlText = runUpdateStmt + " \n " + "UPDATE "
                + Phase1SelectStmt.vltiTargetsTable.get() + " SET "
                + sb.toString() + " WHERE " + ID + " = " + target.getTargetId()
                + " AND " + PROG_ID + " = " + target.getProgrammeId();
        stdlog_.debug(sqlText);
        return sqlText;
    }

    /**
     * Method updateAffilStmt
     * 
     * @param user
     *            EsoUser
     * @param userId
     *            long
     * @return StringBuffer
     */
    private static String updateRunStmt(VltiTarget target) {

        String sqlText = "UPDATE " + Phase1SelectStmt.vltiTargetsTable.get()
                + " SET " + RUN_ID + " = \'" + target.getRunId()
                + "\' WHERE " + RUN_TARGET_ID + " = " + target.getTargetId()
                + " AND " + RUN_TARGET_ID + " = " + target.getProgrammeId();
        return sqlText;

    }

    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be changed
     * from within P2PP.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * 
     * @exception ObjectIOException
     *                operation is not permitted exception
     */
    public void delete(Statement stmt, long id) throws ObjectIOException {
        throw new ObjectIOException("Operation not supported for this object");
    }

    /**
     * Method accept
     * 
     * @param v
     *            BusinessVisitor
     * @param arg
     *            Object
     * @return Object
     */
    public Object accept(BusinessVisitor v, Object arg) {
        return null;
    }

    /**
     * Return the one and only instance of the DbaseHandlerEsoUser.
     * 
     * @return Singleton DbaseHandlerEsoUser instance.
     */
    public static DbaseHandlerVltiTarget getDbaseHandlerEsoUser() {
        if (singleton_ == null) {
            singleton_ = new DbaseHandlerVltiTarget();
        }
        return singleton_;
    }
}