package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.io.StringReader;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.PafData;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.Mask;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;


/**
 * @author tcanavan
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class VIMOSReportImp implements MaskReport {

    private static Logger stdlog_ =
        Logger.getLogger(VIMOSReportImp.class);

    private static String obrepDbName_ =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());
    /**
     * Code to be moved later from MaskDiscarding
     */
    public void receiveDiscardingReport(String report) throws SQLException {
    }

    /**
     * Code to be move later from MaskInsertion
     */
    public void receiveInsertionReport(String report)
        throws SQLException, ObjectIOException, ObjectNotFoundException, IOException {
        ConnectionPool pool = ConnectionPool.getPool();
        Connection conn = null;
        Statement stmt = null;
        Vector obIds = new Vector();
        OBIDType [] queueOBIDs = {
        };
        try {
            PafFile paf = new PafFile(new StringReader(report));
            PafData[] pd = paf.getIdentifierList();
            conn =
                pool.acquireConnection(
                    Config.getCfg().getUrl(ObservationBlock.getSuffix()),
                    Config.getCfg().getDbUserName(),
                    Config.getCfg().getDbPassword());

            conn.setAutoCommit(false);
            stmt = conn.createStatement();
            initSlotNums(stmt);
            resetToMaskManufactured(stmt);
            updateSlots(stmt, pd);
            queueOBIDs = getOBs(stmt);
            conn.commit();
        } catch (SQLException e) {
            try {
                conn.rollback();
            } catch (SQLException exc) {
                exc.printStackTrace();
            }
            e.printStackTrace();

        } finally {
            if (conn != null) {
                try {
                    conn.setAutoCommit(true);
                    stmt.close();
                } catch (SQLException e) {
                    // ignore
                }
                pool.releaseConnection(conn);

            }
        }
        ObjectManager objMgr = ObjectManager.getObjectManager();
        boolean isInRegistry = false;
        if (objMgr
            .isInRegistry(Config.getCfg().tableToUniqueId(1, MaskQueue.class)))
            isInRegistry = true;
        Queue queue =
            (Queue) objMgr.getBusObj(
                Media.DBASE,
                Config.getCfg().tableToUniqueId(1, MaskQueue.class),
                MaskQueue.class);
        // Save it just in case it has been edited.        
        objMgr.putBusObj(queue);
        Vector qVec = new Vector();
        for (int i = 0; i < queueOBIDs.length; i++) {
            QueueItem newQueueItem =
                new QueueItem(queueOBIDs[i].getObId().longValue(), 1, i,queueOBIDs[i].getType());
            qVec.addElement(newQueueItem);
        }
        QueueItem[] queueItems = new QueueItem[qVec.size()];
        qVec.copyInto(queueItems);
        queue.setQueueList(queueItems);
        objMgr.putBusObj(queue);
        objMgr.refreshBusObj(Media.DBASE, queue.getId(), MaskQueue.class);
        queue =
            (Queue) objMgr.getBusObj(
                Media.DBASE,
                Config.getCfg().tableToUniqueId(1, MaskQueue.class),
                MaskQueue.class);
        queue.setIsNew(false);
        if (!isInRegistry)
            objMgr.forgetBusObj(Media.DBASE, queue.getId(), MaskQueue.class);
        queue.setUpdate(false);

    }

    public OBIDType [] getOBs(Statement stmt) throws SQLException {
        String getOBs = "SELECT " 
                + obrepDbName_ + "..obsblk_masks.ob_id,"
                + obrepDbName_ + "..obs_blocks.type,"
                + obrepDbName_ + "..masks.mask_id from "
                + obrepDbName_
                + "..obsblk_masks,"
                + obrepDbName_
                + "..masks,"
                + obrepDbName_
                + "..obs_blocks"
                + " WHERE "
                + obrepDbName_
                + "..obsblk_masks.mask_id"
                + " = "
                + obrepDbName_
                + "..masks.mask_id"
                + " AND "
                + obrepDbName_
                + "..obsblk_masks.ob_id = "
                + obrepDbName_
                + "..obs_blocks.ob_id"
                + " AND "
                + obrepDbName_
                + "..masks.mask_status = "
                + SqlUtils.sqlEscapedString(Mask.MASK_INSERTED)
                + " order by "
                + obrepDbName_
                + "..obsblk_masks.ob_id";
        stdlog_.info("Get OB's " + getOBs);
        ResultSet rs = stmt.executeQuery(getOBs);
        Vector maskIdVec = new Vector();
        Vector obIds = new Vector();
        Vector typeVec = new Vector();
        HashMap listElements = new HashMap();
        long prevId = -1;
        Long obId;
        String type;
        while (rs.next()) {
            obId = new Long(rs.getLong("ob_id"));
            type = rs.getString("type");
            
            boolean newRec = false;
            if (!obIds.contains(obId)) {
                obIds.addElement(obId);
                typeVec.addElement(type);
                newRec = true;
                if (prevId != -1) {
                    // Store new record
                    listElements.put(
                        new Long(prevId),
                        createMaskOB(maskIdVec, prevId));
                    maskIdVec.removeAllElements();
                }
            }
            maskIdVec.addElement(new Long(rs.getLong("mask_id")));
            prevId = obId.longValue();
        }
        // store the last record of thw list of OB's
        if (prevId != -1) {
            // Store new record
            listElements.put(new Long(prevId), createMaskOB(maskIdVec, prevId));
            maskIdVec.removeAllElements();
        }
        Vector maskIdsForOBVec = new Vector();
        Vector returnOBIDs = new Vector();
        for (int i = 0; i < obIds.size(); i++) {
            MaskIdsForOB maskIdForOB =
                (MaskIdsForOB) listElements.get(obIds.elementAt(i));
            if (!maskIdsForOBVec.contains(maskIdForOB)) {
                maskIdsForOBVec.addElement(maskIdForOB);
                String obType = (String)typeVec.elementAt(i);
                Class c = CalibrationBlock.class;
                if(obType.equals(CalibrationBlock.OBSBLK_TYPE)) {
                	c = ObservationBlock.class;
                }
                long retId = Config.getCfg().tableToUniqueId(maskIdForOB.getObId(),c);
                returnOBIDs.addElement(new OBIDType(new Long(retId),obType));
            }
        }
        OBIDType[] returnArray = new OBIDType[returnOBIDs.size()];
        returnOBIDs.copyInto(returnArray);
        return returnArray;
    }

    private MaskIdsForOB createMaskOB(Vector maskIdVec, long prevId) {
        long[] maskIds = new long[maskIdVec.size()];
        for (int i = 0; i < maskIdVec.size(); i++) {
            maskIds[i] = ((Long) maskIdVec.elementAt(i)).longValue();
        }
        MaskIdsForOB maskIdsForOB = new MaskIdsForOB(prevId, maskIds);
        return maskIdsForOB;
    }

    private void updateSlots(Statement stmt, PafData[] pd)
        throws SQLException {
        for (int i = 0; i < pd.length; i++) {
            String name = pd[i].getIdentifier();
            //int j=0;
            if (name.startsWith("INS.MASK") && name.endsWith(".ID")) {
                //j++:
                int startIndex = -1;
                int endIndex = -1;
                for (int k = 0; k < name.length(); k++) {
                    if (Character.isDigit(name.charAt(k))) {
                        startIndex = k;
                    }
                }
                endIndex = name.lastIndexOf('.');
                String idxStr = name.substring(startIndex, endIndex);
                int maskNo = -1;
                stdlog_.info("Idx String " + idxStr);
                try {
                    maskNo = new Integer(idxStr).intValue();
                } catch (NumberFormatException e) {
                    e.printStackTrace();
                }
                String maskId = pd[i].getValue();
                int slotNo = new Integer(pd[i + 1].getValue()).intValue();
                updateSlot(stmt, maskId, slotNo);
            }
        }
    }

    private void resetToMaskManufactured(Statement stmt) throws SQLException {
        String maskManufactured =
            "UPDATE "
                + obrepDbName_
                + "..masks "
                + "SET mask_status = "
                + SqlUtils.sqlEscapedString(Mask.MASK_MANUFACTURED)
                + " WHERE "
                + obrepDbName_
                + "..masks.mask_status = "
                + SqlUtils.sqlEscapedString(Mask.MASK_INSERTED);

        stmt.executeUpdate(maskManufactured);
    }

    private void initSlotNums(Statement stmt) throws SQLException {
        String initSlotNums =
            "UPDATE "
                + obrepDbName_
                + "..masks "
                + "SET mask_slot_num =  "
                + Mask.DEFAULT_MASK_SLOT_NUMBER;
        stmt.executeUpdate(initSlotNums);
    }
    /**
        * Update the slot number in 
        * @param stmt JDBC statement
        * @param maskId Mask id to be updated
        * @param slotNo Slot number to be used in the cabinet
        */

    //     order to keep mask code distinct from the actual SQL.
    private void updateSlot(Statement stmt, String maskId, int slotNo)
        throws SQLException {
        String sql =
            "UPDATE "
                + obrepDbName_
                + "..masks"
                + " SET mask_slot_num = "
                + slotNo
                + " WHERE "
                + obrepDbName_
                + "..masks.bar_code = "
                + "'"
                + maskId
                + "'";

        stdlog_.info("sql = " + sql);
        stmt.executeUpdate(sql);

        String sql1 =
            "UPDATE "
                + obrepDbName_
                + "..masks"
                + " SET mask_status = 'I' "
                + " WHERE "
                + obrepDbName_
                + "..masks.bar_code = "
                + "'"
                + maskId
                + "'";

        stdlog_.info("sql = " + sql);
        stmt.executeUpdate(sql1);
    }

    private class MaskIdsForOB {

        private long obId_;

        private long[] maskIds_;

        public MaskIdsForOB(long obId, long[] maskIds) {
            obId_ = obId;
            maskIds_ = maskIds;
        }

        /**
         * Returns the maskIds.
         * @return int[]
         */
        public long[] getMaskIds() {
            return maskIds_;
        }

        /**
         * Returns the obId.
         * @return int
         */
        public long getObId() {
            return obId_;
        }

        /**
         * Sets the maskIds.
         * @param maskIds The maskIds to set
         */
        public void setMaskIds(long[] maskIds) {
            maskIds_ = maskIds;
        }

        /**
         * Sets the obId.
         * @param obId The obId to set
         */
        public void setObId(long obId) {
            obId_ = obId;                                   
        }
        
        // Since equals is overridden then we must override hashCode
        //
        
        public int hashCode() {
            return new Long(obId_).hashCode();

        }

        public boolean equals(Object object) {
            long[] compareMaskIds = ((MaskIdsForOB)object).getMaskIds();
            if (compareMaskIds.length == maskIds_.length) {
                for (int i = 0; i < maskIds_.length; i++) {
                    boolean found = false;
                    for (int j = 0; j < compareMaskIds.length; j++) {
                        if (maskIds_[i] == compareMaskIds[j]) {
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        return false;
                    }
                }
            } else {
                return false;
            }
            return true;
        }

    }
    
    

    /**
     * @see org.eso.ohs.persistence.dbase.phase2.MaskReport#receiveManufacturingReport(java.lang.String)
     */
    public void receiveManufacturingReport(String report) throws SQLException {
    }
    
    
    private class OBIDType {
    	
    	private String type;
    	
    	private Long obId;
    	
    	public OBIDType(Long obId,String type) {
    		this.obId = obId;
    		this.type = type;
    		
    	}

		public Long getObId() {
			return obId;
		}

		public void setObId(Long obId) {
			this.obId = obId;
		}

		public String getType() {
			return type;
		}

		public void setType(String type) {
			this.type = type;
		}    	
    }

}
