package org.eso.ohs.phase2.apps.masktracker;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.StringTokenizer;

import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.phase2.icdVcs.EventCmdListener;
import org.eso.ohs.phase2.icdVcs.ScriptCmdListener;

/**
 * Listener for script events from the BOB interface
 * 
 * <p>
 * $Id: MaskListener.java,v 1.15 2005/02/11 11:00:27 tcanavan Exp $<br>
 * 
 * @author Bob Kemp
 */
public class MaskListener implements ScriptCmdListener, EventCmdListener {

    private MaskActionGenerator generator_ = null;

    /** Singleton reference to Mask Instance */

    private static MaskListener instance_ = null;

    /** Gets the singleton refence to the MaskListener class */

    public static MaskListener getInstance() {
        if (instance_ == null) {
            instance_ = new MaskListener();
        }

        return instance_;
    }

    /**
     * Handler method for NextObsBlocks requests from BOB.
     * 
     * @param numOBs
     *            Number of OBs to be transferred
     * @param isOffline
     *            Boolean.TRUE if is offline request, Boolean.FALSE if is not an
     *            offline request null if unspecified
     * 
     * @return Observation block in OBD format or null string if an error
     *         occurred.
     */
    public String NextObsBlocks(int numOBs, Boolean isOffline) {

        if (!Boolean.TRUE.equals(isOffline)) {
            return "";
        }

        return generator_ == null ? "" : generator_.generateOrder(numOBs);
    }

    /**
     * Handler method for ObsBlockStatus messages from BOB.
     * 
     * @param msg
     *            status message as a tcl array
     *  
     */
    public void ObsBlockStatus(String id, String when, String what,
            String extraInfo) {
        // ignore -- don't care about OB status
    }

    /**
     * Handler method for TemplateStatus messages from BOB.
     * 
     * @param msg
     *            status message as a tcl array
     */
    public void TemplateStatus(String id, String when, String what,
            String extraInfo) {
        // ignore -- don't care about TS status
    }

    /**
     * Handler method for GetPAF messages from BOB.
     * 
     * @param reply
     *            Reply as a string, eg Mask Manufacturing Report.
     */

    /*
     * public void GetPAFResult(String reply) {
     * 
     * System.out.println( ">>> GetPAFResult( '" + reply +"' )" );
     * System.out.println( ">>> generator_: " + generator_ ); if (generator_ !=
     * null) { generator_.receiveReport(reply); } }
     */

    /**
     * This is the method BOB calls when it sends OT/MaskTracker a Report
     * 
     * @param reply
     *            Report file from BOB. This will be either a MaskDiscarding
     *            MaskInsertion or MaskManufacturing report
     */

    public void GetPAFResult(String reply) {

        System.out.println(">>> GetPAFResult( '" + reply + "' )");
        try {
            PafFile paf = new PafFile(new StringReader(reply));
            if (!paf.isPafFile())
                return; // ERROR: nothing for the time being

            String pafName = paf.readLine("PAF.NAME");
            String pafExt = "";
            StringTokenizer st = new StringTokenizer(pafName, ".");
            while (st.hasMoreTokens())
                pafExt = st.nextToken(); // skip all tokens but the last

            System.out.println(">>> GetPAFResult: PAF.NAME: <" + pafName + ">");
            System.out.println(">>> GetPAFResult: pafExt: <" + pafExt + ">");

            String directory = AppConfig.getAppConfig().getPafDirectory();
            File dir = new File(directory);
            String saveFilename = AppConfig.getAppConfig().getPafDirectory()
                    + File.separator + pafName;

            if (dir.canWrite()) {
                try {
                    FileIOUtils.stringToFile(reply, saveFilename);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            if (pafExt.equals("mmr")) {
                MaskManufacturing.getInstance().receiveReport(reply);
                System.out.println(">>> GetPAFResult: MMR");
            } else if (pafExt.equals("mir")) {
                /*
                 * Process a Mask Insertion Report
                 * ----------------------------------------
                 */
                MaskInsertion.getInstance().receiveReport(reply);
                System.out.println(">>> GetPAFResult: MIR");
            } else if (pafExt.equals("mdr")) {
                /*
                 * Process a Mask Discarding Report
                 * ----------------------------------------
                 */
                System.out.println(">>> GetPAFResult: MDR");
                MaskDiscarding.getInstance().receiveReport(reply);

            } else {
                // ERROR: nothing for the time being
                System.out.println(">>> GetPAFResult: no action");
            }
            File saveFile = new File(saveFilename);
            //if(saveFile.exists()) {
            //   saveFile.delete();
            //}

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Set type
     */

    public void setType(MaskActionGenerator generator) {
        generator_ = generator;
    }

    /**
     * Get Type
     */

    public MaskActionGenerator getType() {
        return generator_;
    }

}

