package org.eso.ohs.phase2.apps.ot;

import java.sql.SQLException;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObjectFactory;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSPersistence;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.phase2.QueueItemDBIO;
import org.eso.ohs.persistence.dbase.phase2.QueueManager;

/**
 * This subclass of the ObjectManager handles all application-specific
 * issues relating to storage of objects.
 * <p>
 * For correct operation, there should be at most one instance of this
 * object.  Thus it is a singleton and can only be instantiated using
 * the static Persistence.getInstance() method.  Each time this
 * method is called it will return the <em>same</em> instance of the
 * ObjectManager.
 * <p>
 * %Id: Persistence.java,v 1.16 2001/06/22 11:47:57 tschlich Exp %
 * <br>
 * @version $Revision: 1.8 $
 * @author Bob Kemp
 */
public class Persistence extends OHSPersistence {

    public final String cvsID_ =
        "$Id: Persistence.java,v 1.8 2005/03/30 09:56:13 tcanavan Exp $";

    protected static Logger stdlog_ =
        Logger.getLogger( Persistence.class );

    private static Persistence instance_;

    //private ObsRunRegistry obsRunRegistry_ = new ObsRunRegistry();

    /** current user id */
    protected int userId_ = -1;


    /**
     * Return the one and only instance of the ObjectManager.
     *
     * @return Singleton ObjectManager instance.
     */
    public static Persistence getInstance() {
        return instance_;
    }


    /**
     * Install a new instance of the Persistence.
     *
     * @param instance  new Persistence instance.
     */
    public static void setInstance(Persistence instance) {
        instance_ = instance;
        setObjectManager(instance);
    }


    public void removeQueueItems(Queue queue,long [] obIds) {
        try {
            queue.removeQueueItems(obIds);
            QueueItemDBIO.removeQueueItems(queue.getId(),obIds);
        } catch(SQLException e) {
        }
    }


    public void markStatus(long [] OBIDs,String status)
    throws SQLException,ObjectNotFoundException,ObjectIOException {
        //QueueManagerImpl qMgr = new QueueManagerImpl();
        QueueManager.updateOBs(OBIDs,status, null, null);
        //updates OBs that are in memory
        for (int j=0;j<OBIDs.length;j++) {
            if (isInRegistry(OBIDs[j])) {
                stdlog_.debug("OB ids " + OBIDs[j]);
                Class c = Config.getCfg().getClassFromId(OBIDs[j]);
                CalibrationBlock cb =
                    (CalibrationBlock)getBusObj(Media.DBASE, OBIDs[j], c);
                cb.setStatus(status);
            }
        }
    }


    public void commentOBs(long [] OBIDs,
        String status, String obsComment, String obcGrade)
    throws ObjectNotFoundException,ObjectIOException,SQLException {
        //QueueManagerImpl qMgr = new QueueManagerImpl();
        QueueManager.updateOBs(OBIDs,status, obsComment, obcGrade);
            //update OBs that are in memory
        for (int j=0;j<OBIDs.length;j++) {
            if (isInRegistry(OBIDs[j])) {
                Class c = Config.getCfg().getClassFromId(OBIDs[j]);
                stdlog_.debug("Class " + c);
                refreshBusObj(Media.DBASE,OBIDs[j],c);
            }
        }
    }


    public static void startup() {
        Persistence.setInstance(new Persistence());
        //getInstance().startAutoSaver();
    }


    /**
     * Shutdown the current instance of ObjectManager and AutoSaver.
     */
    public void shutdown() {
        ConnectionPool.shutdown();
        super.shutdown();
    }
    
    
    /**
     * Creates a StorableObject, registers it with the ObjectManager
     * and, if it's a CalBlk or ObsBlk, sets the ObservingRun property.
     * 
     * @param device    Media on which the object is to be stored
     * @param dir   Directory in which the object is to be stored
     * @param objType   Type of object to be created
     * 
     * @exception ObjectIOException Object I/O Exception     
     */
    public BusinessObject createAndRegister(Media device,
                    DirectoryNode dir, Class objType)
                        throws ObjectIOException {
    BusinessObject sobj = ObjectFactory.create(objType);
    ObjectManager objMgr = ObjectManager.getObjectManager();

    objMgr.recursiveRegister(device, dir, sobj);

    return sobj;
    }    

    /**
     * Re-creation hook for subclasses
     *
     * @see #startup
     * @see #shutdown
     */
    public void createNewInstance() {
        startup();
    }
}

