package org.eso.ohs.phase2.apps.ot.actions;

import java.awt.event.ActionEvent;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.phase2.apps.ot.gui.OTViewManager;
import org.eso.ohs.phase2.apps.ot.gui.QueueContainer;
import org.eso.ohs.phase2.apps.ot.gui.QueueView;


/**
 * Action class reload a queue
 *
 * <p>
 * $Id: ReloadQueueAction.java,v 1.7 2005/02/11 10:31:49 tcanavan Exp $
 * <br>
 * Author: Bob Kemp
 */
public class ReloadQueueAction extends ActionSuperclass {

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( ReloadQueueAction.class );

  //private JTable queueTable_;

    private QueueContainer queueContainer_;

    private JFrame parentFrame_;

    public ReloadQueueAction(QueueContainer queueContainer, String name) {
	super(queueContainer.getParentFrame(),name);
        queueContainer_ = queueContainer;
	parentFrame_ = queueContainer.getParentFrame();
    }


  

    public void actionPerformedImpl(ActionEvent ev) {         
        long [] queueIds = queueContainer_.getSelectedOpenQueueIds();
        int result = JOptionPane.YES_OPTION;
	switch (result) {
	    case JOptionPane.YES_OPTION:	   
	        new IncrementalActionAutomaton(
                    new IncrementalReloadQueue(queueIds),
		    parentFrame_,
                    "Reverting...."	
		);
	   default:return;
	}        
    }




 /**
     * Utility class to duplicate a sequence of OBs, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalReloadQueue implements IncrementalAction {


	/** array of ids of objects to be duplicated */
	private long [] queueIds_;
  
        private int objCount_;

        private Queue queue_;


	/**
	 * Constructor.
	 * 
	 * @param frm	    window on which to base any dialog boxes.
	 * @param device    media containing the objects to be duplicated
	 * @param deepCopy  should we duplicate attached objects?
	 * @param dir	    directory containing the ObsBlocks
	 * @param objType   type of object to be duplicated
	 * @param idLongs   array of ids of objects to be duplicated
	 */
	public IncrementalReloadQueue(long [] queueIds) {	   
	    queueIds_  = queueIds;         
	    objCount_ = queueIds.length;	
	    stdlog_.debug("Afer constructing Incremental Open Queue");   		       
	}

       /**
        * Number of steps in the action that are still remaining to be done.
        * 
        * @return number of steps remaining (eg objects to acted upon)
        */
        public int getNumStepsRemaining(int counter) {
            Long [] queueObjs = new Long[queueIds_.length];
	    for(int i=0;i<queueIds_.length;i++) {
                queueObjs[i] = new Long(queueIds_[i]);
            }
            return ObjUtils.howManyNotNull(queueObjs, counter);
        }

        /**
         * Method to perform one incremental step of a larger action.
         * 
         * @param stepNo	Counter to indicate the n'th step of a larger action
         */
        public void performStep(int counter) {           
	    doReload(queueIds_[counter]);                          
        }

        /**
         * Hook for initialisation of action object.
         */
        public void startOfAction() {
        }

        /**
         * Hook for cleanup of action object.
         */
        public void endOfAction() {             
	    //QueueSummary summ = new QueueSummary();
            //summ.setId(Config.getCfg().uniqueToTableId(queue_.getId()));
            //summ.setName(queue_.getName());
            //summ.setSize(queue_.getSize());  
        }


	/**
	 * Utility method to duplicate a given business object
	 * 
	 * @param id	    id of business object to duplicate
	 */
	public void doReload(long queueId) {
	    stdlog_.debug("doReload: " + queueId);
            OTViewManager manager = OTViewManager.getOTViewMgr();           
            QueueView queueMgr = manager.getQueueView(queueId);
            queue_ = queueMgr.reloadQueue();            
	}            	   
    }
}

