package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbaseDec;
import org.eso.ohs.core.dbb.client.DbaseRA;
import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbCheckBox;
import org.eso.ohs.core.dbb.client.DbbExecutionTime;
import org.eso.ohs.core.dbb.client.DbbHHMMDateTime;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbSortTerminated;
import org.eso.ohs.core.dbb.client.DbbSortTerminatedListener;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.MoonDistance;
import org.eso.ohs.core.dbb.sql.DbbQueryTerminated;
import org.eso.ohs.core.dbb.sql.DbbQueryTerminatedListener;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.MaskStatusType;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.phase2.QueueManager;
import org.eso.ohs.phase2.actions.DumpMaskAdpAdmData;
import org.eso.ohs.phase2.actions.ExportBOAction;
import org.eso.ohs.phase2.apps.ot.actions.ExecutionSequenceAction;
import org.eso.ohs.phase2.apps.ot.actions.MarkReadmeStatusAction;
import org.eso.ohs.phase2.apps.ot.actions.MarkStatusAction;
import org.eso.ohs.phase2.apps.ot.actions.OpenCommentsAction;
import org.eso.ohs.phase2.apps.ot.actions.ReloadQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.ReloadQueueItemsAction;
import org.eso.ohs.phase2.apps.ot.actions.RemoveQueueItemsAction;
import org.eso.ohs.phase2.apps.ot.dbb.OTDbbView;

/**
 * The QueueItemsBView class is a DB search engine used for displaying queues of
 * OBs according to a user-definable set of properties
 * 
 * @author G Passera <br>
 *         ChangeLog: Tim Canavan <br>
 *         ChangeLog: cmavros Add call to unsubscribe execution action. <br>
 *         ChangeLog: cmavros 1709/OT Call ExecutionSequenceAction class to
 *         implement progress bar <br>
 *         ChangeLog: cmavros 23May02 [1643/OT] Add selection items so user can
 *         customize. <br>
 *         ChangeLog: cmavros 25JUL02 [?/OT] Change "Execution Time" label to
 *         "ExecTime" becasue the sort does an comparison with the label.
 * 
 * @version $Id: QueueItemsBView.java,v 1.58 2005/12/12 16:00:55 ddorigo Exp $
 * 
 */

public class QueueItemsBView extends OTQueueBrowserView implements
		QueueContainer {


    private static final String README_STATUS_COLUMN = "Readme Status";



	private static Logger stdlog_ = Logger.getLogger(QueueItemsBView.class);

	private Action reloadQueueItemsAction_;

	private ExecutionSequenceAction executionAction_;

	private Action removeAction_;

	private Action markAction_;

	private Action markReadmeAction_;

	private Action openCommentsAction_;

	private Action exportAction_;

	private Action dumpAdpAdmDataAction_;

	private OTQueueBrowserView execSequence_;

	private static TextDisplayWidget log_ = new TextDisplayWidget(
			"Append Queue Items Report");

	// private DbbSqlTable si_table = BrowserConfig.si;

	private PasteActionListener pasteActionListener_;

	private CopyActionListener copyActionListener_;

	protected DbbArrowButton seqWidget;

	private JButton reload;

	protected static DbbPanel colPanel_;

	protected static DbbPanel rowPanel_;

	protected static DbbPanel sortPanel_;

	private DbbCheckBox maskStatusBox;

	private DbbCheckBox maskSlotNumBox;

	private DbbCheckBox maskChannelBox;

	private DbbSqlChunk obIDChunk;

	private DbbSqlChunk obIDMaskChunk;

	private DbbSqlChunk adpMaskChunk;

	/**
	 * Default constructor <br>
	 * ChangeLog: Gorgio Passera <DMD/ESO> <gpassera@eso.org>
	 * 
	 */
	public QueueItemsBView(Queue newQueue, OBTreeView tree) {
		super();
		initBaseClass(newQueue, "", true, true, true, tree);
	}

	/**
	 * Default constructor <br>
	 * ChangeLog: Gorgio Passera <DMD/ESO> <gpassera@eso.org>
	 * 
	 */
	public QueueItemsBView() {
		super();
	}

	public void initObj(Queue newQueue, OBTreeView tree) {

		try {
			initView(newQueue, "", true, true, true, tree);
			buildColumns();
			buildRows();
			buildSort();
		} catch (Exception e) {
			MsgManager.errDialog(e);
		}
	}

	public void initObjNoSort(Queue newQueue, OBTreeView tree) {
		try {
			initView(newQueue, "", true, true, true, tree);
			buildColumns();
		} catch (Exception e) {
			MsgManager.errDialog(e);
		}
		buildRows();
	}

	public void initOrangObj(Queue newQueue, OBTreeView tree) {
		try {
			initView(newQueue, "", true, true, true, tree);
			buildOrangColumns();
			buildOrangSort();
		} catch (Exception e) {
			MsgManager.errDialog(e);
		}
		buildRows();
	}

	/**
	 * void
	 */
	public void init() {
		execSequence_ = ExecutionSequenceBView.getInstance();
		createActions();
		buildToolBar();
		buildStructure();
	}

	/**
	 * void
	 */
	public void initNoMove() {
		execSequence_ = ExecutionSequenceBView.getInstance();
		createActions();
		buildToolBarNoMove();
		buildStructure();
	}
	
	public void addDisplayTextAction(JFrame frame) {
        view_.addDisplayTextAction(frame,"Text View",queue.getName(),queue.getName());		
	}

	/**
	 * void
	 */
	private void buildStructure() {
		layoutBrowser();
		view_.addQueryTerminatedListener(new QueryTerminatedListener());
		view_.addSortTerminatedListener(new SortTerminatedListener());
        //view_.addDisplayTextAction(this.getParentFrame(),"Text View",queue.getName(),queue.getName());
		addReloadButtons();
		setEnabled(false);
		setMinimumSize(new Dimension(0, 10));
	}

	/**
	 * void
	 */
	private void addReloadButtons() {				
		reload = new JButton("Revert");
		view_.addSummaryButton(reload);
		reload.addActionListener(new ReloadActionListener(this));
		view_.getSummaryButtons().removeStartQueryListener();
		reload.setEnabled(false);
		reloadQueueItemsAction_ = new ReloadQueueItemsAction(this);
		JButton queryButton = view_.getSummaryButtons().getQueryButton();
		queryButton.addActionListener(reloadQueueItemsAction_);
	}

	public QueueItemsBView(Queue newQueue, OBTreeView tree,
			OTQueueBrowserView execSequence) {
		super();
		initBaseClass(newQueue, "", true, true, true, tree);
		
		execSequence_ = execSequence;
		createActions();
		buildToolBar();
		buildStructure();
	}

	public void createActions() {
		super.createActions();
	}

	public void refreshReadmeStatus(long runId, String status) {
		int colIndex = getReadmeStatusColIndex();
		if (colIndex == -1)
			return;
		int[] rows = getRowsForRunID(runId,OTDbbView.RUN_ID);
		for (int i = 0; i < rows.length; i++) {
			// increment colindex because one
			// default column was removed.
			// This should be fixed properly
			view_.setValueAt(status, rows[i], README_STATUS_COLUMN);
		}
		if (rows.length > 0)
			currentRow_ = rows[0];
		setCurrentSelection();
	}

	public int getReadmeStatusColIndex() {
		JTable table = view_.getTable().getTable();
		TableColumnModel tblMdl = table.getColumnModel();
		try {
			int colIndex = tblMdl.getColumnIndex(README_STATUS_COLUMN);
			colIndex++;
			return colIndex;
		} catch (IllegalArgumentException e) {
			return -1;
		}
	}

	private void removeToolBarButtons() {
		while (toolBar_.getComponentAtIndex(0) != null) {
			JButton c = (JButton) toolBar_.getComponentAtIndex(0);
			if (c != null)
				toolBar_.remove(c);

		}
	}

	public void finalize() throws Throwable {
		// This cannot use stdlog_.debug() as it gives the closed appender
		// error message from log4j
		System.out.println("FINALIZE - - ot.gui.QueueItemsBView - - FINALIZE");
		super.finalize();
	}

	public void makeComponentsAavailableForGC() {
		view_.removeGlassPaneMouseListener();
		removeToolBarButtons();
		super.removeListeners();
		executionAction_.unsubscribe();
	}

	public JFrame getParentFrame() {
		Container c = view_.getTable().getTable().getTopLevelAncestor();
		stdlog_.debug("ot.gui.QueueItemsBView: " + c);
		return (JFrame) c;
	}

	public long[] getSelectedOpenQueueIds() {
		long[] ids = new long[1];
		ids[0] = queue.getId();
		return ids;
	}

	/** Builds the toolBar */
	protected void buildToolBar() {

		// toolBar_ = new JToolBar();
		/** cmavros 1709/OT */
		super.buildToolBar();
		addFirstBunchButtons();
		addMoveButtons();
		addLastButtons();

		// setEnabled(false);
		// view_.addListSelectionListener(new TableListSelectionListener());
	}

	/** Builds the toolBar */
	protected void buildToolBarNoMove() {
		super.buildToolBar();
		addFirstBunchButtons();
		addLastButtons();

		// setEnabled(false);
		// view_.addListSelectionListener(new TableListSelectionListener());
	}

	/**
	 * void
	 */
	private void addLastButtons() {
		toolBar_.add(executionAction_);
		toolBar_.add(copyActionListener_);
		toolBar_.add(pasteActionListener_);
		toolBar_.add(exportAction_);
		toolBar_.add(obHistoryAction);
		toolBar_.add(findingChartAction);
		if (AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
			toolBar_.add(dumpAdpAdmDataAction_);
		}
	}

	/**
	 * void
	 */
	private void addFirstBunchButtons() {
		executionAction_ = new ExecutionSequenceAction(view_,
				"Execution Sequence", execSequence_);
		removeAction_ = new RemoveQueueItemsAction(this);
		openCommentsAction_ = new OpenCommentsAction(this);
		exportAction_ = new ExportBOAction(Media.DBASE, view_, "Export...",
				true);

		if (AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
			dumpAdpAdmDataAction_ = new DumpMaskAdpAdmData(view_);

		}

		commentButton = new JButton("Comment...");
		verifyButton = new JButton("Verify");
		copyActionListener_ = new CopyActionListener();
		pasteActionListener_ = new PasteActionListener();
		if (!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
			toolBar_.add(displayAction);
		}
		markAction_ = new MarkStatusAction();
		markButton = toolBar_.add(markAction_);
		MouseListener popupListener = new PopupListener();
		markButton.addMouseListener(popupListener);

		if (AppConfig.getAppConfig().getBoolean(OHSKeys.USGEnabledKey)) {
			markReadmeAction_ = new MarkReadmeStatusAction();
			markReadmeButton = toolBar_.add(markReadmeAction_);
			popupListener = new PopupReadmeStatusListener();
			markReadmeButton.addMouseListener(popupListener);
		}

		toolBar_.add(removeAction_);
		if (!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
			toolBar_.add(openCommentsAction_);
		}
	}

	/**
	 * void
	 */
	private void addMoveButtons() {
		toolBar_.add(moveUpAction);
		toolBar_.add(moveDownAction);
	}

	public void setEnabled(boolean enabled) {
		displayAction.setEnabled(enabled); // Sepetate action
		removeAction_.setEnabled(enabled); // inline action
		if (AppConfig.getAppConfig().getBoolean(OHSKeys.USGEnabledKey)) {
			markButton.setEnabled(enabled); // Seperate action
		}
		if (AppConfig.getAppConfig().getBoolean(OHSKeys.USGEnabledKey)) {
			markReadmeButton.setEnabled(enabled); // Seperate action
		}
		openCommentsAction_.setEnabled(enabled); // Seperate Action
		deselectAllAction.setEnabled(enabled); // inline
		executionAction_.setEnabled(enabled);
		// must have reference to execSequence
		moveUpAction.setEnabled(enabled); // inline
		moveDownAction.setEnabled(enabled);
		copyActionListener_.setEnabled(enabled); // inline
		pasteActionListener_.setEnabled(true);
		// should always be able to paste
		reload.setEnabled(enabled);
		exportAction_.setEnabled(enabled);
		obHistoryAction.setEnabled(enabled);
		findingChartAction.setEnabled(enabled);
		if (AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
			dumpAdpAdmDataAction_.setEnabled(enabled);
		}

	}

	public Action getReloadQueueItemsAction() {
		return reloadQueueItemsAction_;
	}

	public Action getExecutionAction() {
		return executionAction_;
	}

	public Action getRemoveAction() {
		return removeAction_;
	}

	public Action getMarkAction() {
		return markAction_;
	}

	public Action getOpenCommentsAction() {
		return openCommentsAction_;
	}

	public Action getExportAction() {
		return exportAction_;
	}

	/**
	 * Return the action for use when buildig menus
	 * 
	 * @return Action
	 */

	public Action getDumpAdpAdmDataAction() {
		return dumpAdpAdmDataAction_;
	}

	/**
	 * Set the new Queue that will be displayed into the view
	 */
	public void updateView(Queue newQueue) throws SQLException {
		queue = newQueue;
		queueIDTF.set(new Long(Config.getCfg().uniqueToTableId(queue.getId())));
		setOrderDirtyFlag(false);
         if (view_.getSortPanel()!= null) {
             view_.getSortPanel().reset();      
            seqWidget.actionPerformed(new ActionEvent(this, 0, null));
         }
		startQuery();
	}

	/** Appends selected QueueItems to execution sequnce */
	public void appendOBtoES() {
		ExecutionSequenceBView execView = ExecutionSequenceBView.getInstance();
		execView.appendOBs(getSelectedOBIDs(false));
	}

	/** copy to OTClipboard */
	private void copyToOTClipboard() {
		long[] set = getSelectedOBIDs(false);
		Long[] content = new Long[set.length];
		for (int i = 0; i < set.length; i++) {
			content[i] = new Long(set[i]);
		}
		OTClipboard.insert(content);
	}

	/** Paste from OTClipboard */
	private void pasteFromOTClipboard() {
		Object[] content = OTClipboard.getContent();
		long[] set = new long[content.length];
		for (int i = 0; i < content.length; i++) {
			set[i] = ((Long) content[i]).longValue();
		}
		appendOBs(set);
	}

	/**
	 * Appends OBs to the view
	 * 
	 * @param obIds
	 *            specifies the id of obs to be added to the view
	 * 
	 * NOTE: should never add OBs to a queue that have a status of "-" or "D",
	 * this is however difficult to check here without loading the OB from the
	 * database (SLOW!) It is assumed that that check has already been made (at
	 * the Repository Browser level) and that all ids listed here can be safely
	 * added to the queue.
	 */
	public boolean appendOBs(long[] obIds) {
		Vector setOBIDs = new Vector();
		int option = -1;
		StringBuffer messageBuffer = new StringBuffer();
		int numFailed = 0;
		for (int j = 0; j < obIds.length; j++) {
			System.out.println("OB IDs " + obIds[j]);
			if (queue.inQueueItems(obIds[j])) {
				if (numFailed >= 1) {
					messageBuffer.append("\n");
				}
				messageBuffer.append(Config.getCfg().uniqueToTableId(obIds[j]));
				numFailed++;
			} else if (setOBIDs.contains(new Long(obIds[j])) == false) {
				setOBIDs.addElement(new Long(obIds[j]));
				/** 03/12/01 cmavros Flag a change has happened to the Queue */
				queue.setUpdate(true);
			}
		}
		// if an OB must be appended
		if (setOBIDs.size() > 0) {
			try {
				long[] uniqueobIds = new long[setOBIDs.size()];
				for (int i = 0; i < uniqueobIds.length; i++) {
					Long id = (Long) setOBIDs.elementAt(i);
					uniqueobIds[i] = id.longValue();
				}
				QueueManager.appendObsToQueue(queue, queue.getAllUniqueOBIds(),
						uniqueobIds, view_.getRows(), isOutOfOrder());
				queue.appendQueueItems(uniqueobIds, getTypes(uniqueobIds));
				setOrderDirtyFlag(false);
                if (view_.getSortPanel()!= null) {
				 view_.getSortPanel().reset();
               
				// if(seqWidget == null)
				// System.out.println(" >>>>>>>>> There is a problem");
				seqWidget.actionPerformed(new ActionEvent(this, 0, null));
               }
				// sqlStartAction();
				startQuery();
			} catch (ObjectIOException e) {
				stdlog_.error(e);
			}
		}

		if (numFailed == 1) {
			String message = "OB " + messageBuffer.toString()
					+ " already inserted into the Queue";
			JOptionPane.showMessageDialog(null, message, "Warning",
					JOptionPane.WARNING_MESSAGE);
		}
		if (numFailed > 1) {
			log_.clearTextArea();
			String message = "The following OBS are already inserted into the queue\n"
					+ messageBuffer.toString();
			log_.setTextArea(message);
			log_.setVisible(true);
		}
		if (setOBIDs.size() > 0) {
			return true;
		}
		return false;

	}

    
    /**
     * Appends OBs to the view
     * 
     * @param obIds
     *            specifies the id of obs to be added to the view
     * 
     * NOTE: should never add OBs to a queue that have a status of "-" or "D",
     * this is however difficult to check here without loading the OB from the
     * database (SLOW!) It is assumed that that check has already been made (at
     * the Repository Browser level) and that all ids listed here can be safely
     * added to the queue.
     */
    public boolean appendQueueItems(QueueItem[] queueItems) {
        queue.setQueueList(queueItems);
        try {
            QueueManager.appendQueueItemsToQueue(queue);
        } catch (ObjectIOException e) {
            stdlog_.error(e);
        }     
        setOrderDirtyFlag(false);
        seqWidget.actionPerformed(new ActionEvent(this, 0, null));
        startQuery();             
        return true;

    }

	/**
	 * Appends OBs to the view
	 * 
	 * @param obIds
	 *            specifies the id of obs to be added to the view
	 * 
	 * NOTE: should never add OBs to a queue that have a status of "-" or "D",
	 * this is however difficult to check here without loading the OB from the
	 * database (SLOW!) It is assumed that that check has already been made (at
	 * the Repository Browser level) and that all ids listed here can be safely
	 * added to the queue.
	 */
	public boolean appendOBsWithoutMessge(long[] obIds) {
		Vector setOBIDs = new Vector();
		int option = -1;
		StringBuffer messageBuffer = new StringBuffer();
		int numFailed = 0;
		for (int j = 0; j < obIds.length; j++) {
			System.out.println("OB IDs " + obIds[j]);
			if (queue.inQueueItems(obIds[j])) {
				numFailed++;
			} else if (setOBIDs.contains(new Long(obIds[j])) == false) {
				setOBIDs.addElement(new Long(obIds[j]));
				/** 03/12/01 cmavros Flag a change has happened to the Queue */
				queue.setUpdate(true);
			}
		}
		// if an OB must be appended
		if (setOBIDs.size() > 0) {
			try {
				long[] uniqueobIds = new long[setOBIDs.size()];
				for (int i = 0; i < uniqueobIds.length; i++) {
					Long id = (Long) setOBIDs.elementAt(i);
					uniqueobIds[i] = id.longValue();
				}
				QueueManager.appendObsToQueue(queue, queue.getAllUniqueOBIds(),
						uniqueobIds, queue.getQueueList().length,
						isOutOfOrder());
				queue.appendQueueItems(uniqueobIds, getTypes(uniqueobIds));
				setOrderDirtyFlag(false);
                 if (view_.getSortPanel()!= null) {
                     view_.getSortPanel().reset();
                    	// if(seqWidget == null)
				// System.out.println(" >>>>>>>>> There is a problem");
				seqWidget.actionPerformed(new ActionEvent(this, 0, null));
                 }
				// sqlStartAction();
				startQuery();
			} catch (ObjectIOException e) {
				stdlog_.error(e);
			}
		}

		if (setOBIDs.size() > 0) {
			return true;
		}
		return false;

	}

	// ----------------------------------------
	// Private Methods
	// ----------------------------------------

	/**
	 * Build DB browser view: SELECT panel
	 * 
	 * <br>
	 * ChangeLog: Gorgio Passera
	 * 
	 * @exception SQLException
	 *                JDBC exception occurs
	 * @exception RemoteException
	 *                RMI exception occurs
	 * @exception MalformedURLException
	 *                malformed url i.e. servername//port
	 * @exception NotBoundException
	 *                client could not bind to server
	 */
	public void buildColumns() throws NotBoundException, MalformedURLException,
			SQLException, RemoteException {

		// collect the default SQLChunks from super.
		String label_name;
		String column_name;
	

		label_name = "OB ID";
		column_name = "ob_id";
		obIDChunk = new DbbSqlChunk(ob_table, column_name, label_name);
		obIDMaskChunk = new DbbSqlChunk(obsblk_masks_table, column_name,
				label_name);
		adpMaskChunk = new DbbSqlChunk(obsblk_masks_table, "adp_index",
				"Channel Number");

		engine_.addDefaultColumnsChunk(obIDChunk);
		// engine_.addDefaultColumnsChunk(obIDMaskChunk);
		// obIDMaskChunk.setActive(false);
		// engine_.addDefaultColumnsChunk(adpMaskChunk);
		// adpMaskChunk.setActive(false);

		addDefaultColumns();
		colPanel_ = new DbbPanel("Selected Columns", 7);
		DbbSqlChunk c;
		DbbCheckBox w;

		// OB Name
		label_name = "OB Name";
		column_name = "item_name";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				ob_table, column_name, label_name)));

		// ProgID
		label_name = "ProgID";
		column_name = "progid";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				sr_table, column_name, label_name, String.class)));

		// PI (surname)
		label_name = "PI";
		column_name = "surname_dl";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				eu_table, column_name, label_name)));

		// Target
		label_name = "Target";
		column_name = "item_name";
		c = new DbbSqlChunk(tp_table, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		// RA
		label_name = "RA";
		column_name = "ra";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				tp_table, column_name, label_name, DbaseRA.class)));

		// Dec
		label_name = "Dec";
		column_name = "dec";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				tp_table, column_name, label_name, DbaseDec.class)));

		// Instrument widget
		label_name = "Instrument";
		column_name = "instrument";
		c = new DbbSqlChunk(od_table, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		// User's priority
		label_name = "UsrP";
		column_name = "userPriority";
		c = new DbbSqlChunk(ob_table, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		// Template widget
		label_name = "Templ";
		column_name = "item_name";
		c = new DbbSqlChunk(ts_table, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		// Seeing
		label_name = "Seeing";
		column_name = "seeing";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				cs_table, column_name, label_name)));

		// Sky Tran
		label_name = "SkyTran";
		column_name = "sky_transparency";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				cs_table, column_name, label_name)));

		// Airmass
		label_name = "Airmass";
		column_name = "airmass";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				cs_table, column_name, label_name)));

		// FLI
		label_name = "FLI";
		column_name = "fli";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				cs_table, column_name, label_name)));

		// MoonDis
		label_name = "MoonDis";
		column_name = "moon_distance";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				cs_table, column_name, label_name, MoonDistance.class)));

		// StrehlRatio
		label_name = "Strehl";
		column_name = "strehl_ratio";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				cs_table, column_name, label_name)));

		label_name = "ExecTime";
		column_name = "execution_time";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				ob_table, column_name, label_name, DbbExecutionTime.class)));

		// Filter
		// Optical elements widget
		label_name = "OptElem";
		column_name = "value";
		c = new DbbSqlChunk(pa_table, column_name, label_name);
		w = new DbbCheckBox(label_name, c);

		DbbSqlOperator op = null;
		try {
			op = new DbbSqlOperator(">");
		} catch (Exception e) {
			stdlog_.error(e);
		}
		column_name = "subtype";
		c = new DbbSqlChunk(pa_table, column_name, op, Integer.class);
		c.setValue("0");
		w.addSqlChunk(c);

		colPanel_.addWidget(w);

		// Rank class
		label_name = "RankClass";
		column_name = "rank_class";
		c = new DbbSqlChunk(sr_table, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		// QCGrade
		label_name = "QC Grade";
		column_name = "obs_grade";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				ob_table, column_name, label_name)));

		label_name = "Finding Charts";
		column_name = "fc_name";

		if (AppConfig.getAppConfig().getBoolean(OHSKeys.USGEnabledKey)) {
			label_name = "Readme Version";
			column_name = "version";
			c = new DbbSqlChunk(readme, column_name, label_name);
			w = new DbbCheckBox(label_name, c);
			colPanel_.addWidget(w);
			column_name = "status";
			c = new DbbSqlChunk(readme, column_name, README_STATUS_COLUMN);
			w = new DbbCheckBox(README_STATUS_COLUMN, c);
			colPanel_.addWidget(w);
		}

		label_name = "Sidereal Time Start";
		column_name = "start_interval";
		c = new DbbSqlChunk(st_tis_table, column_name, label_name,
				DbbHHMMDateTime.class);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		label_name = "Sidereal Time End";
		column_name = "end_interval";
		c = new DbbSqlChunk(st_tis_table, column_name, label_name,
				DbbHHMMDateTime.class);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		label_name = "Baseline";
		column_name = "baseline_name";
		c = new DbbSqlChunk(baselines, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		label_name = "Ephemeris File";
		column_name = "file_name";
		c = new DbbSqlChunk(ephemeris_file, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		// Mask Status
		/**
		 * cmavros 06/Nov/01 Only display option if Masktracker is Enabled.
		 */
		label_name = "Mask Status";
		column_name = "mask_status";
		maskStatusBox = new DbbCheckBox(label_name, new DbbSqlChunk(
				masks_table, column_name, label_name, MaskStatusType.class));
		colPanel_.addWidget(maskStatusBox);

		label_name = "Mask Slot Num";
		column_name = "mask_slot_num";
		maskSlotNumBox = new DbbCheckBox(label_name, new DbbSqlChunk(
				masks_table, column_name, label_name));
		colPanel_.addWidget(maskSlotNumBox);

		label_name = "Channel";
		column_name = "adp_index";
		maskChannelBox = new DbbCheckBox(label_name, new DbbSqlChunk(
				obsblk_masks_table, column_name, label_name));
		colPanel_.addWidget(maskChannelBox);

        // Type
        column_name = "type";
        label_name = view_.TYPE;
        DbbCheckBox typeCheckBox =  new DbbCheckBox(label_name, new DbbSqlChunk(ob_table, column_name,
                label_name));
        colPanel_.addWidget(typeCheckBox);   
       


        label_name = view_.QUEUE_ID;
        column_name = "sch_id";
        
        c = new DbbSqlChunk(si_table, column_name, label_name);
        DbbCheckBox queueIdWidget = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(queueIdWidget);
        typeCheckBox.set("1");
        queueIdWidget.set("1");
        typeCheckBox.fireDbbWidgetAction();
        queueIdWidget.fireDbbWidgetAction();     
        queueIdWidget.setVisible(false);
        typeCheckBox.setVisible(false);
   
      
		view_.setColumnsPanel(colPanel_, "North");
        
        view_.hideColumn(view_.QUEUE_ID);
        view_.hideColumn(view_.RUN_ID);

        //view_.hideColumn(view_.TYPE);

	}

	/** Build DB browser view: rows selection panel */
	public void buildRows() {
		rowPanel_ = new DbbPanel("Selection Criteria", 5);
		DbbSqlChunk c;

		String label_name;
		String column_name;

		DbbSqlOperator op_equal = null;
		DbbSqlOperator op_min = null;
		DbbSqlOperator op_max = null;
		DbbSqlOperator op_like = null;
		DbbSqlOperator op_in = null;

		try {
			op_equal = new DbbSqlOperator("=");
			op_min = new DbbSqlOperator("<=");
			op_max = new DbbSqlOperator(">=");
			op_like = new DbbSqlOperator("like");
			op_in = new DbbSqlOperator("in");
		} catch (Exception e) {
			stdlog_.error(e);
		}

		// Queue ID
		label_name = "Queue ID";
		column_name = "sch_id";

		c = new DbbSqlChunk(si_table, column_name, label_name, op_equal,
				Integer.class);
		queueIDTF = new DbbTextField(label_name, c);
		rowPanel_.addWidget(queueIDTF);
		queueIDTF.set(new Long(Config.getCfg().uniqueToTableId(queue.getId())));
		queueIDTF.setEnabled(false);
		view_.setWherePanel(rowPanel_, "North", false);
        
	}

	/**
	 * Build DB browser view: SELECT panel
	 * 
	 * <br>
	 * ChangeLog: Gorgio Passera
	 * 
	 * @exception SQLException
	 *                JDBC exception occurs
	 * @exception RemoteException
	 *                RMI exception occurs
	 * @exception MalformedURLException
	 *                malformed url i.e. servername//port
	 * @exception NotBoundException
	 *                client could not bind to server
	 */
	public void buildOrangColumns() throws NotBoundException,
			MalformedURLException, SQLException, RemoteException {

		// collect the default SQLChunks from super.
		String label_name;
		String column_name;

		label_name = "OB ID";
		column_name = "ob_id";
		obIDChunk = new DbbSqlChunk(ob_table, column_name, label_name);
		obIDMaskChunk = new DbbSqlChunk(obsblk_masks_table, column_name,
				label_name);
		adpMaskChunk = new DbbSqlChunk(obsblk_masks_table, "adp_index",
				"Channel Number");

		engine_.addDefaultColumnsChunk(obIDChunk);
		// engine_.addDefaultColumnsChunk(obIDMaskChunk);
		// obIDMaskChunk.setActive(false);
		// engine_.addDefaultColumnsChunk(adpMaskChunk);
		// adpMaskChunk.setActive(false);

        // Status
        column_name = "status";
        label_name = "Status";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(ob_table, column_name,
                label_name));

        column_name = "run_id";
        label_name = view_.RUN_ID;
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(sr_table, column_name,
                label_name));
        
		colPanel_ = new DbbPanel("Selected Columns", 7);
		DbbSqlChunk c;
		DbbCheckBox w;

		// OB Name
		label_name = "OB Name";
		column_name = "item_name";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				ob_table, column_name, label_name)));

		// ProgID
		label_name = "ProgID";
		column_name = "progid";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				sr_table, column_name, label_name, String.class)));

		// PI (surname)
		label_name = "PI";
		column_name = "surname_dl";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				eu_table, column_name, label_name)));

		// Target
		label_name = "Target";
		column_name = "item_name";
		c = new DbbSqlChunk(tp_table, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		// RA
		label_name = "RA";
		column_name = "ra";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				tp_table, column_name, label_name, DbaseRA.class)));

		// Dec
		label_name = "Dec";
		column_name = "dec";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				tp_table, column_name, label_name, DbaseDec.class)));

		// Instrument widget
		label_name = "Instrument";
		column_name = "instrument";
		c = new DbbSqlChunk(od_table, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		// User's priority
		label_name = "UsrP";
		column_name = "userPriority";
		c = new DbbSqlChunk(ob_table, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		// Template widget
		label_name = "Templ";
		column_name = "item_name";
		c = new DbbSqlChunk(ts_table, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		// Seeing
		label_name = "Seeing";
		column_name = "seeing";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				cs_table, column_name, label_name)));

		// Sky Tran
		label_name = "SkyTran";
		column_name = "sky_transparency";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				cs_table, column_name, label_name)));

		// Airmass
		label_name = "Airmass";
		column_name = "airmass";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				cs_table, column_name, label_name)));

		// FLI
		label_name = "FLI";
		column_name = "fli";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				cs_table, column_name, label_name)));

		// MoonDis
		label_name = "MoonDis";
		column_name = "moon_distance";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				cs_table, column_name, label_name, MoonDistance.class)));

		// StrehlRatio
		label_name = "Strehl";
		column_name = "strehl_ratio";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				cs_table, column_name, label_name)));

		label_name = "ExecTime";
		column_name = "execution_time";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				ob_table, column_name, label_name, DbbExecutionTime.class)));

		// Filter
		// Optical elements widget
		label_name = "OptElem";
		column_name = "value";
		c = new DbbSqlChunk(pa_table, column_name, label_name);
		w = new DbbCheckBox(label_name, c);

		DbbSqlOperator op = null;
		try {
			op = new DbbSqlOperator(">");
		} catch (Exception e) {
			stdlog_.error(e);
		}
		column_name = "subtype";
		c = new DbbSqlChunk(pa_table, column_name, op, Integer.class);
		c.setValue("0");
		w.addSqlChunk(c);

		colPanel_.addWidget(w);

		// Rank class
		label_name = "RankClass";
		column_name = "rank_class";
		c = new DbbSqlChunk(sr_table, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		// QCGrade
		label_name = "QC Grade";
		column_name = "obs_grade";
		colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
				ob_table, column_name, label_name)));

		label_name = "Finding Charts";
		column_name = "fc_name";

		if (AppConfig.getAppConfig().getBoolean(OHSKeys.USGEnabledKey)) {
			label_name = "Readme Version";
			column_name = "version";
			c = new DbbSqlChunk(readme, column_name, label_name);
			w = new DbbCheckBox(label_name, c);
			colPanel_.addWidget(w);
			column_name = "status";
			c = new DbbSqlChunk(readme, column_name, README_STATUS_COLUMN);
			w = new DbbCheckBox(README_STATUS_COLUMN, c);
			colPanel_.addWidget(w);
		}

		label_name = "Sidereal Time Start";
		column_name = "start_interval";
		c = new DbbSqlChunk(st_tis_table, column_name, label_name,
				DbbHHMMDateTime.class);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		label_name = "Sidereal Time End";
		column_name = "end_interval";
		c = new DbbSqlChunk(st_tis_table, column_name, label_name,
				DbbHHMMDateTime.class);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		label_name = "Baseline";
		column_name = "baseline_name";
		c = new DbbSqlChunk(baselines, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		label_name = "Ephemeris File";
		column_name = "file_name";
		c = new DbbSqlChunk(ephemeris_file, column_name, label_name);
		w = new DbbCheckBox(label_name, c);
		colPanel_.addWidget(w);

		// Mask Status
		/**
		 * cmavros 06/Nov/01 Only display option if Masktracker is Enabled.
		 */
		label_name = "Mask Status";
		column_name = "mask_status";
		maskStatusBox = new DbbCheckBox(label_name, new DbbSqlChunk(
				masks_table, column_name, label_name, MaskStatusType.class));
		colPanel_.addWidget(maskStatusBox);

		label_name = "Mask Slot Num";
		column_name = "mask_slot_num";
		maskSlotNumBox = new DbbCheckBox(label_name, new DbbSqlChunk(
				masks_table, column_name, label_name));
		colPanel_.addWidget(maskSlotNumBox);

		label_name = "Channel";
		column_name = "adp_index";
		maskChannelBox = new DbbCheckBox(label_name, new DbbSqlChunk(
				obsblk_masks_table, column_name, label_name));
		colPanel_.addWidget(maskChannelBox);
        
//       sequence
        label_name = view_.ORANG_RANK;
        column_name = "rank";

        c = new DbbSqlChunk(si_table, column_name, label_name);
        DbbCheckBox rankWidget = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(rankWidget);
        
        // sequence
        label_name = view_.ORANG_RANK_HIDE;
        column_name = "rank";

        c = new DbbSqlChunk(si_table, column_name, label_name);
        DbbCheckBox rankWidgetHide = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(rankWidgetHide);

        

        label_name = view_.QUEUE_ID;
        column_name = "sch_id";
        
        c = new DbbSqlChunk(si_table, column_name, label_name);
        DbbCheckBox queueIdWidget = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(queueIdWidget);     
        
        // Type
        column_name = "type";
        label_name = view_.TYPE;
        DbbCheckBox typeCheckBox =  new DbbCheckBox(label_name, new DbbSqlChunk(ob_table, column_name,
                label_name));
        colPanel_.addWidget(typeCheckBox);     
        
      
		view_.setColumnsPanel(colPanel_, "North");
          setHideCol(typeCheckBox);      
          setHideCol(queueIdWidget);
          setHideCol(rankWidgetHide);
          //view_.hideColumn(OTDbbView.TYPE);
          view_.hideColumn(OTDbbView.QUEUE_ID);
          view_.hideColumn(OTDbbView.RUN_ID);
          view_.hideColumn(OTDbbView.ORANG_RANK_HIDE);
	}

    /**
     * @param checkBox void
     */
    private void setHideCol(DbbCheckBox checkBox) {
        checkBox.set("1");
        checkBox.fireDbbWidgetAction();
        checkBox.setVisible(false);
    }

	/** Build DB browser view: sort selection panel */
	public void buildSort() {
		sortPanel_ = new DbbPanel("Sorting Criteria", 1);
		DbbSqlChunk c;
		DbbArrowButton w;
		String column_name;
		String label_name;

		// sequence
		label_name = "sequence";
		column_name = "sequence";

		c = new DbbSqlChunk(si_table, column_name, label_name);
		seqWidget = new DbbArrowButton(label_name, c, 1);
		sortPanel_.addWidget(seqWidget);
		seqWidget.actionPerformed(new ActionEvent(this, 0, null));

		// OB ID
		label_name = "OB ID";
		column_name = "ob_id";

		c = new DbbSqlChunk(ob_table, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		// OB NAME
		label_name = "OB Name";
		column_name = "item_name";
		c = new DbbSqlChunk(ob_table, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		// ////////////////////////
		label_name = "ProgID";
		column_name = "progid";

		c = new DbbSqlChunk(sr_table, column_name, label_name);

		// /////// It was a bug /////////
		// label_name = "ProgID";
		// column_name = "proposalID";

		// c = new DbbSqlChunk(op,
		// column_name,
		// label_name);

		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		// RA
		label_name = "RA";
		column_name = "ra";

		c = new DbbSqlChunk(tp_table, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		// DEC
		label_name = "Dec";
		column_name = "dec";

		c = new DbbSqlChunk(tp_table, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		// seeing
		label_name = "Seeing";
		column_name = "seeing";

		c = new DbbSqlChunk(cs_table, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		// Airmass
		label_name = "Airmass";
		column_name = "airmass";

		c = new DbbSqlChunk(cs_table, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		// FLI
		label_name = "FLI";
		column_name = "fli";

		c = new DbbSqlChunk(cs_table, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		// Moon Distance
		label_name = "MoonDis";
		column_name = "moon_distance";

		c = new DbbSqlChunk(cs_table, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		// Sky Trans
		label_name = "SkyTran";
		column_name = "sky_transparency";

		c = new DbbSqlChunk(cs_table, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		// RankClass
		label_name = "RankClass";
		column_name = "rank_class";
		c = new DbbSqlChunk(sr_table, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		// Execution Time
		// cmavros added 10/Sept/2001
		label_name = "ExecTime";
		column_name = "execution_time";
		c = new DbbSqlChunk(ob_table, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		label_name = "Finding Charts";
		column_name = "fc_name";
		c = new DbbSqlChunk(finds_view, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		label_name = "Mask Status";
		column_name = "mask_status";
		w = new DbbArrowButton(label_name, new DbbSqlChunk(masks_table,
				column_name, label_name));
		sortPanel_.addWidget(w);

		label_name = "Mask Slot Num";
		column_name = "mask_slot_num";
		w = new DbbArrowButton(label_name, new DbbSqlChunk(masks_table,
				column_name, label_name));
		sortPanel_.addWidget(w);

		label_name = "Channel";
		column_name = "adp_index";
		w = new DbbArrowButton(label_name, new DbbSqlChunk(obsblk_masks_table,
				column_name, label_name));
		sortPanel_.addWidget(w);

		label_name = "Sidereal Time Start";
		column_name = "start_interval";
		c = new DbbSqlChunk(st_tis_table, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		label_name = "Sidereal Time End";
		column_name = "end_interval";
		c = new DbbSqlChunk(st_tis_table, column_name, label_name);

		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		label_name = "Baseline";
		column_name = "baseline_name";
		c = new DbbSqlChunk(baselines, column_name, label_name);
		w = new DbbArrowButton(label_name, c);
		sortPanel_.addWidget(w);

		view_.setSortPanel(sortPanel_, "West");
		view_.setSortPanel(sortPanel_, "West", true);
	}

	/** Build DB browser view: sort selection panel */
	public void buildOrangSort() {
		sortPanel_ = new DbbPanel("Sorting Criteria", 1);
		DbbSqlChunk c;
		DbbArrowButton w;
		String column_name;
		String label_name;

		// sequence
		label_name = OTDbbView.ORANG_RANK;
		column_name = "rank";

		c = new DbbSqlChunk(si_table, column_name, label_name);
		seqWidget = new DbbArrowButton(label_name, c, 1);
		sortPanel_.addWidget(seqWidget);
		seqWidget.fireDbbWidgetAction();
		view_.setSortPanel(sortPanel_, "West");
		view_.setSortPanel(sortPanel_, "West", true);
		sortPanel_.setVisible(false);
	}

	private class TableListSelectionListener implements ListSelectionListener {
		public TableListSelectionListener() {
			stdlog_
					.debug("ot.gui.QueueItemsBView: @@@@@@@@@@@@@@@ Constructor called");
		}

		public void valueChanged(ListSelectionEvent ev) {
			if (ev.getValueIsAdjusting()) {
				return;
			}
			ListSelectionModel lsm = (ListSelectionModel) ev.getSource();
			int selectedRow = lsm.getMinSelectionIndex();
			stdlog_.debug("ot.gui.QueueItemsBView: Value Changed " + ev);
			if (!lsm.isSelectionEmpty() && selectedRow >= 0) {
				setEnabled(true);
			} else {
				setEnabled(false);
			}
		}
	}

	/**
	 * This class is uesd to bring up a popup menu PopupMenu should be displayed
	 * when the rigth mouse button is pressed. In this case we determine the
	 * coordinates of the click to see if its occurs or not on a tree node or
	 * leaf
	 */
	public class PopupListener extends MouseAdapter {
		JPopupMenu popup;

		public void mousePressed(MouseEvent e) {
			// if (e.isPopupTrigger())
			if (e.getID() == MouseEvent.MOUSE_PRESSED) {
				createPopupMenu();
				popup.show(e.getComponent(), e.getX(), e.getY());
			}
		}

		private void createPopupMenu() {
			popup = new JPopupMenu();
			for (int i = 0; i < CalibrationBlock.markLabels.length; i++) {
				final int j = i;
				JMenuItem crrItem = new JMenuItem(
						CalibrationBlock.markLabels[i]);
				popup.add(crrItem);
				crrItem.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						markStatus(j);
						// queue.setUpdate(true);
					}
				});
			}
		}
	}

	/**
	 * This class is uesd to bring up a popup menu PopupMenu should be displayed
	 * when the rigth mouse button is pressed. In this case we determine the
	 * coordinates of the click to see if its occurs or not on a tree node or
	 * leaf
	 */
	public class PopupReadmeStatusListener extends MouseAdapter {
		JPopupMenu popup;

		public void mousePressed(MouseEvent e) {
			// if (e.isPopupTrigger())
			if (e.getID() == MouseEvent.MOUSE_PRESSED) {
				createPopupMenu();
				popup.show(e.getComponent(), e.getX(), e.getY());
			}
		}

		private void createPopupMenu() {
			popup = new JPopupMenu();
			String[] items = Readme.statusValues_.keys();
			for (int i = 0; i < items.length; i++) {
				final int j = i;
				JMenuItem crrItem = new JMenuItem(items[i]);
				popup.add(crrItem);
				crrItem.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						ObjectManager objMgr = ObjectManager.getObjectManager();
						long[] obIDs = getSelectedOBIDs(false);
						CalibrationBlock cb;
						try {
							cb = (CalibrationBlock) objMgr.getBusObj(
									Media.DBASE, obIDs[0], Config.getCfg()
											.getClassFromId(obIDs[0]));
							ObservingRun obsRun = cb.getObsRun();
							QueueView.markReadmeStatus(obsRun, j);
							// setValueAt
						} catch (ObjectNotFoundException e1) {
							ErrorMessages.announceNoObject(getParent(), e1);
						} catch (ObjectIOException e1) {
							ErrorMessages.announceIOError(getParent(), e1);
						} catch (IOException e1) {
							ErrorMessages.announceIOError(getParent(), e1);
						}
					}
				});
			}
		}
	}

	private class QueryTerminatedListener implements DbbQueryTerminatedListener {

	
        public void queryTerminatedAction(DbbQueryTerminated ev) {
			setCurrentSelection();
           
//			JTable table = view_.getTable().getTable();
//            TableColumn queueColumn = table.getColumn(QUEUE_ID);
//            table.removeColumn(queueColumn);
//            TableColumn typeColumn = table.getColumn(TYPE_ID);
//            table.removeColumn(typeColumn);
//            if (!AppConfig.getAppConfig().getBoolean(
//                    OHSKeys.MaskTrackerEnabledKey)) {
//                TableColumn runIdColumn = table.getColumn(view_.RUN_ID);
//                table.removeColumn(runIdColumn);
//            }
		
		}
	}

	private class SortTerminatedListener implements DbbSortTerminatedListener {
		public void sortTerminatedAction(DbbSortTerminated ev) {

			view_.getTable().saveSortCriteriaToModel();
			setOrderDirtyFlag(true);
			stdlog_
					.debug("ot.gui.QueueItemsBView: !!!!!!! Setting sorted list");
			QueueItem[] qList = getQueueItems();
			long[] obIdList = new long[qList.length];
			for (int i = 0; i < qList.length; i++) {
				obIdList[i] = qList[i].getObId();
			}
			// queue.setQueueList(qList,destIndex);
			queue.sortQueueItems(obIdList);

			view_.getTable().saveSortCriteriaToModel();
		}
	}

	private class ReloadActionListener implements ActionListener {
		QueueItemsBView qbView_;

		protected ReloadActionListener(QueueItemsBView qbView) {
			qbView_ = qbView;
		}

		public void actionPerformed(ActionEvent e) {
			ReloadQueueAction reloadAction = new ReloadQueueAction(qbView_,
					"Revert");
			reloadAction.actionPerformed(e);
		}
	}

	/**
	 * Marks the status of an Observation Block
	 */
	private void markStatus(int i) {
		long[] OBIDs = getSelectedOBIDs(false);
		ObjectManager objMgr = ObjectManager.getObjectManager();
		boolean obDequeueRequired = QueueView.isNotQueueableStatus(i);
		// If this status to be set requirs dequeuing the selected OB,
		// inform the user that this status requires all selected OBs to
		// be removed from Queues prior to setting the status - continue y/n?
		if (!obDequeueRequired
				|| JOptionPane.showConfirmDialog(getParentFrame(),
						"Setting OBs to status '"
								+ CalibrationBlock.markLabels[i]
								+ "'\nwill remove them from all queues."
								+ "\nDo you wish to continue?",
						"Setting Status", JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.QUESTION_MESSAGE, null) == JOptionPane.YES_OPTION) {
			try {
				QueueView.markStatus(OBIDs, i);
			} catch (ObjectIOException e) {
				String message = "Error: " + e.getMessage();
				JOptionPane.showMessageDialog(getParentFrame(), message,
						"ERROR", JOptionPane.ERROR_MESSAGE);
				stdlog_.error(e);
			} catch (ObjectNotFoundException e) {
				String message = "Error: " + e.getMessage();
				JOptionPane.showMessageDialog(getParentFrame(), message,
						"ERROR", JOptionPane.ERROR_MESSAGE);
				stdlog_.error(e);
			}
		}
	}

	private class MaskInfoDbbCheckBox extends DbbCheckBox implements
			ItemListener {
		public MaskInfoDbbCheckBox(String label, DbbSqlChunk c) {
			super(label, c);
			((JCheckBox) widget_).getModel().addItemListener(this);
		}

		public void itemStateChanged(ItemEvent ev) {
			System.out.println("!!!!!!!!!!! State Changed Event " + ev);
			// if(ev.isAdjusting())
			// return;
			if (ev.getStateChange() == ItemEvent.SELECTED
					&& !maskStatusBox.isEmpty() || !maskSlotNumBox.isEmpty()) {
				System.out.println("Setting on mask ID Chunk data");
				obIDMaskChunk.setActive(true);
				adpMaskChunk.setActive(true);
				obIDChunk.setActive(false);
			} else {
				System.out.println("Setting off mask ID Chunk data");
				obIDMaskChunk.setActive(false);
				adpMaskChunk.setActive(false);
				obIDChunk.setActive(true);
				// maskStatusSortChunk.setActive(false);
			}
		}
	}

	private class CopyActionListener extends AbstractAction {

		public CopyActionListener() {
			super("Copy");
		}

		public void actionPerformed(ActionEvent e) {
			copyToOTClipboard();
		}

	}

	private class PasteActionListener extends AbstractAction {

		public PasteActionListener() {
			super("Paste");
		}

		public void actionPerformed(ActionEvent e) {
			pasteFromOTClipboard();
		}
	}

	protected void addDefaultColumns() throws NotBoundException,
			MalformedURLException, SQLException, RemoteException {
		// TODO Auto-generated method stub
		String label_name;
		String column_name;

	
		// Status
		column_name = "status";
		label_name = view_.STATUS;
		engine_.addDefaultColumnsChunk(new DbbSqlChunk(ob_table, column_name,
				label_name));

		column_name = "run_id";
		label_name = view_.RUN_ID;
		engine_.addDefaultColumnsChunk(new DbbSqlChunk(sr_table, column_name,
				label_name));

	}
}
