package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.Border;

import org.eso.ohs.core.dbb.client.DbbTable;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.sql.DbbQueryTerminated;
import org.eso.ohs.core.dbb.sql.DbbQueryTerminatedListener;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.actions.OrangVisiplotAction;
import org.eso.ohs.phase2.apps.ot.dbb.OTDbbView;
import org.eso.ohs.phase2.apps.ot.wizard.OrangModel;
import org.eso.ohs.phase2.orang.OBScheduleInfo;
import org.eso.ohs.phase2.visiplot.VisiPlotOptions;

import com.jeta.forms.components.image.ImageComponent;

public class QueueOrangView extends QueueView implements
        DbbQueryTerminatedListener {

    
	/**
	 * Visiplot Options set by ORANG 
	 */
	private VisiPlotOptions currentSettings_;
	
	
	/*
	 * ORANG Computed OB
	 */
	private OBScheduleInfo [] obs_;


    private OrangModel model;

    
    public QueueOrangView( OBScheduleInfo [] obs, OrangModel model) {
        super();
        obs_ = obs;
        this.model = model;
    }
	
	
	private ImageIcon loadImage(String imageName) {
        String imagePath = "org/eso/ohs/icons/" + imageName;
        try {

            ClassLoader classloader = getClass().getClassLoader();
            java.net.URL url = classloader.getResource(imagePath);
            if (url != null) {
                ImageIcon icon = new ImageIcon(url);
                return icon;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        throw new IllegalArgumentException("Unable to load image: " + imagePath);
    }

    protected JPanel getTitlePanel() {
        JPanel newPanel = new JPanel();
        BoxLayout box = new BoxLayout(newPanel, BoxLayout.X_AXIS);
        newPanel.setLayout(box);
        ImageComponent orange = new ImageComponent();
        Border empty = BorderFactory.createEmptyBorder(2, 2, 0, 5);
        orange.setBorder(empty);
        orange.setIcon(loadImage("orangeSmall.gif"));
        orange.setName("orange");
        newPanel.add(orange);
        newPanel.add(getQueuePropertiesView());
        return newPanel;
    }

    public void initQueueItemsBView() {
        obTreeView_ = new OBTreeView();
        QueueItemsBView queueDbbView = new QueueItemsBView();
        queueDbbView.initObjNoSort(queue_, obTreeView_);
        queueDbbView.initNoMove();
        queueItemsView_ = queueDbbView;
    }

    public void initQueueItemsRankedBView() {
        obTreeView_ = new OBTreeView();
        QueueItemsBView queueDbbView = new QueueItemsBView();
        queueDbbView.initOrangObj(queue_, obTreeView_);
        queueDbbView.initNoMove();
        queueItemsView_ = queueDbbView;

        // add rank incremental action
        DbbView actualOrangView = queueItemsView_.getView();
        actualOrangView.addQueryTerminatedListener(this);

    }



    /**
     * @param newQueue
     *            void
     */
    public void initQueueView(Queue newQueue, VisiPlotOptions settings) {
        currentSettings_ = settings;
        newQueue(newQueue);
        setQueuePropertiesView(new QueuePropertiesView(newQueue));
        initQueueItemsBView();
        buildMainMenu();
        buildMenu();
        createView();
        setTitle("Queue view: ORANG ");
        if (actionMenuDefault_ != null) {
            actionMenuDefault_.actionPerformedImpl(new ActionEvent(
                    actionMenuDefault_, 0, ""));
        }
    }

    protected void createVisiplotMenu(ObjectContainer container) {
        visiplotMenu = new JMenu("Visibility Plots");
        visiplotMenu.add(new OrangVisiplotAction(container, true, "All observation blocks",
                currentSettings_,obs_));
        visiplotMenu.add(new OrangVisiplotAction(container, false,
                "Selected observation blocks", currentSettings_,obs_));
    }

    /**
     * Builds a menu bar
     */
    protected void buildMenu() {
        try {
            buildFileMenu();
            buildEditMenu();
            buildOBsMenuNoMove();
            addSomeMenus();
        } catch (ObjectIOException e) {
            stdlog_.error(e);
        }
    }

   
    
    public boolean closeQueue() {
        OTViewManager otMgr = OTViewManager.getOTViewMgr();
        int index = otMgr.getQueueViews().indexOf(this);
        updateQueue();
        if (index != -1) {                                                                   
             forgetQueue(index);
        }            
        return true;
    }

    
    public void initRankedQueueView(Queue newQueue, VisiPlotOptions settings) {
        currentSettings_ = settings;
        newQueue(newQueue);
        setQueuePropertiesView(new QueuePropertiesView(newQueue));
        initQueueItemsRankedBView();
        buildMainMenu();
        buildMenu();
        createView();
        setTitle("Queue view: ORANG ");
        if (actionMenuDefault_ != null) {
            actionMenuDefault_.actionPerformedImpl(new ActionEvent(
                    actionMenuDefault_, 0, ""));
        }
    }

    public void queryTerminatedAction(DbbQueryTerminated ev) {

        stdlog_.debug("query terminated action");
        DbbView view = queueItemsView_.getView();
        DbbTable table = view.getTable();
        // stdlog_.debug("Init calculateSubtotal");

        int rows = view.getRows();

        Integer[] rankCounter = new Integer[rows];
        ArrayList rankStringList = new ArrayList();

        for (int i = 0; i < rows; i++) {
            RankInfo rankInfo = new RankInfo((String) view.getValueAt(i,
                    OTDbbView.ORANG_RANK),i);
            rankStringList.add(rankInfo);
        }
        // sort according to rank string
        Collections.sort(rankStringList);
        
        for(int i = 0; i < rankStringList.size(); i++) {
            RankInfo currentOrderedRank =  (RankInfo) rankStringList.get(i);
            currentOrderedRank.setRankCounter(new Integer(i));
        }
        // resort according to original
        Collections.sort(rankStringList,new TableIndexComparator());
        
        for(int i = 0; i < rankStringList.size(); i++) {
            RankInfo currentOrderedRank =  (RankInfo) rankStringList.get(i);
            rankCounter[i] = currentOrderedRank.getRankCounter();
        }
        table.addColumn("Rank Score", rankCounter, Integer.class, 0);

    }
    
    public class TableIndexComparator implements Comparator {

        public int compare(Object o1, Object o2) {
            int result = 0;
            if (o1 instanceof RankInfo && o2 instanceof RankInfo ) {
                RankInfo rank1 = (RankInfo) o1;
                RankInfo rank2 = (RankInfo) o2;
                
                result = rank1.getTableIndex().compareTo(rank2.getTableIndex());
            }
            return result;
        }
    }
    
    public class RankInfo implements Comparable {
        String rankString = null;
        Integer  tableIndex = null;
        Integer  rankCounter = null;
        public RankInfo(String rank, int tableIndex) { 
            setRankString(rank);
           setTableIndex(tableIndex);
          
        }
 
        private void setTableIndex(int tableIndex) {
          this.tableIndex = new Integer(tableIndex);
            
        }

        public int compareTo(Object o) {
            int result = 0;
            if (this.getRankString() != null && 
                    o != null && o instanceof RankInfo) {
                RankInfo rankClass = (RankInfo) o;
                result = this.getRankString().compareTo(rankClass.getRankString());   
            }
            return result;
        }

        public Integer getRankCounter() {
            return rankCounter;
        }   
         public Integer getTableIndex() {
            return rankCounter;
        }

        private void setRankCounter(Integer rankCounter) {
            this.rankCounter = rankCounter;
        }

        public String getRankString() {
            return rankString;
        }

        private void setRankString(String rankString) {
            this.rankString = rankString;
        }
      
    }

    public void appendQueueItems(QueueItem[] selectedOBIDs) {
        if (getQueueItemsView().appendQueueItems(selectedOBIDs)) {
            System.out.println("It worked");
            // queueItemsView_.startQuery();
        }
    }
}
