package org.eso.ohs.phase2.apps.ot.icdVcs;

import java.lang.reflect.InvocationTargetException;

import javax.swing.SwingUtilities;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.widgets.OutOfMemoryReport;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.evm.EVMException;
import org.eso.ohs.evm.EvmExecutorImpl;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.dbase.phase2.QueueManager;
import org.eso.ohs.phase2.icdVcs.ProcessTemplateStatusEvents;
import org.eso.ohs.phase2.icdVcs.ScriptCmdListener;
/**
 * Listener for script events from the BOB interface
 * 
 * <p>
 * NOTE: This class must be public in order for Java/Ccs to work.
 *       I'm not sure of the reason but it's probably to do with
 *       their use of the Reflection API.
 * 
 * <p>
 * $Id: ScriptListener.java,v 1.17 2005/04/14 09:41:30 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */
public class ScriptListener implements ScriptCmdListener {

    /** Standard log channel */
    private static Logger stdlog_ =
        Logger.getLogger(ScriptListener.class);

    /** BusinessObject requested by BOB */

    private CalibrationBlock[] cbList;
    /*a vector of Observation Block 
    					 requested by BOB*/

    /**
     * Used to set the status of the BusinessObject that was sent to BOB.
     * If the BO was cloned before sending, the original BO's status is
     * also updated.
     * 
     * @param BOBEventType	Type of event as sent from BOB, eg "ABORTED"
     */
    public void setBusObjStatusFromEventType(long obId, String BOBEventType) {

        ProcessOBEvents obEvents = new ProcessOBEvents(obId, BOBEventType);
        try {
            SwingUtilities.invokeAndWait(obEvents);
        } catch (InterruptedException e) {
        } catch (InvocationTargetException e) {
        }
    }
    /**
           * Used to set the status of the BusinessObject that was sent to BOB.
           * If the BO was cloned before sending, the original BO's status is
           * also updated.
           * 
           * @param BOBEventType   Type of event as sent from BOB, eg "ABORTED"
           * @param idstr The id of the ob that the status event was for.
           */
    public void setTemplateStatusFromEventType(
        String BOBEventType,
        String idStr,
        String tsNo) {
        int id = (int) new Long(idStr).longValue();
        int tsNum = Integer.valueOf(tsNo).intValue();
        System.out.println("We are here again");
        ProcessTemplateStatusEvents templateEvents =
            new ProcessTemplateStatusEvents(
                BOBEventType,
                id,              
                tsNum-1);
        try {
            SwingUtilities.invokeAndWait(templateEvents);
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
    }

    /**
     * Handler method for NextObsBlocks requests from BOB.
     * 
     * @param msg	    status message as a tcl array
     * @return	    Observation block in OBD format 
     *		    or null string if an error occurred.
     */
    public String NextObsBlocks(int maxNumOBs, Boolean isOffline) {

        //stdlog_.debug("Is off line " + isOffline);
        if (Boolean.TRUE.equals(isOffline)) {
            return ""; // Not for us.
        }
        //stdlog_.debug("OT.NextObsBlocks: " + maxNumOBs);
        try {
            OTOBChecker chk = new OTOBChecker(maxNumOBs);
            SwingUtilities.invokeAndWait(chk);
            cbList = chk.getOBs();
            for (int i = 0; i < cbList.length; i++) {
                try {
                    if (cbList[i].isFlames()) {
                        if (EvmExecutorImpl.evmConfigured()) {
                            EvmExecutorImpl evm = EvmExecutorImpl.getInstance();
                            //Check that the evm is initialized
                            if (evm.isInitialized()) {
                                try {
                                    evm.calculateETCS(cbList[i]);
                                } catch (EVMException e) {
                                    // If no exec time calcilation the OB can 
                                    // still be checkedin...
                                }
                            }
                        }
                    }
                    QueueManager.updateOBStatus(
                        cbList[i].getId(),
                        CalibrationBlock.INITIATED);
                } catch (ObjectIOException e) {
                    stdlog_.error(e);
                }
            }

            //stdlog_.debug("CB List " + cbList.length);
            if ((cbList != null) && (cbList.length > 0)) {
                ObjectManager objMgr = ObjectManager.getObjectManager();
                String paf = cbList[0].toOBD(true,objMgr.getDbaseId(cbList[0]),OHSApplication.getShortDesc());
                //cb_=cbList[0];
                for (int i = 1; i < cbList.length; i++) {
                    paf = paf + cbList[i].toOBD(false,objMgr.getDbaseId(cbList[i]),OHSApplication.getShortDesc());
                }
                //stdlog_.debug("PAF result = " + paf);
                return paf;
            }
        } catch (InterruptedException e) {
            stdlog_.error(e);
        } catch (InvocationTargetException e) {
            stdlog_.error(e);
        } catch (Exception e) {
            stdlog_.error(e);
        } catch (OutOfMemoryError e) {
            stdlog_.error(e);
            OutOfMemoryReport.showMessage();
            throw e;
        }

        return "";
    }

}
