/*
 * Created by amchavan on Jul 14, 2005
 */
package org.eso.ohs.phase2.apps.ot.wizard;

import java.awt.Component;
import java.awt.Cursor;
import java.io.IOException;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.widgets.OHSWizard;
import org.eso.ohs.core.gui.widgets.OHSWizardPage;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.NotASCIIException;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.Seeing;
import org.eso.ohs.dfs.SkyTransparency;
import org.eso.ohs.dfs.StrehlRatio;
import org.eso.ohs.dfs.WeatherConstraint;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.phase1.Phase1SelectStmt;
import org.eso.ohs.persistence.dbase.phase2.Phase2SelectStmt;
import org.eso.ohs.phase2.apps.ot.MyPersistCfg;
import org.eso.ohs.phase2.orang.OrangScriptException;
import org.eso.ohs.phase2.visibility.VisibilityCalcUtils;
import org.eso.ohs.phase2.visibility.VisibilityJCalc;
import org.eso.ohs.phase2.visibility.VisicalcOptions;
import org.eso.ohs.phase2.visibility.PCFPlotterFilter.ValueRejectedException;

// $Id: OrangWizard.java,v 1.10 2005/12/12 14:23:17 pnunes Exp $
public class OrangWizard extends OHSWizard implements OrangModel {
  
    private static Logger stdlog_ = Logger.getLogger(OrangWizard.class);

    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    private static final String COMBO_BOX_INSTRUMENT = "comboBoxInstrument";

    private ObSelectorView selectorPage;

    private ObservingConditionFilterView weatherPage;

    private VisibilityConstraintFilterView constrainPage;

    private AlgorithmChooserPage algorithmPage;

    private OrangEngine orang;

    private FinishPage finishedPage;

    private TimelinePage timelinePage;

    private OrangWidgetsData defaultData;

    public static final String COMBO_BOX_ALGORITHM = "comboBoxAlgorithm";

    public static final String SPINNER_TOLERANCE_AIRMASS = "spinnerToleranceAirmass";

    public static final String SPINNER_TOLERANCE_MOON_ANGLE = "spinnerToleranceMoonAngle";

    public static final String SPINNER_TOLERANCE_MOON_DISTANCE = "spinnerToleranceMoonDistance";

    public static final String SPINNER_TOLERANCE_FLI = "spinnerToleranceFLI";

    public static final String SPINNER_TOLERANCE_SUN_ANGLE = "spinnerToleranceSunAngle";

    public static final String INACTIVE = "Inactive";

    public static final String ACTIVE = "Active";

    public static final String BUTTONGROUP_FLI = "buttongroupFli";

    public static final String BUTTONGROUP_MOON_D = "buttongroupMoonD";

    public static final String BUTTONGROUP_SUN = "buttongroupSun";

    public static final String BUTTONGROUP_MOON_A = "buttongroupMoonA";

    public static final String BUTTONGROUP_SID_TIME = "buttongroupSidTime";

    public static final String BUTTONGROUP_TIME_INT = "buttongroupTimeInt";

    public static final String BUTTONGROUP_SKY = "buttongroupSky";

    public static final String BUTTONGROUP_SEEING = "buttongroupSeeing";

    public static final String BUTTONGROUP_STREHL = "buttongroupStrehl";

    public static final String FIELD_SEEING = "textFieldSeeing";

    public static final String FIELD_SKY_TRANSPARENCY = "comboFieldSkyTransparency";

    public static final String FIELD_STREHL_RATIO = "textFieldStrehlRatio";

    public static final String TELESCOPE_QUEUES_COMMAND = "Telescope today queues";

    public static final String INSTRUMENT_QUEUES_COMMAND = "Instrument today queues";

    public static final String QUEUES_COMMAND = "Queues";

    public static final String COMBO_BOX_TELESCOPE = "comboBoxTelescope";

    public static final String COMBO_BOX_QUEUES = "comboBoxQueues";

    public static final String OB_SELECTION_GROUP = "obSelectionGroup";

    public static final String DATE_START = "startTimeTextField";

    public static final String DURATION = "durationComboBox";

    public static final String STEP_INTERVAL = "stepComboBox";

    public static final String LIST_BOX_QUEUES = "listBoxQueues";

    public static final String TIMELINE_ALGORITHM = "cbTimelineAlgorithm";

    static final String BUTTONGROUP_ZENITH = "zenithSelectionGroup";

    public OrangWizard(OrangEngine orang) {

        // view_ = view;
        this.orang = orang;
        
        defaultData = new OrangWidgetsData();
        selectorPage = new ObSelectorView(this, defaultData);
        weatherPage = new ObservingConditionFilterView(this, defaultData);
        constrainPage = new VisibilityConstraintFilterView(this, defaultData);
        algorithmPage = new AlgorithmChooserPage(this, defaultData);
        timelinePage = new TimelinePage(this, defaultData);
        finishedPage = new FinishPage(this, defaultData);

        addPage(selectorPage);
        addPage(weatherPage);
        addPage(constrainPage);
        addPage(algorithmPage);
        addPage(timelinePage);
        addPage(finishedPage);
    }

    protected void leavingPage(OHSWizardPage from, boolean usingNext) {
        if (usingNext) {
            try {

            	// sets the cursor
            	((Component)getTopLevelAncestor()).setCursor(new Cursor(Cursor.WAIT_CURSOR));
            	
                if (from instanceof ObSelectorView) {
                    orang.loadObList(this);
                    weatherPage.setInfo(orang.getLoadedInfo());
                } else if (from instanceof ObservingConditionFilterView) {
                    orang.filterObByWeather(this);
                    constrainPage.setInfo(orang.getWeatherInfo());
                } else if (from instanceof VisibilityConstraintFilterView) {
                    orang.filterObByConstraints(this);
                    algorithmPage.setInfo(orang.getVisibilityInfo());
                    algorithmPage.reloadScriptList();
                 } else if (from instanceof AlgorithmChooserPage) {
                    orang.applyAlgorithm(this);
                    orang.showRankedObs(this);
                    timelinePage.setInfo(orang.getRankedInfo());
                    algorithmPage.reloadScriptList();
                } else if (from instanceof TimelinePage) {
                    orang.showTimeline(this);
                    finishedPage.setInfo(orang.getRankedInfo());
                    algorithmPage.reloadScriptList();
                }
            } catch (ObjectNotFoundException e) {
                MsgManager.errDialog("Object I/O Error", e);
                e.printStackTrace();
            } catch (ObjectIOException e) {
                MsgManager.errDialog("Object I/O Error", e);
                e.printStackTrace();
            } catch (ValueRejectedException e) {
            	MsgManager.errDialog("Unexpected error", e);
                e.printStackTrace();
            } catch (OrangScriptException e) {
                JOptionPane.showMessageDialog(algorithmPage
                        .getTopLevelAncestor(), e.getMessage(),
                        "ORANG Script Error", JOptionPane.ERROR_MESSAGE);

                e.printStackTrace();
                return;
            } finally {
            
            	// 	sets the cursor
            	((Component)getTopLevelAncestor()).setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            }
        }
    }

    protected void leavingWizard(boolean successful) {
        orang.clear();
        this.setVisible(false);
    }

    /**
     * Put up a toplevel frame with a button which will start the wizard.
     */
    public static void main(String[] args) throws InterruptedException {
        try {
            AppConfig appcfg = new AppConfig();
            AppConfig.setAppConfig(appcfg);
            appcfg.addParamfile("config/site.cf");
            appcfg.addParamfile("config/ot-db.cf");
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (NotASCIIException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        Config.setCfg(new MyPersistCfg());
        OrangEngine orang = new OrangEngine();
        OrangWizard wiz = new OrangWizard(orang);
        wiz.showWizard(null, "ORANG Wizard", true);

    }

    public String getObSelectionQueue() {
        return (String) get(COMBO_BOX_QUEUES);
    }

    public String getObSelectionTelescope() {
        return (String) get(COMBO_BOX_TELESCOPE);
    }

    public int getObSelectionType() {
        int result = OrangModel.OB_SELECTION_WRONG;
        String value = (String) get(OB_SELECTION_GROUP);
        stdlog_.debug("Selected value: "+value);
        if (TELESCOPE_QUEUES_COMMAND.equals(value)) {
            result = OrangModel.OB_SELECTION_TELESCOPE;
        } else if (INSTRUMENT_QUEUES_COMMAND.equals(value)) {
            result = OrangModel.OB_SELECTION_INSTRUMENT;
        } else if (QUEUES_COMMAND.equals(value)) {
            result = OrangModel.OB_SELECTION_QUEUES;
        }
        return result;
    }

    public Seeing getSeeingValue() {
    	
    	Seeing seeing = null;
    	if (isSeeingConstrain()) {
    		String seeingString =  (String) get(FIELD_SEEING);
    		
    		if (seeingString != null) {
    			seeing = new Seeing(seeingString);
    		}
    	} else {
    		// doesn't matter what's the seeing in this case
    		seeing = new Seeing(2);
    	}
        return seeing;
    }

    public SkyTransparency getSkyTransp() {
       
        SkyTransparency sky = null;
        
        if (isSkyConstrain()) {
        	String skyString = (String) get(FIELD_SKY_TRANSPARENCY);
        	if (skyString != null) {
        		sky = new SkyTransparency(skyString);
        	}
        } else {
        	//	 doesn't matter what's the transparency in this case
        	sky = new SkyTransparency(SkyTransparency.PHOTOMETRIC);
        }
        	
    
         return sky;
    }

    public StrehlRatio getStrehlRatio() {
        String ratio = (String) get(FIELD_STREHL_RATIO);
        StrehlRatio strehlRatio = new StrehlRatio(0);
        if (ratio != null && ratio.trim().length() > 0){
            strehlRatio = new StrehlRatio(ratio);
        }
        return strehlRatio;
    }

    public int getAirmassValuePercentage() {
        return Integer.parseInt((String) get(SPINNER_TOLERANCE_AIRMASS));
    }

    public boolean isFliConstrain() {
        boolean result = isButtonGroupSelActive(BUTTONGROUP_FLI);
        return result;
    }

    /**
     * @return boolean
     */
    private boolean isButtonGroupSelActive(String buttonGroupName) {
        boolean result = false;
        String value = (String) get(buttonGroupName);
        if (ACTIVE.equals(value)) {
            result = true;
        } else if (INACTIVE.equals(value)) {
            result = false;
        }
        return result;
    }

    public int getFliRelaxPercentage() {
        return Integer.parseInt((String) get(SPINNER_TOLERANCE_FLI));
    }

    public boolean isMoonAngleConstrain() {
        boolean result = isButtonGroupSelActive(BUTTONGROUP_MOON_A);
        return result;
    }

    public boolean isSeeingConstrain() {
        boolean result = isButtonGroupSelActive(BUTTONGROUP_SEEING);
        return result;
    }

    public boolean isSkyConstrain() {
        boolean result = isButtonGroupSelActive(BUTTONGROUP_SKY);
        return result;
    }

    public boolean isStrehlConstrain() { 
        return false;
    }

    public int getMoonAngleDegree() {
        return Integer.parseInt((String) get(SPINNER_TOLERANCE_MOON_ANGLE));
    }

    public boolean isMoonDistanceConstrain() {
        boolean result = isButtonGroupSelActive(BUTTONGROUP_MOON_D);
        return result;
    }

    public int getMoonRelaxPercentage() {
        return Integer.parseInt((String) get(SPINNER_TOLERANCE_MOON_DISTANCE));
    }

    public boolean isSunAngleConstrain() {
        boolean result = isButtonGroupSelActive(BUTTONGROUP_SUN);
        return result;
    }

    public int getSunAngleDegree() {
        return Integer.parseInt((String) get(SPINNER_TOLERANCE_SUN_ANGLE));
    }

    public String getRankingAlgorithm() {
        return (String) get(COMBO_BOX_ALGORITHM);
    }

    public VisicalcOptions getVisiCalcOptions() {
        VisicalcOptions options = new VisicalcOptions();

        options.setCalcZenithInvisibility(isTelescopeZenithConstraint());
        options.setZenithConstraint(getZenithConstraint());
        options.setZenithConstraintCalcPrecision(getZenithConstraintCalcPrecision());
        options.setCalcFli(isFliConstrain());
        options.setCalcMoonDist(isMoonDistanceConstrain());
        options.setCalcSun(isSunAngleConstrain());
        options.setCalcMoon(isMoonAngleConstrain());
        options.setCalcTimeInterval(isTimeIntervalConstrain());
        options.setCalcSiderealTimeInterval(isSidTimeIntervalConstrain());

        options.setFliTolerance(getFliRelaxPercentage());
        options.setMoonConstraint(getMoonRelaxPercentage());
        options.setAirmassTolerance(getAirmassValuePercentage());
        options.setSunConstraint(getSunAngleDegree());
        options.setMoonConstraint(getMoonAngleDegree());

        return options;
    }

    public boolean isTimeIntervalConstrain() {
        boolean result = isButtonGroupSelActive(BUTTONGROUP_TIME_INT);
        return result;
    }

    public boolean isSidTimeIntervalConstrain() {
        boolean result = isButtonGroupSelActive(BUTTONGROUP_SID_TIME);
        return result;
    }

    public String getUser() {
        Config cfg = Config.getCfg();
        return String.valueOf(cfg.getUserName());
    }

    public boolean isSuperUser() {
        Config cfg = Config.getCfg();
        return cfg.isSuperUser();
    }

    public Date getStartDate() {
        Date isoDate = null;
        try {
            isoDate = Convert.ISODateToDate((String) get(DATE_START));
        } catch (ParseException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return isoDate;
    }

    public Date getEndDate() {
        Date isoDate = null;
        try {
            String isoStringDate = getEndISODate();
            isoDate = Convert.ISODateToDate(isoStringDate);
        } catch (ParseException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return isoDate;
    }

    public int getStepInterval() {
        int interval = Integer.parseInt((String) get(STEP_INTERVAL));
        return interval;
    }

    public double getLatitude() {
        return AppConfig.getAppConfig().getLatitude();

    }

    public double getLongitude() {
        return AppConfig.getAppConfig().getLongitude();
    }
    
    public double getZenithConstraint() {
    	return new Double(AppConfig.getAppConfig().getValue(
				VisicalcOptions.telescopeConstraintKeyword)).doubleValue();
    }
    
    public int getZenithConstraintCalcPrecision() {
    	return new Integer(AppConfig.getAppConfig().getValue(
				VisicalcOptions.telescopeConstraintPrecisionKeyword)).intValue();
    }

    public String getEndISODate() {
        String duration = (String) get(DURATION);
        String isoStringDate = null;
        Date endDate;
        Calendar cal = Calendar.getInstance();
        if (OrangWidgetsData.ALL_NIGHT.equals(duration)) {
            cal.setTime(getStartDate());
            VisibilityJCalc calc = new VisibilityJCalc();
            endDate = calc.getSunRiseTime(OrangWidgetsData.SUN_DAWN_DEFAULT,
                    OrangWidgetsData.STEP_INTERVAL_DEFAULT, cal.getTime(),
                    VisibilityCalcUtils.PLONG, VisibilityCalcUtils.PLAT);
        } else {
            Date startDate = getStartDate();
            cal.setTime(startDate);
            cal.add(Calendar.HOUR, Integer.parseInt(duration));
            endDate = cal.getTime();
        }
        isoStringDate = Convert.longToISODateMilliSec(endDate.getTime());
        return isoStringDate;
    }

    public String getStartISODate() {
        return (String) get(DATE_START);
    }

    public String[] getSelectedQueues() {
        return (String[]) get(LIST_BOX_QUEUES);
    }

    // amchavan & pnunes, 17-Oct-2005: added
    public String getTimelineAlgorithm() {
        return (String) get(TIMELINE_ALGORITHM);
    }

    public String getSelectedInst() {
        return (String) get(COMBO_BOX_INSTRUMENT);

    }

    public String getSelectedTel() {
        return (String) get(COMBO_BOX_TELESCOPE);

    }

    /**
     * @return Returns the queues assigned to the telescope today.
     */
    public String[] getQueuesTelescopes() {
        String tel = getSelectedTel();
        String[] queueTel = new String[0];
        if (tel != null) {
            queueTel = defaultData.getQueueTel(tel);
        }
        return queueTel;
    }

    /**
     * @return Returns the queues assigned to the instrument today.
     */
    public String[] getQueuesInstruments() {
        String instrument = getSelectedInst();
        String[] queueInst = new String[0];
        if (instrument != null) {
            queueInst = defaultData.getQueueInst(instrument);
        }
        return queueInst;
    }

    public WeatherConstraint getWeatherConstraint() {
        WeatherConstraint currentWeather = new WeatherConstraint(
                getSeeingValue(), getSkyTransp(), getStrehlRatio(),
                isSeeingConstrain(), isSkyConstrain(), isStrehlConstrain());
        return currentWeather;
    }


    public boolean isTelescopeZenithConstraint() {
        boolean result = isButtonGroupSelActive(BUTTONGROUP_ZENITH);
        return result;
    }

}
