package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;

import javax.swing.JFrame;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.persistence.ObjectSelector;
import org.eso.ohs.phase2.apps.p2pp.BOSelection;
import org.eso.ohs.phase2.apps.p2pp.LocalClipboard;
import org.eso.ohs.phase2.apps.p2pp.Persistence;
import org.eso.ohs.phase2.apps.p2pp.PropertyClipboard;

/**
 * Action class to Copy a BusinessObject to the clipboard
 * <p>
 * $Id: CopyBOAction.java,v 1.11 2005/02/11 10:31:49 tcanavan Exp $
 * <br>
 * Author: Tim Canavan
 */

public class CopyBOAction extends ActionSuperclass {

    private Class objType_;

    private ObjectSelector objSelector_;


/**
 * This is a requirement 
 * 
 * @param objSelector Interface that tells which objevct is Displayed. 
 * @param frm The frame where this action is executed from
 * @param objType The object type for the 
 * @param action The name of the action
 */


    public CopyBOAction(ObjectSelector objSelector, JFrame frm, Class objType,
                        String action) {
        super(frm, action);
        objType_     = objType;
        objSelector_ = objSelector;
    }


    /**
     * Event handler for the create directory action, eg from a menu item.
     *
     * @param ev        user-initiated ActionEvent to create a directory.
     */
    public void actionPerformedImpl(ActionEvent ev) {

        PropertyClipboard c = LocalClipboard.CLIPBOARD;
        Persistence objMgr = Persistence.getInstance();
        BusinessObject objCopy = null;
        BusinessObject bo = objSelector_.getDisplayedObject();

        if (bo instanceof ObservationBlock) {
            ObservationBlock ob = (ObservationBlock)bo;
            if (objType_.equals(Target.class)) {
                objCopy = ob.getTarget().copy();
            }
            if (objType_.equals(ConstraintSet.class)) {
                objCopy = ob.getConstraintSet().copy();
            }
            if (objType_.equals(ObservationDescription.class)) {
                objCopy = ob.getOd().copy();
            }
            
            if (objType_.equals(String.class)) {
                BOSelection boSelect =
                    new BOSelection(ob.getUserComments(), LocalClipboard.BO_FLAVOR);
                c.setContents(boSelect, boSelect);
            }
        } else if (bo instanceof CalibrationBlock) {
            CalibrationBlock cb = (CalibrationBlock)bo;
            if (objType_.equals(ObservationDescription.class)) {
                objCopy = cb.getOd().copy();
            }
        } else if (bo instanceof Target) {
            Target tg = (Target)bo;
            if (objType_.equals(Target.class)) {
                objCopy = tg.copy();
            }
        }

        if (objCopy != null) {
            BOSelection boSelect =
                new BOSelection(objCopy, LocalClipboard.BO_FLAVOR);
            c.setContents(boSelect, boSelect);
        }
        c.firePropertyChangeEvent();
    }
}

