package org.eso.ohs.phase2.obHistory;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbCheckBox;
import org.eso.ohs.core.dbb.client.DbbISODateTime;
import org.eso.ohs.core.dbb.client.DbbIntegerTextField;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbSelectionList;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DbbTextFieldPattern;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.client.DbbWidget;
import org.eso.ohs.core.dbb.client.DisplayTextAction;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbInvalidSqlOperatorType;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.utilities.Utilities;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.persistence.Config;

/**
 * Application database view window
 * <p>
 * @version $Revision: 1.30 $
 * @author Ron J Lobo
 */
public class DbWin extends JFrame {

    public static final String cvsID_ =
        "$Id: DbWin.java,v 1.30 2005/04/11 13:53:44 tcanavan Exp $";

    private static Logger stdlog_ = Logger.getLogger(DbWin.class);

    private DbbView view_;
    private DbbSqlEngine engine_;

    private final int nullOB = -1;
    private int obId = nullOB;

    // DB tables
    private DbbSqlTable oerTbl;
    private DbbSqlTable oeTbl;
    private DbbSqlTable obTbl;

    private DbbIntegerTextField obITF;
    private DbbTextField lTF;

    // column titles
    private static final String OBID_ = "OB ID";
    private static final String OBNAME_ = "OB Name";
    private static final String SITE_ = "Site";
    private static final String TEMPLATE_NAME_ = "Template Name";
    private static final String IGNORE_TEMPLATES_ = "Ignore Templates";
    private static final String COMPONENT_ = "Component";
    private static final String HOSTNAME_ = "Hostname";
    private static final String LOGIN_ = "Login";
    private static final String MESSAGE_ = "Status";
    private static final String TIME_STAMP_ = "Time Stamp";

    private static final String appName_ = "OB History";

    private static final Dimension screen_ =
        java.awt.Toolkit.getDefaultToolkit().getScreenSize();

    private static DbWin dbWin = null;

    public static DbWin getInstance(int obId) {
        if (dbWin == null)
            dbWin = new DbWin(obId);
        else {
            dbWin.initOBID(obId);
            dbWin.setOBID(obId);
            dbWin.resetDbbView();
        }
        return dbWin;
    }

    /**
     * Constructor
     */
    public DbWin(int obId) {
        init(obId);
    }

    private void initOBID(int obId) {
        this.obId = obId >= 0 ? obId : nullOB;
    }

    private void init(int obId) {
        try {
            this.obId = obId >= 0 ? obId : nullOB;
            engine_ = BrowserConfig.getEngine();
            view_ = new DbbView(engine_);

            // setup database tables
            oerTbl = BrowserConfig.OER; // ob_event_register Table
            oeTbl = BrowserConfig.OE; // ob_events Table
            obTbl = BrowserConfig.OB; // obs_blocks Table

            // build the display
            createFileMenu();
            buildColumns();
            makeFilterPanel();
            makeSortPanel();

            final String TextView_ = "Text View";
            JButton print_button = new JButton(TextView_);
            print_button.addActionListener(
                new DisplayTextAction(this, view_, appName_, TextView_));
            view_.addSummaryButton(print_button);

            // remove unwanted default buttons
            // view_.hideCountButton();
            // view_.hideQueryButton();

        } catch (ObjectIOException e) {
            // throw new ObHistoryException(e);
        } catch (RemoteException e) {
            // throw new ObHistoryException(e);
        } catch (MalformedURLException e) {
            // throw new ObHistoryException(e);
        } catch (NotBoundException e) {
            // throw new ObHistoryException(e);
        } catch (SQLException e) {
            // throw new ObHistoryException(e);
        }

        // ask before exiting the application
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent ev) {
                closeWindow();
            }
        });
        /*
                // map double click events to something
                view_.getTable().getTable().addMouseListener(new MouseAdapter() {
                    public void mouseClicked(MouseEvent e) {
                        if (e.getClickCount() == 2) {
                            someClass.actionPerformedImpl(null);
                        }
                    }
                });
        */
        // prepare window for display
        getContentPane().add(view_);
        setTitle("OB History");
        pack();
        setSizeAndLocation();
    }

    private void setSizeAndLocation() {
        final int maxWid = 1200;
        final int maxScr = screen_.width * 9 / 10;
        int wid = (maxWid > maxScr) ? maxScr : maxWid;
        int ht = screen_.height * 3 / 5;
        setSize(wid, ht);
        // place window near the top left
        int posX = (screen_.width - wid) / 4;
        int posY = (screen_.height - ht) / 4;
        setLocation(posX, posY);
    }

    /**
     * Build DB browser view: Select panel
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    private void buildColumns()
        throws
            NotBoundException,
            MalformedURLException,
            SQLException,
            RemoteException {

        DbbSqlChunk c;
        c = new DbbSqlChunk(oeTbl, "ob_id", OBID_);
        engine_.addDefaultColumnsChunk(c);

        c = new DbbSqlChunk(obTbl, "item_name", OBNAME_);
        engine_.addDefaultColumnsChunk(c);

        c = new DbbSqlChunk(oeTbl, "sitecode", SITE_);
        engine_.addDefaultColumnsChunk(c);

        c = new DbbSqlChunk(oerTbl, "template_name", TEMPLATE_NAME_);
        engine_.addDefaultColumnsChunk(c);

        c = new DbbSqlChunk(oerTbl, "component", COMPONENT_);
        engine_.addDefaultColumnsChunk(c);

        c = new DbbSqlChunk(oerTbl, "hostname", HOSTNAME_);
        engine_.addDefaultColumnsChunk(c);

        c = new DbbSqlChunk(oerTbl, "login", LOGIN_);
        engine_.addDefaultColumnsChunk(c);

        c = new DbbSqlChunk(oerTbl, "msg", MESSAGE_);
        engine_.addDefaultColumnsChunk(c);

        c =
            new DbbSqlChunk(
                oeTbl,
                "event_time",
                TIME_STAMP_,
                DbbISODateTime.class);
        engine_.addDefaultColumnsChunk(c);

        // there are no user selectable columns

        DbbPanel panel = new DbbPanel("cols");
        view_.setColumnsPanel(panel, "North", false);
    }

    private void makeFilterPanel() {

        try {
            DbbSqlOperator opIn = new DbbSqlOperator("in");
            DbbSqlOperator opLike = new DbbSqlOperator("like");
            DbbSqlOperator opIsNull = new DbbSqlOperator("is");
            DbbSqlOperator opEquals = new DbbSqlOperator("=");

            String column;
            String label;
            DbbSqlChunk sqlChunk;

            column = "ob_id";
            label = OBID_;
            sqlChunk =
                new DbbSqlChunk(oeTbl, column, label, opEquals, Integer.class);
            obITF = new DbbIntegerTextField(label, sqlChunk);

            column = "item_name";
            label = OBNAME_;
            sqlChunk =
                new DbbSqlChunk(obTbl, column, label, opLike, String.class);
            DbbTextFieldPattern obnameTF = new DbbTextFieldPattern(label, sqlChunk);

            column = "sitecode";
            label = SITE_;
            sqlChunk =
                new DbbSqlChunk(oeTbl, column, label, opIn, String.class);
            final String[] sitesLong = { "Garching", "Paranal", "LaSilla" };
            final String[] sitesArray = { "G", "P", "S" };
            DbbSelectionList siteList =
                new DbbSelectionList(label, sqlChunk, sitesArray, sitesLong);

            column = "template_name";
            label = TEMPLATE_NAME_;
            sqlChunk =
                new DbbSqlChunk(oerTbl, column, label, opLike, String.class);
            DbbTextFieldPattern tnTF = new DbbTextFieldPattern(label, sqlChunk);

            column = "template_name";
            label = IGNORE_TEMPLATES_;
            sqlChunk =
                new DbbSqlChunk(oerTbl, column, label, opIsNull, Boolean.class);
            sqlChunk.setUseNull(true);
            DbbCheckBox tnCB = new DbbCheckBox(label, sqlChunk);

            column = "component";
            label = COMPONENT_;
            sqlChunk =
                new DbbSqlChunk(oerTbl, column, label, opLike, String.class);
            DbbTextFieldPattern cTF = new DbbTextFieldPattern(label, sqlChunk);

            column = "hostname";
            label = HOSTNAME_;
            sqlChunk =
                new DbbSqlChunk(oerTbl, column, label, opLike, String.class);
            DbbTextFieldPattern hnTF = new DbbTextFieldPattern(label, sqlChunk);

            column = "login";
            label = LOGIN_;
            sqlChunk =
                new DbbSqlChunk(oerTbl, column, label, opEquals, String.class);
            lTF = new DbbTextField(label, sqlChunk);

            column = "msg";
            label = MESSAGE_;
            sqlChunk =
                new DbbSqlChunk(oerTbl, column, label, opIn, String.class);
            DbbSelectionList statusList =
                new DbbSelectionList(
                    label,
                    sqlChunk,
                    CalibrationBlock.getStatusValues().valuesAsStrings(),
                    CalibrationBlock.markLabels);

            DbbPanel panel = new DbbPanel("Selection Criteria", 5);
            panel.addWidget(obITF);
            panel.addWidget(obnameTF);
            panel.addWidget(siteList);
            panel.addWidget(tnTF);
            panel.addWidget(tnCB);
            panel.addWidget(cTF);
            panel.addWidget(hnTF);
            panel.addWidget(lTF);
            panel.addWidget(statusList);

            view_.setWherePanel(panel, "North", true, 1, 1);

            // preselect the obId
            setOBID(obId);

        } catch (DbbInvalidSqlOperatorType e) {
            stdlog_.error("dbWin.makeFilterPanel(): " + e);
        }
    }

    private void setOBID(int obId) {
        this.obId = obId >= 0 ? obId : nullOB;
        if (obId != nullOB) {
            stdlog_.debug("Setting OB ID");
            obITF.set(new Integer(obId).toString());
        }

        // ensure users only see their own events (except user 0)
        int userId = Config.getCfg().getUserName();
        if (userId != 0) {
            lTF.set(userId + "");
            lTF.setEnabled(false);
        }
    }

    private void makeSortPanel() {

        String column;
        String label;

        column = "ob_id";
        label = OBID_;
        DbbSqlChunk c = new DbbSqlChunk(oeTbl, column, label);
        DbbWidget obidWgt = new DbbArrowButton(label, c);

        column = "item_name";
        label = OBNAME_;
        c = new DbbSqlChunk(obTbl, column, label);
        DbbWidget obnameWgt = new DbbArrowButton(label, c);

        column = "sitecode";
        label = SITE_;
        c = new DbbSqlChunk(oeTbl, column, label);
        DbbWidget siteWgt = new DbbArrowButton(label, c);

        column = "template_name";
        label = TEMPLATE_NAME_;
        c = new DbbSqlChunk(oerTbl, column, label);
        DbbWidget templateWgt = new DbbArrowButton(label, c);

        column = "component";
        label = COMPONENT_;
        c = new DbbSqlChunk(oerTbl, column, label);
        DbbWidget compWgt = new DbbArrowButton(label, c);

        column = "hostname";
        label = HOSTNAME_;
        c = new DbbSqlChunk(oerTbl, column, label);
        DbbWidget hostWgt = new DbbArrowButton(label, c);

        column = "login";
        label = LOGIN_;
        c = new DbbSqlChunk(oerTbl, column, label);
        DbbWidget loginWgt = new DbbArrowButton(label, c);

        column = "msg";
        label = MESSAGE_;
        c = new DbbSqlChunk(oerTbl, column, label);
        DbbWidget msgWgt = new DbbArrowButton(label, c);

        column = "event_time";
        label = TIME_STAMP_;
        c = new DbbSqlChunk(oeTbl, column, label, DbbISODateTime.class);
        DbbWidget timeWgt = new DbbArrowButton(label, c);

        DbbPanel sortPanel = new DbbPanel("Order By", 1);
        sortPanel.addWidget(obidWgt);
        sortPanel.addWidget(obnameWgt);
        sortPanel.addWidget(siteWgt);
        sortPanel.addWidget(templateWgt);
        sortPanel.addWidget(compWgt);
        sortPanel.addWidget(hostWgt);
        sortPanel.addWidget(loginWgt);
        sortPanel.addWidget(msgWgt);
        sortPanel.addWidget(timeWgt);

        view_.setSortPanel(sortPanel, "West", true, 1, 1);
    }

    /**
     * Create and populate the "File" menu in the database browser view
     * @exception ObjectIOException object IO Exception
     */
    private void createFileMenu() throws ObjectIOException {
        JMenuBar menubar = new JMenuBar();
        setJMenuBar(menubar);
        JMenu menu = new JMenu("File");
        menubar.add(menu);
        menu.addSeparator();
        JMenuItem mnItem = menu.add(new CloseAction(this, "Close"));
        mnItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_MASK, false));
    }

    /**
     * ensure that the query is executed after the frame appears
     */
    public void addNotify() {
        super.addNotify();
        if (obId != nullOB) {
            // automatically launch the query
            resetDbbView();
        }
    }

    /**
     * Rereads the table, and sets focus in the JTable
     */
    public void resetDbbView() {
        Utilities.waitCursor(this);
        view_.sqlStartAction(null);
        view_.getTable().getTable().requestFocusInWindow();
    }

    private void closeWindow() {
        /* mainApp.askAndQuit(); */
        setVisible(false);
    }

    /**
     * Class for closing a window or exiting the application
     */
    class CloseAction extends ActionSuperclass {

        /** Constructs a Close/Exit action */
        public CloseAction(JFrame frm, String closeStr) {
            super(frm, closeStr);
        }

        /**
         * Event handler for the Quit action
         *
         * @param ev user-initiated ActionEvent to quit
         */
        public void actionPerformedImpl(ActionEvent ae) {
            closeWindow();
        }
    }
}
