package org.eso.ohs.phase2.visibility;

/**
 * Julian Date class converts from calendar date and time to Julian date.
 *
 * @version 25 July 2001
 * @author C. Mavros for ESO
 */

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;

import org.apache.log4j.Logger;

public class JulianDate {

   /** 
    * Standard log channel 
    * <br>ChangeLog: cmavros 21-12-01 added
    */
   private static Logger stdlog_ =
   Logger.getLogger( JulianDate.class );

   /**
    * getJDC converts the given date and time + interval a Julian Date
    *
    * @param convDate Date start date and time 
    * @param interval integer the time of each interval
    * @param ctr integer counter of the number of intervals.
    * @return Julian Date double 
    */

    public static double getJDC(Date convDate, int interval, int ctr){

        TimeZone tz = TimeZone.getTimeZone("UT");
        TimeZone.setDefault(tz);
        Calendar calendar = new GregorianCalendar();

        //Set selected date and time
        calendar.setTime(convDate); 

        //If ctr is greater than 0 add interval time to startTime_
        if(ctr>0){
            calendar.add(Calendar.MINUTE, (interval*ctr));
            //stdlog_.info("visiplot.JulianDate.getJDC:  " + calendar.getTime() + " UT");
        }
        //Add one month for calculation because Calendar objects
        //starts months at 0 thru 11
        //calendar.add(Calendar.MONTH, 1 );

        //Get selected all date information  
        int setMinutes = calendar.get(Calendar.MINUTE); 
        int setHours = calendar.get(Calendar.HOUR_OF_DAY); 
        int setDay = calendar.get(Calendar.DAY_OF_MONTH); 
        int setMonth = calendar.get(Calendar.MONTH)+1; 
        int setYear = calendar.get(Calendar.YEAR); 
        int us = calendar.get(Calendar.SECOND);

        //Calculate Julian Date
        double Hours = setHours + (setMinutes / 60.);
        double GGG = 1.0;
        if (setYear <= 1585.) 
           GGG = 0;
        double JD = -1. * Math.floor(7. * (Math.floor((setMonth + 9.) / 12.)
          + setYear) / 4.);
        double S = 1.;
        if ((setMonth - 9.)<0) 
            S=-1.;
        double A = Math.abs(setMonth - 9.);
        double J1 = Math.floor(setYear + S * Math.floor(A / 7.));
        J1 = -1 * Math.floor((Math.floor(J1 / 100.) + 1.) * 3./4.);
        JD = JD + Math.floor(275. * setMonth / 9.) + setDay + (GGG * J1);
        JD = JD + 1721027. + 2. * GGG + 367. * setYear - 0.5;
        JD = JD + (Hours / 24.);

        //stdlog_.info("visiplot.JulianDate.getJDC " + JD);
        return(JD);   
    }

    /**
     * getJDC converts the given date to a Julian Date 
     *
     * @param convDate Date date 
     * @return Julian Date double 
     */

    public static double getJDC(Date convDate){
        return getJDC(convDate, 0, 0);  
    }


    /**
     * getJDC Add the interval to the start time 
     * @param convDate Date start date of observation 
     * @param interval integer the time of each interval
     * @return next time interval, Date
     */
    public static Date getJDC(Date convDate, int interval){

        TimeZone tz = TimeZone.getTimeZone("UT");
        TimeZone.setDefault(tz);
        Calendar calendar = new GregorianCalendar();

        calendar.setTime(convDate); 
        calendar.add(Calendar.MINUTE, interval);
        Date newDate = calendar.getTime();
        //stdlog_.info("visiplot.JulianDate.getJDC NextInterval:  " + calendar.getTime());
        return(newDate);
    }
    
    public static void main(String[] args) {
        
    }
}
