package org.eso.ohs.phase2.visiplot;

import java.awt.Color;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;

import org.eso.ohs.phase2.actions.AreaColorEditorAction;
import org.eso.ohs.phase2.actions.LabelsColorEditorAction;
import org.eso.ohs.phase2.actions.PerimeterColorEditorAction;
/**
 * Onjects that contains the information about the colors used by a chart
 * @author gdonino
 *
 */
public class ChartColorItem {
	
	
	private JLabel name;
	
	//We don't really need a JButton. Probably we could manage it using Color class
	private JButton perimeter;
	private JButton area;
	private JButton labels;
	private boolean hasLine = false;
	private boolean hasLabels = false;
	private JComponent[] components;
/**
 * Constructor.
 * @param name The name of the chart
 * @param hasLine True if the chart uses perimeter and area colors. False if the chart uses only area color.
 * @param hasLabels True if the chart shows the labels.
 */
	public ChartColorItem(String name,  boolean hasLine, boolean hasLabels){
		this.hasLine = hasLine;
		this.hasLabels = hasLabels;
		
	
		
		components = new JComponent[4];
		
		this.name = new JLabel();
		perimeter = new JButton();
		area = new JButton();
		labels = new JButton();
		this.name.setText(name);
		perimeter.setEnabled(true);
		area.setEnabled(true);
		labels.setEnabled(true);
		
		String title = "Chart: " + this.name.getText() + " - Choose color for: ";
		
		
		area.addActionListener( new AreaColorEditorAction(this, new JFrame(), title + " plotting line"));
		area.setToolTipText(getToolTipText("area", name));
		
		if (!hasLine){
			perimeter.setEnabled(false);
			perimeter.setVisible(false);
		}else{
			perimeter.addActionListener(new PerimeterColorEditorAction(this, new JFrame(), title + " plotting area"));
			perimeter.setToolTipText(getToolTipText("line", name));
		}
		if (!hasLabels){
			labels.setEnabled(false);
			labels.setVisible(false);
		}else{
			labels.addActionListener(new LabelsColorEditorAction(this, new JFrame(), title + " chart labels"));
			labels.setToolTipText(getToolTipText("labels", name));
		}
		
		components[0] = this.name;
		components[1] = perimeter;
		components[2] = area;
		components[3] = labels;
	}

	public Color getAreaColor() {
		return area.getBackground();
	}

	public void setAreaColor(Color color) {
		this.area.setBackground(color);
	//	this.area.setForeground(color);
		this.area.setText(color.getRed() + ", " + color.getGreen() + ", " + color.getBlue());
	}

	public Color getLabelsColor() {
		return labels.getBackground();
	}

	public void setLabelsColor(Color color) {
		this.labels.setBackground(color);
		//this.labels.setForeground(color);
		this.labels.setText(color.getRed() + ", " + color.getGreen() + ", " + color.getBlue());
	}

	public String getName() {
		return name.getText();
	}

	public void setName(String name) {
		this.name.setText(name);
	}

	public Color getPerimeterColor() {
		return perimeter.getBackground();
	}

	public void setPerimeterColor(Color color) {
		this.perimeter.setBackground(color);
	//	this.perimeter.setForeground(color);
		this.perimeter.setText(color.getRed() + ", " + color.getGreen() + ", " + color.getBlue());
	}

	public boolean hasPerimeter() {
		return hasLine;
	}

	public boolean hasLabels() {
		return hasLabels;
	}

	public JComponent[] getComponents() {
		return components;
	}
	
	private String getToolTipText (String object, String label){
		return "Set the color for the " + object + " of the " + label + " chart";
	}
	
	public Object clone(){
		ChartColorItem result  = new ChartColorItem(name + "Clone", true, true);
		Color areaCol  = this.getAreaColor();
		Color perimeterCol = this.getPerimeterColor();
		Color labelsCol = this.getLabelsColor();
		
		Color newAreaCol = new Color(areaCol.getRed(), areaCol.getGreen(), areaCol.getBlue());
		Color newPerimeterCol = new Color(perimeterCol.getRed(), perimeterCol.getGreen(), perimeterCol.getBlue());
		Color newLabelsColor = new Color(labelsCol.getRed(), labelsCol.getGreen(), labelsCol.getBlue());
		
		result.setAreaColor(newAreaCol);
		result.setPerimeterColor(newPerimeterCol);
		result.setLabelsColor(newLabelsColor);
		
		return result;
	}
	
}
