package org.eso.ohs.phase2.visiplot;

/**
 * Dialog for loading, editing and saving visibility plot calculation and
 * display parameters.
 * 
 * @version $Revision: 1.18 $
 * @author Nick Kornweibel for ESO. Created on July 31, 2001, 10:14 AM
 * <br>ChangeLog: cmavros 26/09/01 Add Time Interval menu option 
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.EmptyBorder;

import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.jfree.chart.ui.ColorPalette;

public class VPlotOptionsDialog extends JDialog {
	
	private JPanel buttonPanel_;
	private JButton okButton_;
	private JButton cancelButton_;
	private JButton loadButton_;
	private JButton saveButton_;
	private JTabbedPane optionsEditorTabPane_;
	private PlotOptionsPanel plotOptionsPanel_;
	private HeaderOptionsPanel headerOptionsPanel_;
	private MiscOptionsPanel miscOptionsPanel_;
	private ColorOptionsPanel colorOptionsPanel_;
	private static JFileChooser fileChooser_;
	private boolean settingsAccepted_ = false;
	
	/**
	 * Creates new form VPlotOptionsDialog
	 */
	
	public VPlotOptionsDialog(Frame parent, boolean modal, boolean disablePlotOptions) {
		super(parent, modal);
		initComponents(disablePlotOptions);
		setTitle("Visibility Plot Options");
		pack();
	}
			
	/**
	 * Shows the dialog and initializes it with the VisiplotOptions
	 * @param visiplotOptions
	 * @return
	 */
	public boolean showDialog(VisiPlotOptions visiplotOptions) {
		
		
		
		loadSettings(visiplotOptions);
        show();
        if (getSettingsAccepted()) {
            saveSettings(visiplotOptions);
        }        
        
        return getSettingsAccepted();
	}
	
	/**
	 * This method is called from within the constructor to
	 * initialize the form.
	 */	
	private void initComponents(boolean disablePlotOptions) {
		buttonPanel_ = new JPanel();
		okButton_ = new JButton();
		cancelButton_ = new JButton();
		loadButton_ = new JButton();
		saveButton_ = new JButton();
		optionsEditorTabPane_ = new JTabbedPane();
		
		buttonPanel_.setLayout(new FlowLayout());
		okButton_.setText("OK");
		okButton_.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String entries = checkEntries();
				if (entries.length() > 0) {
					showMessage(entries,
					"Please correct settings before proceeding");
				} else {
					closeDialog(true);
				}
			}
		});
		buttonPanel_.add(okButton_);
		cancelButton_.setText("cancel");
		cancelButton_.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				closeDialog(false);
			}
		});
		buttonPanel_.add(cancelButton_);
		loadButton_.setText("load settings");
		loadButton_.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadSettings();
			}
		});
		buttonPanel_.add(loadButton_);
		saveButton_.setText("save settings");
		saveButton_.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveSettings();
			}
		});
		buttonPanel_.add(saveButton_);
		
		getContentPane().add(buttonPanel_, BorderLayout.SOUTH);
		getContentPane().add(optionsEditorTabPane_, BorderLayout.CENTER);
		
		// Create the panels
		plotOptionsPanel_ = new PlotOptionsPanel(disablePlotOptions);
		plotOptionsPanel_.setBorder(
				new EmptyBorder(new Insets(5, 5, 5, 5)));
		
		headerOptionsPanel_ = new HeaderOptionsPanel();
		headerOptionsPanel_.setBorder(
				new EmptyBorder(new Insets(5, 5, 5, 5)));
		
		miscOptionsPanel_ = new MiscOptionsPanel();
		miscOptionsPanel_.setBorder(
				new EmptyBorder(new Insets(5, 5, 5, 5)));
		
		colorOptionsPanel_ = new ColorOptionsPanel();
		colorOptionsPanel_.setBorder(
				new EmptyBorder(new Insets(5, 5, 5, 5)));
		
		buttonPanel_.setBorder(
				new EmptyBorder(new Insets(10, 10, 10, 10)));
		

		optionsEditorTabPane_.addTab("plot", plotOptionsPanel_);
		optionsEditorTabPane_.addTab("header", headerOptionsPanel_);
		optionsEditorTabPane_.addTab("advanced", miscOptionsPanel_);
		optionsEditorTabPane_.addTab("colors", colorOptionsPanel_);
	}
	
	/**
	 * Closes the dialog
	 */
	
	private void closeDialog(WindowEvent evt) {
		closeDialog(false);
	}
	
	public ChartColorItem[] getChartColorItems(){
		return	colorOptionsPanel_.getChartColorItems();
	}
	
	private void closeDialog(boolean acceptSettings) {
		settingsAccepted_ = acceptSettings;
		setVisible(false);
		dispose();
	}
	
	private void showMessage(String msg, String title) {
		JOptionPane.showMessageDialog(this,
				new Object[] { msg }, title, JOptionPane.ERROR_MESSAGE);
	}
	
	/**
	 * Inquire whether or not the OK button was pressed.
	 * @return true if ok was pressed.
	 */
	
	public boolean getSettingsAccepted() {
		return settingsAccepted_;
	}
	
	/**
	 * Prompt the user for a settings file and initialize the panel
	 * to the settings in that file.
	 */
	
	private void loadSettings() {
		JFileChooser fc = getFileChooser();
		fc.setDialogTitle("Select settings file to load");

	    	int returnVal = fc.showOpenDialog(this);
		File target = fc.getSelectedFile();
		if (target != null && (returnVal == JFileChooser.APPROVE_OPTION)) {
			try {
				FileReader fr = new FileReader(target);
				
				// keep the old start and end dates
				String startDate = plotOptionsPanel_.getStartTime();
				String endDate = plotOptionsPanel_.getEndTime();
				
				VisiPlotOptions opts = new VisiPlotOptions(fr);
				
				//	put back
				opts.setValue(VisiPlotOptions.startTimeKey,
						startDate);
				opts.setValue(VisiPlotOptions.endTimeKey,
						endDate);
				loadSettings(opts);
				
				fr.close();
				
				
			} catch (IOException e) {
				// show the error to the user
				JOptionPane.showMessageDialog(this,
						new Object[] { "Error reading settings: "+e.getMessage() },
						"Error reading settings",
						JOptionPane.ERROR_MESSAGE
				);
			}
		}
	}
	
	/**
	 * Prompt the user to select a file and save the settings to the
	 * selected file.
	 */
	
	private void saveSettings() {
		String entries = checkEntries();
		if (entries.length() > 0) {
			// report to user...fix errors before saving.
			JOptionPane.showMessageDialog(this,
					new Object[] { entries },
					"Please correct settings before saving",
					JOptionPane.ERROR_MESSAGE
			);
		} else {
			JFileChooser fc = getFileChooser();
			fc.setDialogTitle("Select file to save settings");
			fc.showSaveDialog(this);
			File target = fc.getSelectedFile();
			if (target != null) {
				boolean saved = false;
				try {
					FileWriter fw = new FileWriter(target);
					VisiPlotOptions opts = new VisiPlotOptions();
					saveSettings(opts);
					opts.writeTo(fw);
					fw.flush();
					fw.close();
					saved = true;
				} catch (IOException e) {
					// Show the error to the user.
					JOptionPane.showMessageDialog(this,
							new Object[] { "Error saving settings: "
							+e.getMessage() },
							"Error saving settings",
							JOptionPane.ERROR_MESSAGE
					);
				}
				if (saved) {
					// inform user that the settings are saved
					JOptionPane.showMessageDialog(this,
							new Object[] {"Options saved to file." },
							"Saving settings",
							JOptionPane.INFORMATION_MESSAGE
					);
				}
			}
		}
	}
	
	/**
	 * Update the settings object passed, with the values from the dialog
	 * @param the settings object to be updated.
	 */
	
	public void saveSettings(VisiPlotOptions settings) {
		// update the plotOptionsPanel
		settings.setValue(VisiPlotOptions.startTimeKey,
				plotOptionsPanel_.getStartTime());
		settings.setValue(VisiPlotOptions.endTimeKey,
				plotOptionsPanel_.getEndTime());
		settings.setValue(
				VisiPlotOptions.airMassConEnabledKey,
				plotOptionsPanel_.airMassConstraintSelected());
		settings.setValue(
				VisiPlotOptions.airMassConKey,
				plotOptionsPanel_.getAirMassConstraint());
		
		settings.setValue(
				VisiPlotOptions.obexecConKey,
				plotOptionsPanel_.execTimeConstraintSelected());
		
		settings.setValue(
				VisiPlotOptions.obtimeIntervalConKey,
				plotOptionsPanel_.timeIntervalConstraintSelected());
		
		settings.setValue(
				VisiPlotOptions.obSiderealTimeIntervalConKey,
				plotOptionsPanel_.siderealTimneIntervalConstraintSelected());            
		
		settings.setValue(
				VisiPlotOptions.fliConEnabledKey,
				plotOptionsPanel_.fliConstraintSelected());
		
		settings.setValue(
				VisiPlotOptions.fliConKey,
				plotOptionsPanel_.getFLIConstraint());
		
		settings.setValue(
				VisiPlotOptions.groupPlotKey,
				plotOptionsPanel_.groupIdentOBsConstraintSelected());
		
		settings.setValue(
				VisiPlotOptions.moonDistConEnabledKey,
				plotOptionsPanel_.moonDistConstraintSelected());
		settings.setValue(
				VisiPlotOptions.moonDistConKey,
				plotOptionsPanel_.getMoonDistConstraint());
		
		settings.setValue(
				VisiPlotOptions.sunConEnabledKey,
				plotOptionsPanel_.sunConstraintSelected());
		
		settings.setValue(
				VisiPlotOptions.sunConKey,
				plotOptionsPanel_.getSunConstraint());
		
		settings.setValue(
				VisiPlotOptions.zenithConKey,
				plotOptionsPanel_.zenithConstraintSelected());
		
		
		
		// update the headerOptionsPanel
		settings.setValue(
				VisiPlotOptions.reportDecKey,
				headerOptionsPanel_.getDec());		
		
		settings.setValue(
				VisiPlotOptions.reportExecTimeKey,
				headerOptionsPanel_.getExecTime());
		
		settings.setValue(
				VisiPlotOptions.reportStartTimeKey,
				headerOptionsPanel_.getStartTime());
		
		settings.setValue(
				VisiPlotOptions.reportStartTimeKey,
				headerOptionsPanel_.getStartTime());
		
		settings.setValue(
				VisiPlotOptions.reportFLIKey,
				headerOptionsPanel_.getFLI());
		
		settings.setValue(
				VisiPlotOptions.reportAirMassKey,
				headerOptionsPanel_.getAirMass());
		
		settings.setValue(
				VisiPlotOptions.reportMoonDistKey,
				headerOptionsPanel_.getMoonDistance());
		
		settings.setValue(
				VisiPlotOptions.reportOBIdKey,
				headerOptionsPanel_.getOBId());
		
		settings.setValue(
				VisiPlotOptions.reportProgIdKey,
				headerOptionsPanel_.getProgId());
		
		settings.setValue(
				VisiPlotOptions.reportRAKey,
				headerOptionsPanel_.getRA());
		
		settings.setValue(
				VisiPlotOptions.reportRankKey,
				headerOptionsPanel_.getRankClass());
		
		settings.setValue(
				VisiPlotOptions.reportSeeingKey,
				headerOptionsPanel_.getSeeing());
		
		settings.setValue(
				VisiPlotOptions.reportSkyTransKey,
				headerOptionsPanel_.getSkyTrans());
		
		settings.setValue(
				VisiPlotOptions.sortPlotKey,
				headerOptionsPanel_.getSortOrder());
		
		// update the misc options panel
		settings.setValue(VisiPlotOptions.dataQueryIntKey,
				miscOptionsPanel_.getPrecision());
		
		settings.setValue(VisiPlotOptions.printerName,
				miscOptionsPanel_.getPrinterName());
		settings.setValue(VisiPlotOptions.printerName,
				miscOptionsPanel_.getPrinterName());
		
		settings.setValue(VisiPlotOptions.airMassLineColor,
				getColorLineValue(colorOptionsPanel_.getAirMass()));
		settings.setValue(VisiPlotOptions.airMassAreaColor,
				getColorAreaValue(colorOptionsPanel_.getAirMass()));
		settings.setValue(VisiPlotOptions.airMassLabelsColor, 
				getColorLabelsValue(colorOptionsPanel_.getAirMass()));
		settings.setValue(VisiPlotOptions.fliLineColor,
				getColorLineValue(colorOptionsPanel_.getFli()));
		settings.setValue(VisiPlotOptions.fliAreaColor,
				getColorAreaValue(colorOptionsPanel_.getFli()));
		settings.setValue(VisiPlotOptions.fliLabelsColor,
				getColorLabelsValue(colorOptionsPanel_.getFli()));
		settings.setValue(VisiPlotOptions.moonDistLineColor,
				getColorLineValue(colorOptionsPanel_.getMoonDist()));
		settings.setValue(VisiPlotOptions.moonDistAreaColor,
				getColorAreaValue(colorOptionsPanel_.getMoonDist()));
		settings.setValue(VisiPlotOptions.moonDistLabelsColor,
				getColorLabelsValue(colorOptionsPanel_.getMoonDist()));
		settings.setValue(VisiPlotOptions.sunLineColor,
				getColorLineValue(colorOptionsPanel_.getSun()));
		settings.setValue(VisiPlotOptions.sunAreaColor,
				getColorAreaValue(colorOptionsPanel_.getSun()));
		settings.setValue(VisiPlotOptions.sunLabelsColor,
				getColorLabelsValue(colorOptionsPanel_.getSun()));
		settings.setValue(VisiPlotOptions.obexecLineColor,
				getColorLineValue(colorOptionsPanel_.getExecTime()));
		settings.setValue(VisiPlotOptions.obexecAreaColor,
				getColorAreaValue(colorOptionsPanel_.getExecTime()));
		settings.setValue(VisiPlotOptions.obexecLabelsColor,
				getColorLabelsValue(colorOptionsPanel_.getExecTime()));
		settings.setValue(VisiPlotOptions.obSiderealTimeIntervalLineColor,
				getColorLineValue(colorOptionsPanel_.getSidereal()));
		settings.setValue(VisiPlotOptions.obSiderealTimeIntervalAreaColor,
				getColorAreaValue(colorOptionsPanel_.getSidereal()));
		settings.setValue(VisiPlotOptions.obSiderealTimeIntervalLabelsColor,
				getColorLabelsValue(colorOptionsPanel_.getSidereal()));
		settings.setValue(VisiPlotOptions.timeLineColor,
				getColorLineValue(colorOptionsPanel_.getTimeInterval()));
		settings.setValue(VisiPlotOptions.timeAreaColor,
				getColorAreaValue(colorOptionsPanel_.getTimeInterval()));
		settings.setValue(VisiPlotOptions.timeLabelsColor,
				getColorLabelsValue(colorOptionsPanel_.getTimeInterval()));
		
		
		/*   
		 ChartColorItem timeInterval;
		 ChartColorItem sidereal;*/
		
	}
	
	private String getColorLineValue(ChartColorItem item) {
		String result;
		int red = item.getPerimeterColor().getRed();
		int green = item.getPerimeterColor().getGreen();
		int blue = item.getPerimeterColor().getBlue();
		result = red + "," + green + "," + blue;	
		return result;
	}
	private String getColorAreaValue(ChartColorItem item) {
		String result;
		int red = item.getAreaColor().getRed();
		int green = item.getAreaColor().getGreen();
		int blue = item.getAreaColor().getBlue();
		result = red + "," + green + "," + blue;	
		return result;
	}
	private String getColorLabelsValue(ChartColorItem item) {
		String result;
		int red = item.getLabelsColor().getRed();
		int green = item.getLabelsColor().getGreen();
		int blue = item.getLabelsColor().getBlue();
		result = red + "," + green + "," + blue;	
		return result;
	}
	
	/**
	 * Load the settings from the options and update the display widgets
	 * @param the settings/options object.
	 */
	
	public void loadSettings(VisiPlotOptions settings) {
		// update the plotOptionsPanel
		plotOptionsPanel_.setStartTime(
				settings.getValue(VisiPlotOptions.startTimeKey));
		plotOptionsPanel_.setEndTime(
				settings.getValue(VisiPlotOptions.endTimeKey));
		plotOptionsPanel_.setAirMassConstraint(
				settings.isEnabled(VisiPlotOptions.airMassConEnabledKey),
				settings.getValue(VisiPlotOptions.airMassConKey));
		plotOptionsPanel_.setExecTimeConstraint(
				settings.isEnabled(VisiPlotOptions.obexecConKey));
		plotOptionsPanel_.setTimeIntervalConstraint(
				settings.isEnabled(VisiPlotOptions.obtimeIntervalConKey));
		plotOptionsPanel_.setSiderealTimeIntervalConstraint(
				settings.isEnabled(VisiPlotOptions.obSiderealTimeIntervalConKey));                        
		plotOptionsPanel_.setFLIConstraint(
				settings.isEnabled(VisiPlotOptions.fliConEnabledKey),
				settings.getValue(VisiPlotOptions.fliConKey));
		plotOptionsPanel_.setGroupIdentOBsConstraint(
				settings.isEnabled(VisiPlotOptions.groupPlotKey));
		plotOptionsPanel_.setMoonDistConstraint(
				settings.isEnabled(VisiPlotOptions.moonDistConEnabledKey),
				settings.getValue(VisiPlotOptions.moonDistConKey));
		plotOptionsPanel_.setSunConstraint(
				settings.isEnabled(VisiPlotOptions.sunConEnabledKey),
				settings.getValue(VisiPlotOptions.sunConKey));
		
		plotOptionsPanel_.setZenithConstraint(
				settings.isEnabled(VisiPlotOptions.zenithConKey)			
				);
		
		// update the headerOptionsPanel
		headerOptionsPanel_.setDec(
				settings.isEnabled(VisiPlotOptions.reportDecKey));
		headerOptionsPanel_.setExecTime(
				settings.isEnabled(VisiPlotOptions.reportExecTimeKey));
		headerOptionsPanel_.setStartTime(
				settings.isEnabled(VisiPlotOptions.reportStartTimeKey));
		headerOptionsPanel_.setFLI(
				settings.isEnabled(VisiPlotOptions.reportFLIKey));
		headerOptionsPanel_.setAirMass(
				settings.isEnabled(VisiPlotOptions.reportAirMassKey));
		headerOptionsPanel_.setMoonDistance(
				settings.isEnabled(VisiPlotOptions.reportMoonDistKey));
		headerOptionsPanel_.setOBId(
				settings.isEnabled(VisiPlotOptions.reportOBIdKey));
		headerOptionsPanel_.setProgId(
				settings.isEnabled(VisiPlotOptions.reportProgIdKey));
		headerOptionsPanel_.setRA(
				settings.isEnabled(VisiPlotOptions.reportRAKey));
		headerOptionsPanel_.setRankClass(
				settings.isEnabled(VisiPlotOptions.reportRankKey));
		headerOptionsPanel_.setSeeing(
				settings.isEnabled(VisiPlotOptions.reportSeeingKey));
		headerOptionsPanel_.setSkyTrans(
				settings.isEnabled(VisiPlotOptions.reportSkyTransKey));
		headerOptionsPanel_.setSortOrder(
				settings.getValue(VisiPlotOptions.sortPlotKey));
		
		//Plotting colors.
		String lineColor = settings.getValue(VisiPlotOptions.airMassLineColor);
		String areaColor = settings.getValue(VisiPlotOptions.airMassAreaColor);
		String labelsColor = settings.getValue(VisiPlotOptions.airMassLabelsColor);
		
		colorOptionsPanel_.setAirMass(setChartColorItem(colorOptionsPanel_.getAirMass(), true, true,
				lineColor, areaColor, labelsColor));
		
		lineColor = settings.getValue(VisiPlotOptions.fliLineColor);
		areaColor = settings.getValue(VisiPlotOptions.fliAreaColor);
		labelsColor = settings.getValue(VisiPlotOptions.fliLabelsColor);
		
		colorOptionsPanel_.setFli(setChartColorItem(colorOptionsPanel_.getFli(), true, true,
				lineColor, areaColor, labelsColor));
		
		lineColor = settings.getValue(VisiPlotOptions.moonDistLineColor);
		areaColor = settings.getValue(VisiPlotOptions.moonDistAreaColor);
		labelsColor = settings.getValue(VisiPlotOptions.moonDistLabelsColor);
		
		colorOptionsPanel_.setMoonDist(setChartColorItem(colorOptionsPanel_.getMoonDist(), true, true,
				lineColor, areaColor, labelsColor));
		
		lineColor = settings.getValue(VisiPlotOptions.sunLineColor);
		areaColor = settings.getValue(VisiPlotOptions.sunAreaColor);
		labelsColor = settings.getValue(VisiPlotOptions.sunLabelsColor);
		
		colorOptionsPanel_.setSun(setChartColorItem(colorOptionsPanel_.getSun(), true, true,
				lineColor, areaColor, labelsColor));
		
		lineColor = settings.getValue(VisiPlotOptions.obexecLineColor);
		areaColor = settings.getValue(VisiPlotOptions.obexecAreaColor);
		labelsColor = settings.getValue(VisiPlotOptions.obexecLabelsColor);
		
		colorOptionsPanel_.setExecTime(setChartColorItem(colorOptionsPanel_.getExecTime(), true, true,
				lineColor, areaColor, labelsColor));
		
		lineColor = settings.getValue(VisiPlotOptions.obSiderealTimeIntervalLineColor);
		areaColor = settings.getValue(VisiPlotOptions.obSiderealTimeIntervalAreaColor);
		labelsColor = settings.getValue(VisiPlotOptions.obSiderealTimeIntervalLabelsColor);
		
		colorOptionsPanel_.setSidereal(setChartColorItem(colorOptionsPanel_.getSidereal(), true, true,
				lineColor, areaColor, labelsColor));
		
		lineColor = settings.getValue(VisiPlotOptions.timeLineColor);
		areaColor = settings.getValue(VisiPlotOptions.timeAreaColor);
		labelsColor = settings.getValue(VisiPlotOptions.timeLabelsColor);
		
		colorOptionsPanel_.setTimeInterval(setChartColorItem(colorOptionsPanel_.getTimeInterval(), true, true,
				lineColor, areaColor, labelsColor));
		
		
		// update the misc options panel
		int prec = VisiPlotOptions.defaultPrecision;
		try {
			prec = settings.getIntValue(VisiPlotOptions.dataQueryIntKey);
		} catch (NumberFormatException e) {
			prec = VisiPlotOptions.defaultPrecision;
		}
		miscOptionsPanel_.setPrecision(prec);
		miscOptionsPanel_.setPrinterName(
				settings.getValue(VisiPlotOptions.printerName));
	}
	
	private ChartColorItem setChartColorItem(ChartColorItem item, boolean hasArea, boolean hasLabels, String lineColor, String areaColor, String labelsColor) {
		ChartColorItem result = item;
		int[] lineRGB;
		int[] areaRGB;
		int[] labelsRGB;
		System.out.println();
		System.out.println("VPlotOptionsDialog.java: setting ChartColorItem " + item.getName());
		System.out.println("currentPerimeter: " + item.getPerimeterColor().toString() + " lineCol: " + lineColor);
		System.out.println("currentArea: " + item.getAreaColor().toString() + " areaCol: " + areaColor);
		System.out.println("currentLabels: " + item.getLabelsColor().toString() + " labelsCol: " + labelsColor);
		
		if (lineColor != null)
			lineRGB = getRGBColors(lineColor);
		else
			lineRGB = new int[]{80, 80, 80};
		
		if (areaColor != null)
			areaRGB = getRGBColors(areaColor);
		else
			areaRGB = new int[]{0, 0, 0};
		
		if (labelsColor != null)
			labelsRGB = getRGBColors(labelsColor);
		else
			labelsRGB = new int[]{0, 0, 50};
		
		Color lineCol = new Color (lineRGB[0], lineRGB[1], lineRGB[2]);
		Color areaCol = new Color (areaRGB[0], areaRGB[1], areaRGB[2]);
		Color labelsCol = new Color (labelsRGB[0], labelsRGB[1], labelsRGB[2]);
		
		result.setPerimeterColor(lineCol);
		result.setAreaColor(areaCol);
		result.setLabelsColor(labelsCol);
		System.out.println("currentPerimeter: " + item.getPerimeterColor().toString() + " lineCol: " + lineColor);
		System.out.println("currentArea: " + item.getAreaColor().toString() + " areaCol: " + areaColor);
		System.out.println("currentLabels: " + item.getLabelsColor().toString() + " labelsCol: " + labelsColor);
		
		
		return result;
	}
	
	private int[] getRGBColors(String colorString){
		int[] result = new int[3];
		
		StringTokenizer st = new StringTokenizer(colorString, ",");
		int i = 0;
		while (st.hasMoreElements()){
			result[i++] = Integer.parseInt(st.nextToken());
		}
		
		return result;
	}
	
	private static JFileChooser getFileChooser() {
		if (fileChooser_ == null) {
			fileChooser_ = new SwingFileChooser();
		}
		return fileChooser_;
	}
	
	/**
	 * Check all entries in the sub panels
	 * @return a string error code, empty string indicates no errors.
	 */
	
	public String checkEntries() {
		StringBuffer rc = new StringBuffer();
		String msg = plotOptionsPanel_.checkEntries();
		if (msg.length() > 0) {
			rc.append(msg+"\n");
		}
		msg = headerOptionsPanel_.checkEntries();
		if (msg.length() > 0) {
			rc.append(msg+"\n");
		}
		msg = miscOptionsPanel_.checkEntries();
		if (msg.length() > 0) {
			rc.append(msg+"\n");
		}
		return rc.toString();
	}
	
	
	
}
